	SUBROUTINE DM_OPEN ( filnam, wrtflg, shrflg, iflno, iftype, 
     +                       ifsrce, nrow, ncol, nprt, nfhdrs, iret )
C************************************************************************
C* DM_OPEN								*
C*									*
C* This subroutine opens a data management (DM) file.			*
C*									*
C* DM_OPEN  ( FILNAM, WRTFLG, SHRFLG, IFLNO, IFTYPE, IFSRCE, NROW, 	*
C*            NCOL,   NPRT,   NFHDRS, IRET )				*
C*									*
C* Input parameters:							*
C*	FILNAM		CHAR*		File name			*
C*	WRTFLG		LOGICAL		Write access flag		*
C*	SHRFLG		LOGICAL		Shared file flag		*
C*									*
C* Output parameters:							*
C*	IFLNO		INTEGER		File number			*
C*	IFTYPE		INTEGER		File type			*
C*	IFSRCE		INTEGER		File source			*
C*	NROW		INTEGER		Number of rows			*
C*	NCOL		INTEGER		Number of columns		*
C*	NPRT		INTEGER		Number of parts			*
C*	NFHDRS		INTEGER		Number of file headers		*
C*	IRET		INTEGER		Return code			*
C*					  0 = normal return		*
C*					 -2 = file cannot be opened	*
C*					 -3 = too many files open	*
C*					 -6 = write error		*
C*					 -7 = read error		*
C*					-23 = wrong machine type	*
C*					-32 = invalid machine for write	*
C**									*
C* Log:									*
C* M. desJardins/GSFC	 5/87						                    *
C* M. desJardins/GSFC	 7/87	Added record length to OPEN		        *
C* M. desJardins/GSFC	 5/90	Check for files from other machines	    *
C* M. desJardins/NMC	 4/91	Write to files from other machines	    *
C* M. desJardins/NMC	 5/91	Flags for machine types			        *
C* S. Jacobs/EAI	 8/92	Added check for ULTRIX machine		        *
C* K. Brill/NMC		 5/93	Added check for HP machine		            *
C* S. Jacobs/EAI	10/93	Added check for ALPHA machine		        *
C* S. Jacobs/NCEP	 2/01	Made MTLNUX a separate machine type	        *
C* T. Lee/SAIC		 2/05	Handle error message			            *
C* m. gamazaychikov/CWS 04/11   Add code for A2DB connectivity          *
C* S. Jacobs/NCEP	 8/13	Added check for non-gempak files	        *
C* S. Gilbert/NCEP   9/15   Passed filnam to DA_INFO                    *
C************************************************************************
	INCLUDE		'GEMPRM.PRM'
	INCLUDE		'GMBDTA.CMN'
	INCLUDE		'dmcmn.cmn'
C
	CHARACTER*(*)	filnam
	LOGICAL		wrtflg, shrflg
C------------------------------------------------------------------------
C*	Get a file number for this file.  Return if there are no more
C*	file numbers.
C
        write(*,*) " DM_OPEN: filename = ", filnam, "|"
	CALL DM_GETF  ( iflno, iret )
	IF  ( iret .ne. 0 ) RETURN
C
C*	Check for non-standard "files".
C
	ipos = INDEX ( filnam, 'AWIPSDB' )
	IF  ( ipos .eq. 0 )  THEN
	    stdgem ( iflno ) = .true.
	  ELSE
	    stdgem ( iflno ) = .false.
	END IF
C
C*	Open the file.  For open error, print error message and return.
C
        write(*,*) "DM_OPEN: iflno = ", iflno

	IF  ( stdgem(iflno) )  THEN
	    IF  ( wrtflg .and. shrflg )  THEN
		CALL FL_DSOP  ( filnam, MBLKSZ, lun, iret )
		kshare ( iflno ) = .true.
	      ELSE
                write(*,*) "DM_OPEN: FL_DOPN, filenam = ", filnam, "|"
		CALL FL_DOPN  ( filnam, MBLKSZ, wrtflg, lun, iret )
		kshare ( iflno ) = .false.
	    END IF
	  ELSE
	    IF  ( wrtflg )  THEN
		iret = -13
		RETURN
	      ELSE
		CALL DA_OPEN ( filnam, iflno, lun, iret )
		kshare ( iflno ) = .false.
	    END IF
	END IF
C
C*	Check return code from the appropriate open function.
C
	IF  ( iret .eq. 0 )  THEN
	    lundm  (iflno) = lun
	    wflag  (iflno) = wrtflg
	  ELSE
	    CALL ST_LSTR ( filnam, nf, ier )
	    CALL ER_WMSG ( 'FL', iret, filnam (:nf), ier )
	    iret = -2
	    RETURN
	END IF
C
C*	Load the DM common blocks
C
	IF  ( .not. stdgem(iflno) )  THEN
C
C*	    Get the information from the non-standard data source
C
	    CALL DA_INFO ( filnam, iflno, iret )
	ELSE
C
C*	    Read the file label into common.
C
	    CALL DM_RLBL ( iflno, iret )
	    IF  ( iret .ne. 0 ) GOTO 900
C
C*	    Check that this is a valid machine.
C
	    IF  ( ( kmachn ( iflno ) .ne. MTMACH ) .and.
     +	      	  ( kmachn ( iflno ) .ne. MTVAX  ) .and.
     +	      	  ( kmachn ( iflno ) .ne. MTSUN  ) .and.
     +	      	  ( kmachn ( iflno ) .ne. MTAPOL ) .and.
     +	      	  ( kmachn ( iflno ) .ne. MTIRIS ) .and.
     +	      	  ( kmachn ( iflno ) .ne. MTIGPH ) .and.
     +	      	  ( kmachn ( iflno ) .ne. MTULTX ) .and.
     +	      	  ( kmachn ( iflno ) .ne. MTIBM  ) .and.
     +        	  ( kmachn ( iflno ) .ne. MTHP   ) .and.
     +        	  ( kmachn ( iflno ) .ne. MTLNUX ) .and.
     +	      	  ( kmachn ( iflno ) .ne. MTALPH ) )  THEN
		IF  ( wrtflg )  THEN
		    iret = -32
		    CALL ST_LSTR ( filnam, nf, ier )
		    CALL ER_WMSG  ( 'DM', iret, filnam (:nf), ier )
		    RETURN
		END IF
	    END IF
C
C*	    Read the data management record.
C
	    CALL DM_RDMG ( iflno, iret )
C
C*	    Read all keys from the file into common.
C
	    CALL DM_RKEY ( iflno, iret )
	    IF  ( iret .ne. 0 ) GOTO 900
C
C*	    Read headers from file.
C
	    CALL DM_RHDA ( iflno, iret )
	    IF  ( iret .ne. 0 ) GOTO 900
C
C*	    Read part information.
C
	    CALL DM_RPRT ( iflno, iret )
	    IF  ( iret .ne. 0 ) GOTO 900
C
C*	    Read file information.
C
	    CALL DM_RFIL ( iflno, iret )
	END IF
	IF  ( iret .ne. 0 ) GOTO 900
C
C*	Set search information in common.
C
	srcflg ( iflno ) = .false.
	nsrch  ( iflno ) =  0
	ksrow  ( iflno ) =  0
	kscol  ( iflno ) = 0
C
C*	Return number of rows, columns and parts.
C
	nrow   = krow ( iflno )
	ncol   = kcol ( iflno )
	nprt   = kprt ( iflno )
	nfhdrs = kfhdrs ( iflno )
C
C*	Return file type and source.
C
	iftype = kftype ( iflno )
	ifsrce = kfsrce ( iflno )
C
	GOTO 999
C
C*	Close file if an error was encountered.
C
900	CONTINUE
	CALL DM_CLOS ( iflno, ier )
C*
999	CONTINUE
	RETURN
	END
