// NeuroSolutions v4.0
// Copyright (c) 1994, NeuroDimension, Inc. All rights reserved.
//#include "C:\NeuroSolutions\stdafx.h"
#ifndef NS_GUI
//#error ERROR: Comment out include of stdafx.h
#include <malloc.h>
#endif

#include "NSLearn.h"

// Author: Wesley Curtis Lefebvre
#ifdef NS_GUI
#define new DEBUG_NEW
#endif
/*
static UINT             NSMallocBytes = 0;

  void* NSMalloc(size_t aSize)
  {       void*           returnValue;

	NSMallocBytes += aSize;
	#undef malloc
	returnValue = malloc(aSize);
	#define malloc NSMalloc
	return returnValue;
	}

	  void* NSCalloc(size_t aNumber, size_t aSize)
	  {       void*           returnValue;

        NSMallocBytes += aSize * aNumber;
        #undef calloc
        returnValue = calloc(aNumber, aSize);
        #define calloc NSCalloc
        return returnValue;
		}

		  void NSFreeMemory(void* memBlock)
		  {       size_t          blockSize;

			#undef free
			if (memBlock) {
			blockSize = _msize(memBlock);
			NSMallocBytes -= blockSize;
			free(memBlock);
			}
			#define free NSFreeMemory
			}

			  UINT NSBytesAllocated()
			  {
			  return NSMallocBytes;
			  }
*/

unsigned short int getWeightFileVersion(FILE *file) {
	unsigned short int returnValue = 100;
	fseek(file,0,NSSEEK_SET);
	while (!feof(file)) {
		while((getc(file)) != '#')
			if(feof(file))
				break;
			if (!feof(file)) {
				char nameString[128];
				fscanf(file,"%s",&nameString);
				if(!strcmp("NSWeightFileVersion",nameString)) {
					fscanf(file,"%hu",&returnValue);
					return returnValue;
				}
			}
	}
	return returnValue;
}

FILE *seekComponent(FILE *file, const char *className, const char *name) {
	fseek(file,0,NSSEEK_SET);
	while (!feof(file)) {
		while((getc(file)) != '#')
			if(feof(file))
				break;
			if (!feof(file)) {
				char nameString[128];
				fscanf(file,"%s",&nameString);
				if(!strcmp(name,nameString)) {
					char classString[128];
					fscanf(file,"%s",&classString);
					if(!strcmp(className,classString))
						return file;
					else {
						fprintf(stderr, "The %s named %s does not match the %s found in the weight file.",className, name, classString);
						exit(1);
					}
				}
			}
	}
	fprintf(stderr, "The %s named %s was not found in the weight file.",className, name);
	exit(1);
	return NULL;
}

FILE *seekComponentReturnNull(FILE *file, const char *className, const char *name) {
	fseek(file,0,NSSEEK_SET);
	while (!feof(file)) {
		while((getc(file)) != '#')
			if(feof(file))
				break;
			if (!feof(file)) {
				char nameString[128];
				fscanf(file,"%s",&nameString);
				if(!strcmp(name,nameString)) {
					char classString[128];
					fscanf(file,"%s",&classString);
					if(!strcmp(className,classString))
						return file;
					else {
						return NULL;
					}
				}
			}
	}
	return NULL;
}

FILE *putComponent(FILE *file, const char *className, const char *name) {
	fprintf(file,"#%s %s\n",name,className);
	return file;
}

void writeWeightFileVersion(FILE *file) {
	fprintf(file,"#NSWeightFileVersion %d;\n\n",WEIGHTFILEVERSION);
}

// Author: Wesley Curtis Lefebvre


void checkParameterIndices(int *row,int *col)
{
	if (*row > DLL_PARAM_ROWS-1)
		*row = DLL_PARAM_ROWS-1;
	if (*col > DLL_PARAM_COLS-1)
		*col = DLL_PARAM_COLS-1;
	if (*row < 0)
		*row = 0;
	if (*col < 0)
		*col = 0;
}

DLLData *allocDLLInstance(DLLData *oldInstance)
{
	DLLData *data = (DLLData*)malloc(sizeof(DLLData));
	data->weights = NULL;
	data->parameters = NULL;
	data->userData = NULL;
	if (oldInstance) {
		if (oldInstance->weights) {
			data->weights = oldInstance->weights;
			oldInstance->weights = NULL;
		}
		if (oldInstance->parameters) {
			data->parameters = oldInstance->parameters;
			oldInstance->parameters = NULL;
		}
	}
	return data;
}

void freeDLLInstance(DLLData *instance)
{
	if (instance) {
		if (instance->weights) {
			free(instance->weights);
			instance->weights = NULL;
		}
		if (instance->parameters) {
			free(instance->parameters);
			instance->parameters = NULL;
		}
		free(instance);
	}
}

DLLData *setWeights(DLLData *instance, int numberOfWeights)
{
	if (!instance->weights)
		instance->weights = (DLLWeights*)malloc(sizeof(DLLWeights));
	instance->weights->length = numberOfWeights;
	return instance;
}

DLLData *allocParameters(DLLData *instance)
{
	int i,j;

	if (!instance->parameters) {
		DLLParameters *parameters = (DLLParameters*)malloc(sizeof(DLLParameters));
		for (i=0; i<DLL_PARAM_ROWS; i++)
			for (j=0; j<DLL_PARAM_COLS; j++) {
				strcpy(parameters->parameters[i][j], "");
				strcpy(parameters->parameterNames[i][j], "");
			}
			instance->parameters = parameters;
	}
	return instance;
}

NSFloat *getWeights(DLLData *instance)
{
	return instance->weights->data;
}

void setUserData(DLLData *instance, void *userData)
{
	instance->userData = userData;
}

void *getUserData(DLLData *instance)
{
	return instance? instance->userData: NULL;
}

void setParameterName(DLLData *instance, int row, int col, char *name, BOOL realloc)
{
	allocParameters(instance);
	checkParameterIndices(&row,&col);
	strcpy(instance->parameters->parameterNames[row][col], name);
	instance->parameters->parameterReallocs[row][col] = realloc;
}

int getIntParameter(DLLData *instance, int row, int col)
{
	int intValue=0;
	checkParameterIndices(&row,&col);
	if (sscanf(instance->parameters->parameters[row][col], "%d", &intValue) == EOF)
		return 0;
	return intValue;
}

NSFloat getFloatParameter(DLLData *instance, int row, int col)
{
	NSFloat floatValue=0.0f;
	checkParameterIndices(&row,&col);
	if (sscanf(instance->parameters->parameters[row][col], "%f", &floatValue) == EOF)
		return 0.0f;
	return floatValue;
}

char *getStringParameter(DLLData *instance, int row, int col)
{
	checkParameterIndices(&row,&col);
	return instance->parameters->parameters[row][col];
}

BOOL getBoolParameter(DLLData *instance, int row, int col)
{
	char tempString[128];
	checkParameterIndices(&row,&col);
	strcpy(tempString,instance->parameters->parameters[row][col]);
	return ((tolower(tempString[0]) == 't') && (tolower(tempString[1]) == 'r') && (tolower(tempString[2]) == 'u') && (tolower(tempString[3]) == 'e') && (tolower(tempString[4]) == '\0'));
}

void setIntParameter(DLLData *instance, int row, int col, int intValue, BOOL force)
{
	allocParameters(instance);
	checkParameterIndices(&row,&col);
	if (force || !strcmp(instance->parameters->parameters[row][col],""))
		sprintf(instance->parameters->parameters[row][col], "%d", intValue);
}

void setFloatParameter(DLLData *instance, int row, int col, NSFloat floatValue, BOOL force)
{
	allocParameters(instance);
	checkParameterIndices(&row,&col);
	if (force || !strcmp(instance->parameters->parameters[row][col],""))
		sprintf(instance->parameters->parameters[row][col], "%f", floatValue);
}

void setStringParameter(DLLData *instance, int row, int col, char *stringValue, BOOL force)
{
	allocParameters(instance);
	checkParameterIndices(&row,&col);
	if (force || !strcmp(instance->parameters->parameters[row][col],""))
		sprintf(instance->parameters->parameters[row][col], "%s", stringValue);
}

void setBoolParameter(DLLData *instance, int row, int col, BOOL boolValue, BOOL force)
{
	allocParameters(instance);
	checkParameterIndices(&row,&col);
	if (force || !strcmp(instance->parameters->parameters[row][col],""))
		sprintf(instance->parameters->parameters[row][col], "%s", boolValue? "TRUE": "FALSE");
}

// Author: Wesley Curtis Lefebvre

BOOL	SpatialAccess::m_bRefreshAccess = FALSE;

SpatialAccess::SpatialAccess()
{
	rowCount = 0;
	colCount = 0;
	spatialCount = 0;
	data = NULL;
	stackedAccess = nil;
	m_bAccessTesting = NO;
	m_bTesting = NO;
}

void SpatialAccess::attachTo(SpatialAccess *otherData)
{
	rowCount = otherData->rowCount;
	colCount = otherData->colCount;
	spatialCount = otherData->spatialCount;
	data = otherData->data;
}

void SpatialAccess::setStackedAccess(SpatialAccess *anId)
{
	if (stackedAccess = anId)
		stackedAccess->setData(data, rowCount, colCount);
}

void SpatialAccess::reportSpatialAccess()
{
	if (!(m_bTesting ^ m_bAccessTesting))
		reportSpatialData();
	if (stackedAccess)
		stackedAccess->reportSpatialAccess();
}

void SpatialAccess::reportSpatialAccess(NSFloat *aFloat)
{
	data = aFloat;
	if (!(m_bTesting ^ m_bAccessTesting))
		reportSpatialData();
	if (stackedAccess)
		stackedAccess->reportSpatialAccess(aFloat);
}

void SpatialAccess::reportSpatialData()
{
}

void SpatialAccess::setData(NSFloat *aFloat, int rowInt, int colInt)
{
	rowCount = rowInt;
	colCount = colInt;
	spatialCount = rowCount * colCount;
	data = aFloat;
	setStackedAccess(stackedAccess);
}

SpatialAccess::~SpatialAccess()
{
}

// Author: Wesley Curtis Lefebvre



SpatialData::SpatialData()
{
	randomRange = 0.5f;
	allocatedData = NULL;
	m_bAccessEnabled = TRUE;
}

void SpatialData::assignScalar(NSFloat aFloat) {
	register NSFloat *pos=data, *endPos=data+spatialCount;
	while (pos<endPos)
		*pos++ = aFloat;
	reportSpatialAccess();
}

void SpatialData::copyDataFrom(NSFloat *aFloat) {
	register NSFloat *pos=data, *endPos=data+spatialCount;
	while (pos<endPos)
		*pos++ = (NSFloat)*aFloat++;
	reportSpatialAccess();
}

void SpatialData::copyDataFromDouble(NSFloat *aFloat) {
	register NSFloat *pos=data, *endPos=data+spatialCount;
	while (pos<endPos)
		*pos++ = *aFloat++;
	reportSpatialAccess();
}

void SpatialData::copyDataTo(NSFloat *aFloat) {
	register NSFloat *pos=data, *endPos=data+spatialCount;
	while (pos<endPos)
		*aFloat++ = (NSFloat)*pos++;
	reportSpatialAccess();
}

void SpatialData::deleteData()
{
	if (allocatedData)
		free((char*)allocatedData);
	data = allocatedData = NULL;
	spatialCount = rowCount = colCount = 0;
	reportSpatialAccess();
}

void SpatialData::oversizeSpatialDimension(int anInt) {
    if ((anInt >= rowCount*colCount) && anInt) {
		spatialCount = anInt;
		if (allocatedData)
			free((char*)allocatedData);
		data = allocatedData = (NSFloat*)calloc(spatialCount, sizeof(NSFloat));
	}
}

void SpatialData::setSpatialDimension(int rowInt, int colInt) {
	rowCount = rowInt;
	colCount = colInt;
	oversizeSpatialDimension(rowCount*colCount);
}

void SpatialData::randomize() {
	register NSFloat *pos=data, *endPos=data+spatialCount;
	while (pos<endPos)
		*pos++ = 2*randomRange/RAND_MAX*rand()-randomRange;
	reportSpatialAccess();
}

void SpatialData::zero() {
	register NSFloat *pos=data, *endPos=data+spatialCount;
	while (pos<endPos)
		*pos++ = (NSFloat)0.0;
	//      reportSpatialAccess();
}

SpatialData::~SpatialData() {
	deleteData();
}

// InputSpatialData.cpp

InputSpatialData::InputSpatialData()
{
	m_fOffset = NULL;
	m_fDenormalizedData = NULL;
	m_fAmplitude = NULL;
	m_bPerformNormalization = FALSE;
	m_iAllocatedChannels = 0;
}

InputSpatialData::~InputSpatialData()
{
	if (m_iAllocatedChannels) {
		free(m_fAmplitude);
		free(m_fOffset);
		free(m_fDenormalizedData);
	}
}

void InputSpatialData::denormalizeData()
{
	if (m_iAllocatedChannels == spatialCount) {
		register NSFloat *pos1 = data;
		register NSFloat *endPos1 = data+spatialCount;
		register NSFloat *pos2 = m_fDenormalizedData;
		int j=0;
		while (pos1<endPos1) {
			*pos2 = (*pos1 - *(m_fOffset+j)) / *(m_fAmplitude+j);
			j++;
			pos1++;
			pos2++;
		}
	}
}

void InputSpatialData::loadWeights(FILE *file, unsigned short int version)
{
	//printf("InputSpatialData::loadWeights\n");
    int peCount;
    
    fscanf(file,"%d",&peCount);
	//printf("InputSpatialData::loadWeights peCount = %i\n",peCount);
    NSFloat dummy1, dummy2;
    for (int i=0; i < peCount; i++)
		fscanf(file,"%f %f", (i < m_iAllocatedChannels) ? (m_fAmplitude+i) : &dummy1, (i < m_iAllocatedChannels) ? (m_fOffset+i) : &dummy2);
	//printf("InputSpatialData::loadWeights version = %i\n",version);
	//if (version > 213) {
	if (version > 225) {
		int dummyInt, totalChannels, nn;
		NSFloat dummyFloat1, dummyFloat2;
		nn = fscanf(file,"%d",&dummyInt);
		//printf("InputSpatialData::loadWeights dummyInt = %d, nn = %i\n",dummyInt,nn);
        int i;
		//for (i=0; i < dummyInt; i++)
		//	fscanf(file,"%f %f", &dummyFloat1, &dummyFloat2);
		for (i=0; i < dummyInt; i++) {
			fscanf(file,"%f %f", &dummyFloat1, &dummyFloat2);
		//printf("InputSpatialData::loadWeights %i dummyFloat1 = %f\n",i,dummyFloat1);
		//printf("InputSpatialData::loadWeights %i dummyFloat2 = %f\n",i,dummyFloat2);
		}
		
		if (dummyInt) {
			nn = fscanf(file,"%d",&dummyInt);
		//printf("dummyInt = %d\n", dummyInt);
			if (dummyInt) {
				fscanf(file,"%d",&dummyInt);
				fscanf(file,"%d",&totalChannels);
		//printf("dummyInt = %d totalChannels = %d\n", dummyInt, totalChannels);
				for (i=0; i < totalChannels; i++)
					fscanf(file,"%d", &dummyFloat1);
				for (i=0; i < totalChannels; i++)
					fscanf(file,"%d", &dummyFloat1);
			}
		}
	}
	//printf("Finished loadweights\n");
}

void InputSpatialData::normalizeData()
{
	if (m_iAllocatedChannels == spatialCount) {
		register NSFloat *pos = data;
		register NSFloat *endPos = data+spatialCount;
		int j=0;
		while (pos<endPos) {
			*pos = *(m_fAmplitude+j) * *pos + *(m_fOffset+j);
			j++;
			pos++;
		}
	}
}

void InputSpatialData::saveWeights(FILE *file)
{
	if (m_iAllocatedChannels) {
        fprintf(file,"%d\n",m_iAllocatedChannels);
        for (int i=0; i < m_iAllocatedChannels; i++) {
			fprintf(file,"%21.15e %21.15e",*(m_fAmplitude+i),*(m_fOffset+i));
			fprintf(file,"\n");
		}
        fprintf(file,"0\n");	//NMultiChannelStream::m_nGeneticSwitchedOn
        fprintf(file,"\n");
	}
}

void InputSpatialData::setSpatialDimension(int rowInt, int colInt) {
	SpatialData::setSpatialDimension(rowInt, colInt);
	if (rowInt*colInt) {
		if (m_iAllocatedChannels) {
			free(m_fAmplitude);
			free(m_fOffset);
			free(m_fDenormalizedData);
		}
		m_iAllocatedChannels = rowInt*colInt;
		m_fAmplitude = (NSFloat *)malloc(m_iAllocatedChannels * sizeof(NSFloat));
		m_fOffset = (NSFloat *)malloc(m_iAllocatedChannels * sizeof(NSFloat));
		m_fDenormalizedData = (NSFloat *)malloc(m_iAllocatedChannels * sizeof(NSFloat));
	}
}

// File.h


File::File()
{
	m_pFileStream = NULL;
	strcpy(m_sFilePath,"");
	m_iMode = BINARY;
	m_readWrite = READ;
	m_AdvanceStream = TRUE;
	m_StreamConstant = FALSE;
	m_FixStream = FALSE;
	m_iDitherInput = -1;
	m_fDither = 0.1f;
}

File::File(char *aChar, RW readWrite, MODE mode)
{
	m_pFileStream = NULL;
	strcpy(m_sFilePath, aChar);
	m_iMode = mode;
	m_readWrite = readWrite;
	m_AdvanceStream = TRUE;
	m_StreamConstant = FALSE;
	m_FixStream = FALSE;
	m_iDitherInput = -1;
	m_fDither = 0.1f;
	openFile();
}

File::~File()
{
	closeFile();
}

void File::advanceStream()
{
	if (m_StreamConstant)
		fseek(m_pFileStream, spatialCount*sizeof(NSFloat), NSSEEK_CUR);
	else
		m_AdvanceStream = TRUE;
}

void File::fixStream(BOOL aFlag)
{
	if (m_FixStream = aFlag) {
		m_AdvanceStream = TRUE;
		m_StreamConstant = TRUE;
	} else {
		m_AdvanceStream = TRUE;
		m_StreamConstant = FALSE;
	}
}

BOOL File::closeFile()
{
	int returnValue;
	if((m_iMode != STANDARD) && m_pFileStream)
		returnValue = fclose(m_pFileStream);
	m_pFileStream = NULL;
	return (BOOL)!returnValue;
}

void File::ditherInput(int anInt, NSFloat aFloat)
{
	m_iDitherInput = anInt;
	m_fDither = aFloat;
}

BOOL File::openFile()
{
	if(m_pFileStream)
		closeFile();
	switch(m_iMode){
	case ASCII:
		if(!strlen(m_sFilePath))
			return FALSE;
		if(m_readWrite)
			m_pFileStream = fopen(m_sFilePath,"r");
		else
			m_pFileStream = fopen(m_sFilePath,"w");
		break;
	case BINARY:
		if(!strlen(m_sFilePath))
			return FALSE;
		if(m_readWrite)
			m_pFileStream = fopen(m_sFilePath,"rb");
		else
			m_pFileStream = fopen(m_sFilePath,"wb");
		break;
	case STANDARD:
		if(m_readWrite)
			m_pFileStream = stdin;
		else
			m_pFileStream = stdout;
		break;
	}
	if(!m_pFileStream) {
		fprintf(stderr, "Error: Unable to open file %s.\n", m_sFilePath);
		exit(1);
	}
	return TRUE;
}

void File::reportSpatialData()
{
	if (m_pFileStream && m_AdvanceStream && m_bAccessEnabled){
		register NSFloat *pos, *endPos, *currentData;
		currentData = data;
		if (!m_readWrite && m_bPerformNormalization) {
			denormalizeData();
			currentData = m_fDenormalizedData;
		}
		switch(m_iMode) {
		case ASCII:
		case STANDARD:
			pos = currentData;
			endPos=pos+spatialCount;
			if (m_readWrite) {
				while (pos<endPos)
					if (fscanf(m_pFileStream,"%f", pos++) == EOF) {
						fseek(m_pFileStream,0,NSSEEK_SET);
						pos--;
						fscanf(m_pFileStream,"%f", pos++);
					}
			}
			else {
				while (pos<endPos)
					fprintf(m_pFileStream,"%21.15e ", *pos++);
				fprintf(m_pFileStream,"\n");
			}
			break;
		case BINARY:
			if(m_readWrite) {
				fread((char *)currentData,sizeof(NSFloat), spatialCount, m_pFileStream);
#ifdef SWAP

				union lconv {
					NSFloat ul;
					unsigned char uc[4];
				} *inp, outv;
				pos = currentData;
				endPos = currentData+spatialCount;
				while (pos<endPos) {
					NSFloat inv = *pos;
					inp = (union lconv *)&inv;
					outv.uc[0] = inp->uc[3];
					outv.uc[1] = inp->uc[2];
					outv.uc[2] = inp->uc[1];
					outv.uc[3] = inp->uc[0];
					*pos++ = outv.ul;
				}
#endif
			} else
				fwrite((char *)currentData,sizeof(NSFloat), spatialCount, m_pFileStream);
			break;
		}
		if (feof(m_pFileStream)) {
			fseek(m_pFileStream,0,NSSEEK_SET);
			if (m_iMode && !feof(m_pFileStream)) {
				reportSpatialData();
				return;
			}
		}
		if (m_iDitherInput >= 0)
			*(currentData+m_iDitherInput) += m_fDither;
		if (m_readWrite && m_bPerformNormalization)
			normalizeData();
		if (m_StreamConstant)
			fseek(m_pFileStream,-(int)(spatialCount*sizeof(NSFloat)), NSSEEK_CUR);
		else
			if (m_FixStream)
				m_AdvanceStream = FALSE;
	}
}

void File::setMode(RW readWrite, MODE mode)
{
	m_readWrite = readWrite;
	m_iMode = mode;
	openFile();
}

void File::setFilePath(char *aPath)
{
	strcpy(m_sFilePath,aPath);
}

//      Auther: Wesley Curtis Lefebvre


DLLInput::DLLInput()
{
	dllStruct = NULL;
	m_pCallingObject = NULL;
}

void DLLInput::reportSpatialData()
{
	if (dllStruct && dllStruct->function) {
		if (m_pCallingObject)
			(*(DLLInput_MAP_CSW)dllStruct->function)(dllStruct->instance, (NSFloat *)data, rows(), cols(), m_pCallingObject);
		else
			(*(DLLInput_MAP)dllStruct->function)(dllStruct->instance, (NSFloat *)data, rows(), cols());
	}
	if (m_bPerformNormalization)
		normalizeData();
	InputSpatialData::reportSpatialData();
}

void DLLInput::constructDLL(void) {
	if (dllStruct->constructor) {
		if (m_pCallingObject)
			dllStruct->instance = (*(DLLInput_Constructor_CSW)dllStruct->constructor)(dllStruct->instance, rows(), cols(), m_pCallingObject);
		else
			dllStruct->instance = (*(DLLInput_Constructor)dllStruct->constructor)(dllStruct->instance, rows(), cols());
	}
}

void DLLInput::destroyDLL(DLLData *instance) {
	if (dllStruct->destructor) {
		if (m_pCallingObject)
			(*(DLL_Destructor_CSW)dllStruct->destructor)(instance, m_pCallingObject);
		else
			(*(DLL_Destructor)dllStruct->destructor)(instance);
	}
}

void DLLInput::initializeDLL(void) {
	if (dllStruct) {
		DLLData *oldInstance = dllStruct->instance;
		constructDLL();
		destroyDLL(oldInstance);
	}
}

void DLLInput::setData(NSFloat *aFloat, int rowInt, int colInt) {
	InputSpatialData::setData(aFloat, rowInt, colInt);
	initializeDLL();
}

void DLLInput::setDLL(void *function, void *constructor, void *destructor, void *instance, void *callingObject)
{
	dllStruct = new DLLStruct;
	dllStruct->function = function;
	dllStruct->constructor = constructor;
	dllStruct->destructor = destructor;
	dllStruct->instance = (DLLData*)instance;
	m_pCallingObject = callingObject;
	initializeDLL();
}

void DLLInput::setSpatialDimension(int rowInt, int colInt) {
	InputSpatialData::setSpatialDimension(rowInt, colInt);
	initializeDLL();
}


//      Auther: Wesley Curtis Lefebvre


DLLPreprocessor::DLLPreprocessor()
{
	m_bInputDataLocal = FALSE;
}

DLLPreprocessor::~DLLPreprocessor()
{
	if (m_pData && m_bInputDataLocal)
		delete m_pData;
}

BOOL DLLPreprocessor::isPreprocessor()
{
	return TRUE;
}

void DLLPreprocessor::reportSpatialData()
{
	BOOL inject=FALSE;
	while (!inject && m_pData) {
		m_pData->zero();
		m_pData->reportSpatialAccess();
		if (dllStruct && dllStruct->function) {
			if (m_pCallingObject)
				inject = (*(DLLPreprocessor_MAP_CSW)dllStruct->function)(dllStruct->instance, (NSFloat *)&m_pData->S(0), (NSFloat *)data, rows(), cols(), isPreprocessor(), m_pCallingObject);
			else
				inject = (*(DLLPreprocessor_MAP)dllStruct->function)(dllStruct->instance, (NSFloat *)&m_pData->S(0), (NSFloat *)data, rows(), cols(), isPreprocessor());
		}
		else
			inject = TRUE;
	}
	InputSpatialData::reportSpatialData();
}

void DLLPreprocessor::constructDLL() {
	if (dllStruct->constructor) {
		int length=rows(), width=cols();
		if (m_pCallingObject)
			dllStruct->instance = (*(DLLPreprocessor_Constructor_CSW)dllStruct->constructor)(dllStruct->instance, &length, &width, isPreprocessor(), m_pCallingObject);
		else
			dllStruct->instance = (*(DLLPreprocessor_Constructor)dllStruct->constructor)(dllStruct->instance, &length, &width, isPreprocessor());
		m_pData->setSpatialDimension(length, width);
	}
}

void DLLPreprocessor::setInputData(SpatialData *anId)
{
	if (!anId || (anId != m_pData)) {
		if (m_pData && m_bInputDataLocal)
			delete m_pData;
		if (anId) {
			m_pData = anId;
			m_bInputDataLocal = FALSE;
		}
		else {
			m_pData = new SpatialData;
			m_bInputDataLocal = TRUE;
		}
	}
}

//      Auther: Wesley Curtis Lefebvre


DLLPostprocessor::DLLPostprocessor()
{
}

BOOL DLLPostprocessor::isPreprocessor()
{
	return FALSE;
}

void DLLPostprocessor::reportSpatialData()
{
	if (m_bAccessEnabled) {
        BOOL inject = dllStruct && (BOOL)dllStruct->function;
        if (inject) {
			if (m_pCallingObject)
                inject = (*(DLLPreprocessor_MAP_CSW)dllStruct->function)(dllStruct->instance, (NSFloat *)data, (NSFloat *)&m_pData->S(0), rows(), cols(), isPreprocessor(), m_pCallingObject);
			else
                inject = (*(DLLPreprocessor_MAP)dllStruct->function)(dllStruct->instance, (NSFloat *)data, (NSFloat *)&m_pData->S(0), rows(), cols(), isPreprocessor());
		}
        if (inject)
			m_pData->reportSpatialAccess();
	}
}


// Author: Wesley Curtis Lefebvre



Soma::Soma()
{
    activityAccess = linkAccess = dllLinkAccess = NULL;
    Next = NULL;
    inputCells = outputCells = links = cells = dllLinks = 0;
    somaTemporalDimension = 1;
    cell = t = T = link = NULL;
    m_fWeightVariance = (NSFloat)0.5;
    m_fWeightMean = (NSFloat)0.0;
    m_bWeightsFixed = NO;
    dllStruct = NULL;
	m_bFreeCell = TRUE;
	m_pCallingObject = NULL;
	m_bWeightsLocallyAllocated = TRUE;
}

Soma::~Soma()
{
	deleteLinks();
	if (cell && m_bFreeCell) {
		free(cell);
		cell = NULL;
	}
}

void Soma::epochEnded()
{
}

void Soma::loadWeights(FILE *file, unsigned short int version)
{
	int weights;
	//printf("Soma::loadWeights\n");
	fscanf(file,"%d",&weights);
	//        if (weights != W()) {
	//                fprintf(stderr, "The number of stored weights (%d) does not equal those in component (%d). Reading as many as possible.",weights, W());
	//                exit(1);
	//        }
	NSFloat dummy;
	for (int i=0; i < W(); i++)
		fscanf(file,"%f", (i < W()) ? &w(i) : &dummy);
	loadStateVariables(file, version);
	if (linkAccess)
		linkAccess->reportSpatialAccess();
	if (dllLinkAccess)
		dllLinkAccess->reportSpatialAccess();
}

void Soma::loadStateVariables(FILE *file, unsigned short int version)
{
}

void Soma::saveWeights(FILE *file, BOOL addLineFeed)
{
    fprintf(file,"%d ",W());
    for (int i=0; i < W(); i++)
        fprintf(file,"%21.15e ",w(i));
    fprintf(file,"\n");
    saveStateVariables(file);
	if (addLineFeed)
		fprintf(file,"\n");
}

void Soma::saveStateVariables(FILE *file)
{
}

void Soma::setWeightMean(NSFloat aFloat) {
	m_fWeightMean = aFloat;
	networkRandomize();
}

void Soma::setWeightVariance(NSFloat aFloat) {
	m_fWeightVariance = aFloat;
	networkRandomize();
}

const char *Soma::dllFunctionName()
{
	return NULL;
}
void Soma::setDLLFunction(DLLStruct *aDLL)
{
	dllStruct = aDLL;
	if (aDLL) {
		initializeDLL();
		if (!dllStruct)
			allocateLinks(links, TRUE);
	}
}
void Soma::setDLL(void *function, void *constructor, void *destructor, void *instance, void *callingObject)
{
	dllStruct = new DLLStruct;
	dllStruct->function = function;
	dllStruct->constructor = constructor;
	dllStruct->destructor = destructor;
	dllStruct->instance = (DLLData*)instance;
	m_pCallingObject = callingObject;
	initializeDLL();
	if (!dllStruct)
		allocateLinks(links, TRUE);
}
void Soma::rangeLinks()
{
	register NSFloat *pos=&w(0), *endPos=pos+W();
	while (pos<endPos) {
		if (!finite(*pos)) {
			*pos = 0.0f;
		}
		pos++;
	}
	//      for (register int i=0; i<links; i++)
	//              if (!_finite(w(i)))
	//                      w(i) = (NSFloat)0;
	if (linkAccess)
		linkAccess->reportSpatialAccess();
	if (dllLinkAccess)
		dllLinkAccess->reportSpatialAccess();
}
void Soma::storeBufPosition()
{
	bufPosition = t;
}
void Soma::recallBufPosition()
{
	t = bufPosition;
	bufPosition = NULL;
}
void Soma::constructDLL(void) {
	if (dllStruct->constructor) {
		if (m_pCallingObject)
			dllStruct->instance = (*(DLL_Constructor_CSW)dllStruct->constructor)(dllStruct->instance, m_pCallingObject);
		else
			dllStruct->instance = (*(DLL_Constructor)dllStruct->constructor)(dllStruct->instance);
	}
}
void Soma::destroyDLL(DLLData *instance) {
	if (dllStruct->destructor) {
		if (m_pCallingObject)
			(*(DLL_Destructor_CSW)dllStruct->destructor)(instance, m_pCallingObject);
		else
			(*(DLL_Destructor)dllStruct->destructor)(instance);
	}
}
void Soma::initializeDLL(void) {
	if (dllStruct) {
		NSFloat *tempArray = NULL;
		DLLData *oldInstance = dllStruct->instance;
		int oldWeights=0;
		if (oldInstance && oldInstance->weights && oldInstance->weights->data) {
			oldWeights = oldInstance->weights->length;
			tempArray = (NSFloat *)malloc(oldWeights * sizeof(NSFloat));
			for (int i=0; i<oldWeights; i++)
				*(tempArray + i) = *(oldInstance->weights->data+i);
		}
		constructDLL();
		if (oldInstance && (oldInstance != dllStruct->instance)) {
			int newWeights=0;
			if (dllStruct->instance && dllStruct->instance->weights)
				newWeights = dllStruct->instance->weights->length;
			if ((newWeights != oldWeights) || (dllStruct->instance->weights && !dllStruct->instance->weights->data))
				allocateLinks(links, TRUE);
			if (newWeights && oldWeights) {
				int lastWeightIndex = MIN(newWeights,oldWeights);
				for (int i=0; i<lastWeightIndex; i++)
					*(dllStruct->instance->weights->data+i) = *(tempArray + i);
			}
			destroyDLL(oldInstance);
		}
		if (tempArray)
			free(tempArray);
	}
}
void Soma::allocateLinks(int _links, BOOL force)
{
    if (force || (_links != links)) {
        links = _links;
        if (link && m_bWeightsLocallyAllocated)
            free((char*)link);
        if (dllStruct && dllStruct->instance && dllStruct->instance->weights)
            dllLinks = dllStruct->instance->weights->length;
        else
			dllLinks = 0;
		int linksToAllocate = MAX(links+dllLinks,1);
        link = (NSFloat*)malloc(linksToAllocate*sizeof(NSFloat));
		m_bWeightsLocallyAllocated = TRUE;
        networkRandomize();
        linksAltered();
		if (dllLinks)
			dllStruct->instance->weights->data = link+links;
    }
}
void Soma::assignLinks(NSFloat *newLink, BOOL freeLater)
{
    if (link && m_bWeightsLocallyAllocated)
		free((char*)link);
	m_bWeightsLocallyAllocated = freeLater;
    link = newLink;
    linksAltered();
    if (dllStruct && dllStruct->instance && dllStruct->instance->weights)
		dllLinks = dllStruct->instance->weights->length;
    else
		dllLinks = 0;
    if (dllLinks)
		dllStruct->instance->weights->data = link+links;
}
void Soma::attachTo(Soma *aSoma, int delay)
{
	if (aSoma) {
		if (aSoma->temporalDimension() < delay + 1)
			aSoma->setTemporalDimension(delay + 1);
		Soma::setOutputCells(aSoma->V());
		cells = aSoma->cells;
		cell = aSoma->cell;
		m_bFreeCell = FALSE;
		t = aSoma->t;
		T = aSoma->T;
		for (int i=0; i<delay; i++)
			decrement();
		activityAltered();
	} else {
		//      Soma::setOutputCells(0);
		cells = 0;
		cell = NULL;
		t = NULL;
		T = NULL;
	}
}
void Soma::decrementTime()
{
	decrement();
}
void Soma::deleteLinks()
{
	if (link) {
		if (m_bWeightsLocallyAllocated && link)
			free((char*)link);
		link = NULL;
		links = 0;
		dllLinks = 0;
		linksAltered();
	}
}
void Soma::incrementTime()
{
	increment();
}
void Soma::fire()
{
	incrementTime();
	if (Next)
		Next->fire();
}
void Soma::fireDummy()
{
	if (Next)
		Next->fireDummy();
}
void Soma::activityAltered()
{
	if (activityAccess)
		activityAccess->setData(t, V());
}
void Soma::linksAltered()
{
	if (linkAccess)
		linkAccess->setData(link, w());
	if (dllLinkAccess)
		dllLinkAccess->setData(link+w(), dllLinks);
}
void Soma::networkJog()
{
	register NSFloat *pos=link, *endPos=pos+links;
	while (pos<endPos)
		*pos++ += 2*m_fWeightVariance/RAND_MAX*rand()-m_fWeightVariance;
	if (linkAccess)
		linkAccess->reportSpatialAccess();
	if (dllLinkAccess)
		dllLinkAccess->reportSpatialAccess();
}
void Soma::networkRandomize()
{
	if (!weightsFixed() && link) {
		for (int i=0; i<W(); i++)
			w(i) = 2*m_fWeightVariance/RAND_MAX*rand()-m_fWeightVariance + m_fWeightMean;
	}
	if (linkAccess)
		linkAccess->reportSpatialAccess();
	if (dllLinkAccess)
		dllLinkAccess->reportSpatialAccess();
}
void Soma::networkReset()
{
	SpatialAccess::m_bRefreshAccess = TRUE;
    for (NSFloat *floatPtr=cell; floatPtr<T; floatPtr++)
		*floatPtr = (NSFloat)0;
	//      zeroState();
    if (activityAccess)
        activityAccess->reportSpatialAccess();
	//      if (preActivityAccess)
	//              preActivityAccess->reportSpatialAccess();
	SpatialAccess::m_bRefreshAccess = FALSE;
}
void Soma::setActivityAccess(SpatialAccess *aNSAccess)
{
	activityAccess = aNSAccess;
	activityAltered();
}
void Soma::setLinkAccess(SpatialAccess *aNSAccess)
{
	linkAccess = aNSAccess;
	linksAltered();
}
void Soma::setDLLLinkAccess(SpatialAccess *aNSAccess)
{
	dllLinkAccess = aNSAccess;
	linksAltered();
}
void Soma::setInputCells(int numberOfCells)
{
	inputCells = numberOfCells;
}
void Soma::setOutputCells(int numberOfCells)
{
	outputCells = numberOfCells;
}
void Soma::setTemporalDimension(int T)
{
	somaTemporalDimension = T;
}

// Author: Wesley Curtis Lefebvre

Axon::Axon() {
	m_bFirePending = FALSE;
    m_iRows = 0;
    m_iCols = 1;
    dummyFiredCounter = firedCounter = lastCount = nextCount = 0;
    forcedLast = NULL;
    preActivityAccess = NULL;
    m_bFireNext = YES;
	m_bFireNextOnReset = YES;
	m_pIPInputBuffer = NULL;
}
Axon::~Axon(void) {
}
void Axon::activityAltered(void) {
	if (preActivityAccess)
		preActivityAccess->setData(t, rows(), cols());
	if (activityAccess)
		activityAccess->setData(t, rows(), cols());
}
void Axon::loadWeights(FILE *file, unsigned short int version)
{
	int rows, cols;
	//printf("Axon::loadWeights\n");
	fscanf(file,"%d",&rows);
	fscanf(file,"%d",&cols);
	if ((rows < 1) || (rows > 1000000)) {
		fprintf(stderr, "Number of rows in axon out of range. Read %i, valid range is 1-1000000.",rows);
		exit(1);
	}
	if ((cols < 1) || (cols > 1000000)) {
		fprintf(stderr, "Number of columns in axon out of range. Read %i, valid range is 1-1000000.",cols);
		exit(1);
	}
	setRows(rows);
	setCols(cols);
	if (version > 223) {
		fscanf(file,"%d",&m_bFireNext);
	}
	Soma::loadWeights(file, version);
}
void Axon::saveWeights(FILE *file, BOOL addLineFeed)
{
	fprintf(file,"%d ",rows());
	fprintf(file,"%d\n",cols());
	fprintf(file,"%d\n",m_bFireNext);
	Soma::saveWeights(file, addLineFeed);
}
void Axon::networkReset()
{
	Soma::networkReset();
	setFireNext(m_bFireNextOnReset);
}
void Axon::initialize(void) {
	int oldCells = cells;
	NSFloat *cellToFree = cell;
	NSFloat *oldT = T;
	cells = MAX(V(), U());
	if (cells <= 1)
		cells = 1;
	t = cell = (NSFloat*)calloc((temporalDimension()+1) * cells, sizeof(NSFloat));
	T = cell + (temporalDimension()+1) * cells;
	if (cellToFree) {
		if (oldCells == cells)
			for (int i=0; i<cells; i++)
				*(T+i-cells) = *(oldT+i-cells);
			if (m_bFreeCell)
				free((char*)cellToFree);
	}
	m_bFreeCell = TRUE;
	activityAltered();
	initializeDLL();
}
void Axon::constructDLL(void) {
	if (dllStruct->constructor) {
		if (m_pCallingObject)
			dllStruct->instance = (DLLData*)(*(Axon_Constructor_CSW)dllStruct->constructor)(dllStruct->instance,m_iRows, m_iCols, m_pCallingObject);
		else
			dllStruct->instance = (DLLData*)(*(Axon_Constructor)dllStruct->constructor)(dllStruct->instance,m_iRows, m_iCols);
	}
}
void Axon::incrementTime(void) {
	Soma::incrementTime();
	register NSFloat *pos=t, *endPos=pos+cells;
	while (pos<endPos)
		*pos++ = 0.0f;
}
void Axon::fireFeed(void) {
	register NSFloat *pos=&V(0), *nextPos=&(Next->V(0)), *endPos=pos+V();
	while (pos<endPos)
		*nextPos++ += *pos++;
}
const char *Axon::dllFunctionName()
{
	return "performAxon";
}
void Axon::fireDLL(void)
{
	if (m_pCallingObject)
        (*(Axon_MAP_CSW)dllStruct->function)(dllStruct->instance, &V(0), m_iRows, m_iCols, m_pCallingObject);
	else
        (*(Axon_MAP)dllStruct->function)(dllStruct->instance, &V(0), m_iRows, m_iCols);
}
void Axon::fireMap(void)
{
}
void Axon::fire(void) {
	if (++firedCounter >= lastCount) {
		firedCounter = 0;
		if (preActivityAccess)
			preActivityAccess->reportSpatialAccess(t);
		if (m_bFireNext) {
			m_bFirePending = FALSE;
			dummyFiredCounter = 0;
			if (dllStruct)
				fireDLL();
			else
				fireMap();
			if (activityAccess)
				activityAccess->reportSpatialAccess(t);
			if (m_pIPInputBuffer)
				for (int i=0; i<V(); i++)
					*(m_pIPInputBuffer+i) = V(i);
				register Soma **pos=nextConnection+nextCount, **endPos=nextConnection;
				while (--pos >= endPos) {
					Next = *pos;
					if (Next->isAnAxon())
						fireFeed();
					Next->fire();
				}
				incrementTime();
		} else {
			register Soma **pos=nextConnection+nextCount, **endPos=nextConnection;
			while (--pos >= endPos) {
				Next = *pos;
				Next->fireDummy();
			}
			register NSFloat *pos2=t, *endPos2=pos2+cells;
			while (pos2<endPos2)
				*pos2++ = 0.0f;
		}
		/*              for (register int i=nextCount-1; i>=0; i--) {
		Next = nextConnection[i];
		if (Next->isAnAxon())
		fireFeed();
		Next->fire();
	}  */
	}
	else
		m_bFirePending = m_bFireNext;
}
void Axon::fireDummy(void) {
	if ((++firedCounter >= lastCount) && m_bFirePending)
		fire();
	else
		if (++dummyFiredCounter >= lastCount) {
			dummyFiredCounter = 0;
			register Soma **pos=nextConnection+nextCount, **endPos=nextConnection;
			while (--pos >= endPos) {
				Next = *pos;
				Next->fireDummy();
			}
		}
}
void Axon::removeLast(Soma *aSoma) {
	lastCount--;
}
void Axon::removeNext(Soma *aSoma) {
	BOOL shiftDown = FALSE;
	for (int i=0; i<nextCount; i++) {
		if (shiftDown)
			nextConnection[i-1] = nextConnection[i];
		if (aSoma == nextConnection[i])
			shiftDown = TRUE;
	}
	if (shiftDown)
		nextCount--;
}
void Axon::setCols(int anInt) {
	if (anInt != cols()) {
		if (cols() < 1)
			m_iCols = 1;
		m_iCols = anInt;
		setInputCells(rows() * cols());
	}
}
void Axon::setDLLFunction(DLLStruct *aDLL)
{
	Soma::setDLLFunction(aDLL);
	//      initialize();
}
void Axon::setInputCells(int numberOfCells) {
	if (numberOfCells) {
		Soma::setInputCells(numberOfCells);
		Soma::setOutputCells(numberOfCells);
		initialize();
	}
}
void Axon::setLast(Soma *aSoma) {
	lastCount++;
}
void Axon::setNext(Soma *nextSoma) {
	if (nextSoma && !(nextSoma->isASynapse() && nextSoma->temporalDimension()))
		nextConnection[nextCount++] = nextSoma;
}
void Axon::setOutputCells(int numberOfCells) {
	Soma::setOutputCells(V());
}
void Axon::setPreActivityAccess(SpatialAccess* aNSAccess) {
	preActivityAccess = aNSAccess;
	activityAltered();
}
void Axon::setRows(int anInt) {
	if (anInt != rows()) {
		m_iRows = anInt;
		if (rows() < 1)
			m_iRows = 1;
		setInputCells(rows() * cols());
	}
}
void Axon::setTemporalDimension(int T) {
	if (T != temporalDimension()) {
		Soma::setTemporalDimension(T);
		initialize();
	}
}
/*
//ExpanderAxon
  ExpanderAxon::ExpanderAxon() {
  m_nExpansionFactor = 2;
  }
	void ExpanderAxon::fireMap(void) {
    register NSFloat *pos1=&V(0), *pos2=&V(0)+U(), *endPos=pos1+V();
    while (pos2<endPos) {
	for (int i=0; i<U(); i++)
	*pos2++ = *pos1++;
	pos1=&V(0);
	}
	}
	  void ExpanderAxon::setInputCells(int anInt) {
	  Soma::setInputCells(anInt);
	  Soma::setOutputCells(m_nExpansionFactor * U());
	  initialize();
	  }
*/
//FuzzyAxon
FuzzyAxon::FuzzyAxon() {
	m_nMFCount = 2;
	m_WinningMF = NULL;
	m_WinningInput = NULL;
	m_bOldFuzzyFormat = FALSE;
	MFOutputAccess = NULL;
	m_MFOutput = NULL;
}
FuzzyAxon::~FuzzyAxon() {
	if (m_WinningMF) {
		free(m_WinningMF);
		m_WinningMF = NULL;
	}
	if (m_WinningInput) {
		free(m_WinningInput);
		m_WinningInput = NULL;
	}
	if (m_MFOutput) {
		free(m_MFOutput);
		m_MFOutput = NULL;
	}
}
void FuzzyAxon::activityAltered(void) {
	if (!m_bOldFuzzyFormat) {
		if (preActivityAccess)
			preActivityAccess->setData(t, rows(), cols());
		if (activityAccess)
			activityAccess->setData(&V(0), (int)pow(m_nMFCount,U()-1), m_nMFCount);
	}
}
void FuzzyAxon::constructDLL(void) {
    if (dllStruct->constructor) {
		if (m_pCallingObject)
            dllStruct->instance = (DLLData*)(*(FuzzyAxon_Constructor_CSW)dllStruct->constructor)(dllStruct->instance, m_iRows, m_iCols, &m_nMFCount, m_pCallingObject);
		else
            dllStruct->instance = (DLLData*)(*(FuzzyAxon_Constructor)dllStruct->constructor)(dllStruct->instance, m_iRows, m_iCols, &m_nMFCount);
	}
}
const char *FuzzyAxon::dllFunctionName()
{
    return "performFuzzyAxon";
}
void FuzzyAxon::fireDLL(void)
{
	fireMap();
}
void FuzzyAxon::fireMap(void) {
	NSFloat *firingStrengthArray = (NSFloat *)malloc(V() * sizeof(NSFloat));
    register NSFloat *pos=&V(0), *endPos=pos+V(), *firingPos = firingStrengthArray, *endFiringPos = firingStrengthArray+V();
	BOOL dllActive = (dllStruct) ? TRUE : FALSE;
	if (!m_bOldFuzzyFormat) {
		if (dllActive) {
			NSFloat newValue;
			for (int i=0; i<U(); i++)
				for (int j=0; j<m_nMFCount; j++) {
					int paramIndex = i*m_nMFCount + j;
					if (m_pCallingObject)
						(*(FuzzyAxon_MAP_CSW)dllStruct->function)(dllStruct->instance, &V(0), m_iRows, m_iCols, &w(0), paramIndex, i, &newValue, m_pCallingObject);
					else
						(*(FuzzyAxon_MAP)dllStruct->function)(dllStruct->instance, &V(0), m_iRows, m_iCols, &w(0), paramIndex, i, &newValue);
					*(m_MFOutput + paramIndex) = newValue;
				}
		}
		else {
			for (int i=0; i<U(); i++)
				for (int j=0; j<m_nMFCount; j++) {
					int paramIndex = i*m_nMFCount + j;
					*(m_MFOutput + paramIndex) = applyMF(paramIndex, V(i));
				}
		}
	}
	int i=0;
	while (firingPos<endFiringPos) {
		*firingPos++ = firingStrength(i++,dllActive);
	}
	firingPos = firingStrengthArray;
    while (pos<endPos) {
		*pos++ = *firingPos++;
	}
	if (MFOutputAccess)
		MFOutputAccess->reportSpatialAccess();
	free(firingStrengthArray);
}
NSFloat FuzzyAxon::firingStrength(int index, BOOL callDLL) {
	NSFloat returnValue = 9.9e10f;
	NSFloat newValue;
	if (m_bOldFuzzyFormat) {
		if (callDLL) {
			for (int i=0; i<U(); i++) {
				int paramIndex = i * m_nMFCount + index;
				if (m_pCallingObject)
					(*(FuzzyAxon_MAP_CSW)dllStruct->function)(dllStruct->instance, &V(0), m_iRows, m_iCols, &w(0), paramIndex, i, &newValue, m_pCallingObject);
				else
					(*(FuzzyAxon_MAP)dllStruct->function)(dllStruct->instance, &V(0), m_iRows, m_iCols, &w(0), paramIndex, i, &newValue);
				if (newValue < returnValue) {
					returnValue = newValue;
					*(m_WinningMF + index) = paramIndex;
					*(m_WinningInput + index) = V(i);
				}
			}
		}
		else {
			for (int i=0; i<U(); i++) {
				int paramIndex = i * m_nMFCount + index;
				if ((newValue = applyMF(paramIndex, V(i))) < returnValue) {
					returnValue = newValue;
					*(m_WinningMF + index) = paramIndex;
					*(m_WinningInput + index) = V(i);
				}
			}
		}
	}
	else {
		for (int i=0; i<U(); i++) {
			int paramIndex = i * m_nMFCount + MFMapping(i, index);
			if ((newValue = *(m_MFOutput + paramIndex)) < returnValue) {
				returnValue = newValue;
				*(m_WinningMF + index) = paramIndex;
				*(m_WinningInput + index) = V(i);
			}
		}
	}
	return returnValue;
}
void FuzzyAxon::initialize(void) {
    Axon::initialize();
    allocateLinks(U() * m_nMFCount * MFParameters());
	if (m_WinningMF)
		free(m_WinningMF);
	m_WinningMF = (int *)malloc(V() * sizeof(int));
	if (m_WinningInput)
		free(m_WinningInput);
	m_WinningInput = (NSFloat *)malloc(V() * sizeof(NSFloat));
	if (m_MFOutput)
		free(m_MFOutput);
	m_MFOutput = (NSFloat *)malloc(U() * m_nMFCount * sizeof(NSFloat));
    if (MFOutputAccess)
        MFOutputAccess->setData(m_MFOutput, U(), m_nMFCount);
}
//Comment for Single Conection
int FuzzyAxon::MFMapping(int inputIndex, int outputIndex) {
	int mult = (int)pow(m_nMFCount, inputIndex);
	int mult1 = (mult * m_nMFCount);
	int remainder = outputIndex - (((outputIndex / mult1) * mult1));
	return remainder/mult;
}
//Comment for Single Conection
void FuzzyAxon::setInputCells(int anInt) {
    Soma::setInputCells(anInt);
	Soma::setOutputCells(m_bOldFuzzyFormat ? m_nMFCount : (int)pow(m_nMFCount,U()));
    initialize();
}
void FuzzyAxon::setMFOutputAccess(SpatialAccess *aNSAccess)
{
	MFOutputAccess = aNSAccess;
    if (MFOutputAccess)
        MFOutputAccess->setData(m_MFOutput, U(), m_nMFCount);
	activityAltered();
}
//BellFuzzyAxon
NSFloat BellFuzzyAxon::applyMF(int paramIndex, NSFloat x)
{
	int baseIndex = paramIndex * MFParameters();
	NSFloat a = w(baseIndex);
	NSFloat b = w(baseIndex + 1);
	NSFloat c = w(baseIndex + 2);
	if (a == 0.0f)
		return 0.0f;
	NSFloat tmp1 = (x - c) / a;
	NSFloat tmp2 = tmp1 == 0.0f ? 0.0f : (NSFloat)pow((tmp1*tmp1), b);
	return(1 / (1 + tmp2));
}
void BellFuzzyAxon::networkRandomize()
{	BOOL resetFixedFlag = FALSE;
	if (!weightsFixed() && w()) {
        int i;
		for (i=0; i<w(); i=i+MFParameters())
			w(i) = (m_nMFCount == 1) ? 1.0f : 1.0f/((NSFloat)(m_nMFCount - 1));	// Assumes -1 to 1 range
		for (i=1; i<w(); i=i+MFParameters())
			w(i) = 1.0f;
		int j=0;
		for (i=2; i<w(); i=i+MFParameters()) {
//			w(i) = (m_nMFCount == 1) ? 0.0f : -1.0f + (2.0f*(NSFloat)j)/((NSFloat)(m_nMFCount - 1)); // Assumes -1 to 1 range
			w(i) = -1.0f + ((NSFloat)j + 0.5f) * (2.0f/(NSFloat)m_nMFCount); // Assumes -1 to 1 range
			j++;
			if (j == m_nMFCount)
				j = 0;
		}
		m_bWeightsFixed = TRUE;
		resetFixedFlag = TRUE;
	}
	Axon::networkRandomize();
	if (resetFixedFlag)
		m_bWeightsFixed = FALSE;
}
//GaussianFuzzyAxon
/*
const char *GaussianFuzzyAxon::dllFunctionName()
{
return "performGaussianFuzzyAxon";
}
  void GaussianFuzzyAxon::fireDLL(void)
  {
  if (m_pCallingObject)
  (*(GaussianFuzzyAxon_MAP_CSW)dllStruct->function)(dllStruct->instance, &V(0), m_iRows, m_iCols, &w(0), m_pCallingObject);
  else
  (*(GaussianFuzzyAxon_MAP)dllStruct->function)(dllStruct->instance, &V(0), m_iRows, m_iCols, &w(0));
  }
*/
NSFloat GaussianFuzzyAxon::applyMF(int paramIndex, NSFloat x)
{
	int baseIndex = paramIndex * MFParameters();
	NSFloat c = w(baseIndex);
	NSFloat sigma = w(baseIndex + 1);
	if (sigma == 0.0f)
		return 0.0f;
	NSFloat exp_fraction = (x - c) / sigma;
	NSFloat exp_final =  (NSFloat) (((double)exp_fraction*(double)exp_fraction) / (double)-2.0);
	return (NSFloat)exp(exp_final);
}
void GaussianFuzzyAxon::networkRandomize()
{	BOOL resetFixedFlag = FALSE;
	if (!weightsFixed() && w()) {
        int i;
		for (i=1; i<w(); i=i+MFParameters())
			w(i) = (m_nMFCount == 1) ? 1.0f : 1.0f/((NSFloat)(m_nMFCount - 1));	// Assumes -1 to 1 range
		int j=0;
		for (i=0; i<w(); i=i+MFParameters()) {
//			w(i) = (m_nMFCount == 1) ? 0.0f : -1.0f + (2.0f*(NSFloat)j)/((NSFloat)(m_nMFCount - 1)); // Assumes -1 to 1 range
			w(i) = -1.0f + ((NSFloat)j + 0.5f) * (2.0f/(NSFloat)m_nMFCount); // Assumes -1 to 1 range
			j++;
			if (j == m_nMFCount)
				j = 0;
		}
		m_bWeightsFixed = TRUE;
		resetFixedFlag = TRUE;
	}
	Axon::networkRandomize();
	if (resetFixedFlag)
		m_bWeightsFixed = FALSE;
}
//CombinerAxon
CombinerAxon::CombinerAxon()
{
	m_iRows = 2;
}
void CombinerAxon::activityAltered(void) {
	if (preActivityAccess)
		preActivityAccess->setData(t, rows(), cols());
	if (activityAccess)
		activityAccess->setData(&V(0), rows()/2, cols());
}
void CombinerAxon::fireMap(void) {
    register NSFloat *pos=&V(0), *endPos=pos+V();
    while (pos<endPos)
		*pos++ = *pos * *(pos + V());
}
void CombinerAxon::setInputCells(int anInt) {
    Soma::setInputCells(anInt);
    Soma::setOutputCells(U()/2);
    initialize();
}
//NormalizedAxon
void NormalizedAxon::fireMap(void) {
	int weightCount = 0;
	if (nextCount && ((Synapse *)nextConnection[0])->Next && ((Axon *)((Synapse *)nextConnection[0])->Next)->nextCount) {
		int divisor = ((Synapse *)((Axon *)((Synapse *)nextConnection[0])->Next)->nextConnection[0])->U();
		if (divisor)
			weightCount = U() / divisor;
	}
    register NSFloat *pos=&V(0), *endPos=pos+weightCount, *weightPos=&w(0), *WFPos;
	NSFloat sumWeight = 0.0f;
    while (pos<endPos) {
		sumWeight += *pos++;
	}
	if (m_nWeightingFactorArraySize != V()) {
		if (m_WeightingFactorArray)
			free(m_WeightingFactorArray);
		m_WeightingFactorArray = (NSFloat *)calloc(V(), sizeof(NSFloat));
		m_nWeightingFactorArraySize = V();
	}
	if (sumWeight != 0.0f) {
		WFPos = m_WeightingFactorArray;
		pos=&V(0);
		endPos=pos+V();
		while (pos<endPos) {
			NSFloat weight = *pos;
			*WFPos = weight / sumWeight;
			*pos++ = *WFPos++;
		}
	}
}
// Author: Wesley Curtis Lefebvre

const char *BiasAxon::dllFunctionName()
{
	return "performBiasAxon";
}
void BiasAxon::fireDLL(void)
{
	if (m_pCallingObject)
        (*(BiasAxon_MAP_CSW)dllStruct->function)(dllStruct->instance, &V(0), m_iRows, m_iCols, &w(0), m_pCallingObject);
	else
        (*(BiasAxon_MAP)dllStruct->function)(dllStruct->instance, &V(0), m_iRows, m_iCols, &w(0));
}
void BiasAxon::initialize(void) {
	allocateLinks(U());
	Axon::initialize();
}
void BiasAxon::fireMap(void) {
	register NSFloat *pos=&V(0), *endPos=pos+V(), *weightPos=&w(0);
	while (pos<endPos)
		fireInline(*pos++,*weightPos++);
}

// Author: Wesley Curtis Lefebvre

LinearAxon::LinearAxon()
{
	m_fBeta = 1.0f;
}
const char *LinearAxon::dllFunctionName()
{
	return "performLinearAxon";
}
void LinearAxon::fireDLL(void)
{
	if (m_pCallingObject)
        (*(LinearAxon_MAP_CSW)dllStruct->function)(dllStruct->instance, (NSFloat *)&V(0), m_iRows, m_iCols, (NSFloat *)&w(0), (NSFloat)m_fBeta, m_pCallingObject);
	else
        (*(LinearAxon_MAP)dllStruct->function)(dllStruct->instance, (NSFloat *)&V(0), m_iRows, m_iCols, (NSFloat *)&w(0), (NSFloat)m_fBeta);
}
void LinearAxon::fireMap(void) {
	register NSFloat *pos=&V(0), *endPos=pos+V(), *weightPos=&w(0);
	while (pos<endPos)
		fireInline(*pos++,*weightPos++);
}

// Author: Wesley Curtis Lefebvre

void SigmoidAxon::fireMap(void) {
	register NSFloat *pos=&V(0), *endPos=pos+V(), *weightPos=&w(0);
	while (pos<endPos)
		fireInline(*pos++,*weightPos++);
}

//NormalizedSigmoidAxon
void NormalizedSigmoidAxon::fireMap(void) {
	int weightCount = 0;
	Soma *nextSoma = nextCount ? nextConnection[0] : NULL;
	if (nextSoma) {
		int divisor = nextSoma->U();
		if (divisor)
			weightCount = U() / divisor;
	}
    register NSFloat *pos=&V(0), *endPos=pos+weightCount, *weightPos=&w(0), *WFPos;
	NSFloat sumWeight = 0.0f;
    while (pos<endPos) {
		sumWeight += *pos++;
	}
	if (m_nWeightingFactorArraySize != V()) {
		if (m_WeightingFactorArray)
			free(m_WeightingFactorArray);
		m_WeightingFactorArray = (NSFloat *)calloc(V(), sizeof(NSFloat));
		m_nWeightingFactorArraySize = V();
	}
	if (sumWeight != 0.0f) {
		WFPos = m_WeightingFactorArray;
		pos=&V(0);
		endPos=pos+V();
		while (pos<endPos) {
			NSFloat weight = *pos;
			fireInline(*pos,*weightPos++);
			*WFPos = weight / sumWeight;
			*pos++ *= *WFPos++;
		}
	}
}
// Author: Wesley Curtis Lefebvre

void TanhAxon::fireMap(void) {
	register NSFloat *pos=&V(0), *endPos=pos+V(), *weightPos=&w(0);
	while (pos<endPos)
		fireInline(*pos++,*weightPos++);
}

// Author: Wesley Curtis Lefebvre

void LinearSigmoidAxon::fireMap(void) {
	register NSFloat *pos=&V(0), *endPos=pos+V(), *weightPos=&w(0);
	while (pos<endPos)
		fireInline(*pos++,*weightPos++);
}

// Author: Wesley Curtis Lefebvre

void LinearTanhAxon::fireMap(void) {
	register NSFloat *pos=&V(0), *endPos=pos+V(), *weightPos=&w(0);
	while (pos<endPos)
		fireInline(*pos++,*weightPos++);
}

// Author: Wesley Curtis Lefebvre

SoftMaxAxon::SoftMaxAxon()
{
	m_fWeightVariance = (NSFloat)0.0;
}
void SoftMaxAxon::fireMap(void) {
	register NSFloat *pos=&V(0), *endPos=pos+V(), sum=0.0f;
	while (pos<endPos) {
		*pos *= m_fBeta;
		if (*pos > 88.0f)
			*pos = 88.0f;
		if (*pos < -88.0f)
			*pos = -88.0f;
		*pos = (NSFloat)exp(*pos);
		sum += *pos++;
	}
	pos=&V(0);
	while (pos<endPos)
		*pos++ /= sum;
}
// Author: Wesley Curtis Lefebvre

WinnerTakeAllAxon::WinnerTakeAllAxon()
{
    m_bMaxWinner = NO;
    m_fWinningPE = 0;
	winningPEAccess = NULL;
}
WinnerTakeAllAxon::~WinnerTakeAllAxon()
{
}
void WinnerTakeAllAxon::fireMap(void) {
	register NSFloat *pos=&V(0), *endPos=pos+V(), *winner=pos++;
	if (winningPEAccess) {
		m_fWinningPE = 0.0f;
		int posCounter = 1;
		if (m_bMaxWinner)
			while (pos<endPos) {
				if (*pos > *winner) {
					winner = pos;
					m_fWinningPE = (NSFloat)posCounter;
				}
				pos++;
				posCounter++;
			}
			else
				while (pos<endPos) {
					if (*pos < *winner) {
						winner = pos;
						m_fWinningPE = (NSFloat)posCounter;
					}
					pos++;
					posCounter++;
				}
				winningPEAccess->reportSpatialAccess();
	}
	else {
		if (m_bMaxWinner)
			while (pos<endPos) {
				if (*pos > *winner)
					winner = pos;
				pos++;
			}
			else
				while (pos<endPos) {
					if (*pos < *winner)
						winner = pos;
					pos++;
				}
	}
	pos = &V(0);
	while (pos<endPos)
		*pos++ = 0.0f;
	*winner = 1.0f;
}
void WinnerTakeAllAxon::setWinningPEAccess(SpatialAccess *anId)
{
    winningPEAccess = anId;
    if (winningPEAccess)
        winningPEAccess->setData(&m_fWinningPE, 1);
}
// Author: Wesley Curtis Lefebvre

void ThresholdAxon::fireMap(void) {
	register NSFloat *pos=&V(0), *endPos=pos+V(), *weightPos=&w(0);
	while (pos<endPos)
		fireInline(*pos++,*weightPos++);
}

// Author: Wesley Curtis Lefebvre

GaussianAxon::GaussianAxon() {
	variance = nil;
	m_pFullSynapse = nil;
	m_iMetric = 0;
	m_iNeighbors = 1;
	m_bBetaDataLocal = FALSE;
	m_bNormalize = FALSE;
}
GaussianAxon::~GaussianAxon() {
	if (variance && m_bBetaDataLocal)
		delete variance;
}
void GaussianAxon::assignCenters()
{
	register NSFloat *pos=&w(0), *endPos=pos+w();
	if (m_iMetric == 0) {
		int i=0;
		while (pos<endPos) {
			*pos = 0.0f;
			register NSFloat *weightPos=&(m_pFullSynapse->W(i++,0)), *endWeightPos=weightPos+m_pFullSynapse->V();
			while (weightPos<endWeightPos) {
				*pos -= *weightPos * *weightPos;
				weightPos++;
			}
			pos++;
		}
	} else
		while (pos<endPos)
			*pos++ = 0.0f;
        if (linkAccess)
			linkAccess->reportSpatialAccess();
}
void GaussianAxon::assignVariance()
{
	int neighborhood = neighbors()<V()-1? neighbors(): V()-1;
	NSFloat *minDistance = (NSFloat*)calloc(neighborhood, sizeof(NSFloat));
	register NSFloat *pos=minDistance, *endPos=pos+neighborhood;
	while (pos<endPos)
		*pos++ = 1e10f;
	for (int i=0; i<V(); i++) {
		register NSFloat *endWeightPos=&(m_pFullSynapse->W(i,0))+m_pFullSynapse->V(), distance;
		for (int j=0; j<V(); j++) {
			if (i != j) {
				register NSFloat *weightPos=&(m_pFullSynapse->W(i,0));
				register NSFloat *weight2Pos=&(m_pFullSynapse->W(j,0));
				distance = 0.0f;
				while (weightPos<endWeightPos) {
					register NSFloat x = *weightPos++ - *weight2Pos++;
					distance += x * x;
				}
				pos = minDistance;
				while (pos<endPos) {
					if (distance < *pos) {
						NSFloat *temp = pos;
						while (++pos<endPos)
							*pos = *(pos-1);
						*temp = distance;
					}
					pos++;
				}
			}
		}
		pos = minDistance;
		register NSFloat averageDistance = 0.0f;
		while (pos<endPos) {
			averageDistance += *pos;
			*pos++ = 1e10f;
		}
		variance->S(i) = (averageDistance > 0.0f) ? 0.5f*neighborhood/averageDistance : 1e10f;
	}
	variance->reportSpatialAccess();
}
void GaussianAxon::fireMap(void)
{
	register NSFloat *pos=&V(0), *endPos=pos+V(), *weightPos=&w(0), *variancePos=&(variance->S(0)), tempSum = 0.0f;
	while (pos<endPos) {
		fireInline(*pos, *weightPos++, *variancePos++);
		tempSum += *pos++;
	}
	if (m_bNormalize) {
		pos = &V(0);
		while (pos<endPos)
			if (tempSum != 0.0f)
				*pos++ /= tempSum;
			else
				*pos++ = 0.0f;
	}
}
void GaussianAxon::loadWeights(FILE *file, unsigned short int version)
{
	//printf("GaussianAxon::loadWeights\n");
	LinearAxon::loadWeights(file, version);
	for (int i=0; i < V(); i++)
		fscanf(file,"%f", &(variance->S(i)));
	if (variance)
		variance->reportSpatialAccess();
}
void GaussianAxon::saveWeights(FILE *file, BOOL addLineFeed)
{
	LinearAxon::saveWeights(file,FALSE);
	for (int i=0; i < V(); i++)
		fprintf(file,"%21.15e ",variance->S(i));
	fprintf(file,"\n\n");
}
void GaussianAxon::initialize()
{
	LinearAxon::initialize();
	variance->setSpatialDimension(V());
	setBeta(m_fBeta);
}
void GaussianAxon::setBeta(NSFloat aFloat)
{
	LinearAxon::setBeta(aFloat);
	variance->assignScalar(m_fBeta);
}
void GaussianAxon::setBetaData(SpatialData *anId)
{
	if (!anId || (anId != variance)) {
		if (variance && m_bBetaDataLocal)
			delete variance;
		if (anId) {
			variance = anId;
			m_bBetaDataLocal = FALSE;
		}
		else {
			variance = new SpatialData;
			m_bBetaDataLocal = TRUE;
		}
	}
}
void GaussianAxon::setNeighbors(int anInt)
{
	m_iNeighbors = anInt;
	if (m_iNeighbors < 1)
		m_iNeighbors = 1;
}

// Author: Wesley Curtis Lefebvre

TDNNAxon::TDNNAxon() {
	memoryTaps = delay = 1;
	delayedReference = new Soma;
}
void TDNNAxon::activityAltered(void) {
	if (preActivityAccess)
		preActivityAccess->setData(&V(0), rows(), cols());
	if (activityAccess)
		activityAccess->setData(&V(0), taps(), rows()*cols());
}
void TDNNAxon::setTemporalDimension(int T) {
	if ((T != temporalDimension()) && ((temporalDimension() != delay+1) || (T > temporalDimension())))
		Axon::setTemporalDimension(T);
}
void TDNNAxon::loadWeights(FILE *file, unsigned short int version)
{
	int taps;
	//printf("TDNNAxon::loadWeights\n");
	fscanf(file,"%d", &taps);
	if ((taps < 1) || (taps > 1000000)) {
		fprintf(stderr, "Number of taps in TDNNAxon out of range. Read %i, valid range is 1-1000000.",taps);
		exit(1);
	}
	setTaps(taps);
	Axon::loadWeights(file, version);
}
void TDNNAxon::saveWeights(FILE *file, BOOL addLineFeed)
{
	fprintf(file,"%d\n", memoryTaps);
	Axon::saveWeights(file, addLineFeed);
}
void TDNNAxon::decrementTime(void) {
	delayedReference->decrementTime();
	Axon::decrementTime();
}
const char *TDNNAxon::dllFunctionName()
{
	return "performTDNNAxon";
}
void TDNNAxon::fireDLL(void)
{
	if (m_pCallingObject)
        (*(TDNNAxon_MAP_CSW)dllStruct->function)(dllStruct->instance, &V(0), m_iRows, m_iCols, &Vd(0), memoryTaps, m_pCallingObject);
	else
        (*(TDNNAxon_MAP)dllStruct->function)(dllStruct->instance, &V(0), m_iRows, m_iCols, &Vd(0), memoryTaps);
}
void TDNNAxon::fireMap(void) {
	register NSFloat *pos=&V(U()), *endPos=pos+V()-U(), *delayPos=&Vd(0);
	while (pos<endPos)
		*pos++ = *delayPos++;
}
void TDNNAxon::initialize(void) {
	Axon::initialize();
	delayedReference->attachTo(this, delay);
}
void TDNNAxon::constructDLL(void) {
	if (dllStruct->constructor) {
		if (m_pCallingObject)
			dllStruct->instance = (*(TDNNAxon_Constructor_CSW)dllStruct->constructor)(dllStruct->instance, m_iRows, m_iCols, memoryTaps, m_pCallingObject);
		else
			dllStruct->instance = (*(TDNNAxon_Constructor)dllStruct->constructor)(dllStruct->instance, m_iRows, m_iCols, memoryTaps);
	}
}
void TDNNAxon::incrementTime(void) {
	delayedReference->incrementTime();
	Axon::incrementTime();
}
void TDNNAxon::loadStateVariables(FILE *file, unsigned short int version)
{
	if (version > 123) {
		int elementCount;
		NSFloat dummy;
		fscanf(file,"%d", &elementCount);
		for (int i=0; i<elementCount; i++)
			fscanf(file,"%f", i<V() ? &Vd(i) : &dummy);
	}
	Axon::loadStateVariables(file, version);
}
void TDNNAxon::saveStateVariables(FILE *file)
{
    fprintf(file,"%d ", V());
    for (int i=0; i<V(); i++)
		fprintf(file,"%21.15e ", Vd(i));
    fprintf(file,"\n");
    Axon::saveStateVariables(file);
}
void TDNNAxon::recallBufPosition(void) {
	delayedReference->recallBufPosition();
	Axon::recallBufPosition();
}
void TDNNAxon::setTapDelay(int anInt) {
	if (anInt && (anInt != delay)) {
		delay = anInt;
		initialize();
	}
}
void TDNNAxon::setInputCells(int anInt) {
	Soma::setInputCells(anInt);
	Soma::setOutputCells(memoryTaps * U());
	initialize();
}
void TDNNAxon::setOutputCells(int anInt) {
	Soma::setOutputCells(V());
}
void TDNNAxon::setMemoryTaps(int anInt) {
	memoryTaps = anInt;
	if (memoryTaps < 1)
		memoryTaps = 1;
}
void TDNNAxon::setTaps(int anInt) {
	if (anInt && (anInt != memoryTaps)) {
		setMemoryTaps(anInt);
		Soma::setOutputCells(memoryTaps * U());
		initialize();
	}
}
void TDNNAxon::storeBufPosition(void) {
	delayedReference->storeBufPosition();
	Axon::storeBufPosition();
}
void TDNNAxon::zeroState(void) {
	Axon::zeroState();
	delayedReference->networkReset();
}
TDNNAxon::~TDNNAxon(void) {
	delayedReference->attachTo(NULL,0);
	delete delayedReference;
}
// Author: Wesley Curtis Lefebvre

const char *GammaAxon::dllFunctionName()
{
	return "performGammaAxon";
}
void GammaAxon::fireDLL(void)
{
	if (m_pCallingObject)
        (*(GammaAxon_MAP_CSW)dllStruct->function)(dllStruct->instance, &V(0), m_iRows, m_iCols, &Vd(0), memoryTaps, &w(0), m_pCallingObject);
	else
        (*(GammaAxon_MAP)dllStruct->function)(dllStruct->instance, &V(0), m_iRows, m_iCols, &Vd(0), memoryTaps, &w(0));
}
void GammaAxon::initialize(void) {
	if (w() != U()) {
		allocateLinks(U());
		for (int i=0; i<w(); i++)
			w(i) = (NSFloat)1;
	}
	TDNNAxon::initialize();
}
void GammaAxon::fireMap(void) {
	register NSFloat
		*pos=&V(U()),
		*endPos=pos+V()-U(),
		*delayPos=&Vd(0),
		*delayPosTap=&Vd(U()),
		*weightPos=&w(0)-1,
		*endWeightPos=weightPos+w();
	while (pos<endPos) {
		while (++weightPos<=endWeightPos)
			*pos++ = *weightPos * *delayPos++ + (1.0f-*weightPos) * *delayPosTap++;
		weightPos=&w(0)-1;
	}
}
void GammaAxon::networkRandomize(void) {
	TDNNAxon::networkRandomize();
	for (register int i=0; i<w(); i++)
		w(i) = (NSFloat)1;
	if (linkAccess)
		linkAccess->reportSpatialAccess();
	if (dllLinkAccess)
		dllLinkAccess->reportSpatialAccess();
}
void GammaAxon::rangeLinks(void) {
	for (register int i=0; i<U(); i++) {
		if (w(i) > 1.9)
			w(i) = (NSFloat)1.9;
		if (w(i) < 0.1)
			w(i) = (NSFloat)0.1;
	}
	TDNNAxon::rangeLinks();
}

// Author: Wesley Curtis Lefebvre

void LaguarreAxon::initialize(void) {
	if (w() != U()) {
		allocateLinks(U());
		for (int i=0; i<w(); i++)
			w(i) = 0.0f;
	}
	TDNNAxon::initialize();
}
void LaguarreAxon::fireMap(void) {
	for (register int i=0,j,k; i<U(); i++) {
		NSFloat gain = (NSFloat)sqrt(1.0-(w(i)*w(i)));
		for (j=0; j<memoryTaps; j++) {
			k = i + j*U();
			if (j==0) {
				V(k) *= gain;
				V(k) += w(i)*Vd(k);
			}
			else
				V(k) = w(i)*Vd(k) + Vd(k-U()) - w(i)*V(k-U());
		}
	}
}
void LaguarreAxon::networkRandomize(void) {
	TDNNAxon::networkRandomize();
	for (register int i=0; i<W(); i++)
		w(i) = 0.0f;
	if (linkAccess)
		linkAccess->reportSpatialAccess();
}
void LaguarreAxon::rangeLinks(void) {
	register NSFloat *pos=&w(0), *endPos=pos+w();
	while (pos<endPos) {
		if (*pos > 0.99f)
			*pos = 0.99f;
		if (*pos < -0.99f)
			*pos = -0.99f;
		pos++;
	}
	TDNNAxon::rangeLinks();
}

// Author: Wesley Curtis Lefebvre

ContextAxon::ContextAxon() {
	delay = 1;
	delayedReference = new Soma;
	m_fWeightMean = 0.5f;
}
void ContextAxon::decrementTime(void) {
	delayedReference->decrementTime();
	LinearAxon::decrementTime();
}
const char *ContextAxon::dllFunctionName()
{
	return "performContextAxon";
}
void ContextAxon::fireDLL(void)
{
	if (m_pCallingObject)
        (*(ContextAxon_MAP_CSW)dllStruct->function)(dllStruct->instance, &V(0), m_iRows, m_iCols, &Vd(0), &w(0), m_fBeta, m_pCallingObject);
	else
        (*(ContextAxon_MAP)dllStruct->function)(dllStruct->instance, &V(0), m_iRows, m_iCols, &Vd(0), &w(0), m_fBeta);
}
void ContextAxon::fireMap(void) {
	register NSFloat *pos=&V(0), *endPos=pos+V(), *delayPos=&Vd(0), *weightPos=&w(0);
	while (pos<endPos)
		*pos++ = m_fBeta * (*pos + *weightPos++ * *delayPos++);
}
void ContextAxon::initialize(void) {
	LinearAxon::initialize();
	rangeLinks();
	delayedReference->attachTo(this, delay);
}
void ContextAxon::incrementTime(void) {
	delayedReference->incrementTime();
	LinearAxon::incrementTime();
}
void ContextAxon::rangeLinks(void) {
	register NSFloat *pos=&w(0), *endPos=pos+w();
	while (pos<endPos) {
		if (*pos > 0.99f)
			*pos = 0.99f;
		if (*pos < 0.01f)
			*pos = 0.01f;
		pos++;
	}
	LinearAxon::rangeLinks();
}
void ContextAxon::recallBufPosition(void) {
	delayedReference->recallBufPosition();
	LinearAxon::recallBufPosition();
}
void ContextAxon::loadStateVariables(FILE *file, unsigned short int version)
{
	if (version > 123) {
		int elementCount;
		NSFloat dummy;
		fscanf(file,"%d", &elementCount);
		for (int i=0; i<elementCount; i++)
			fscanf(file,"%f", i<V() ? &Vd(i) : &dummy);
	}
	LinearAxon::loadStateVariables(file, version);
}
void ContextAxon::saveStateVariables(FILE *file)
{
    fprintf(file,"%d ", V());
    for (int i=0; i<V(); i++)
        fprintf(file,"%21.15e ", Vd(i));
    fprintf(file,"\n");
    LinearAxon::saveStateVariables(file);
}
void ContextAxon::storeBufPosition(void) {
	delayedReference->storeBufPosition();
	LinearAxon::storeBufPosition();
}
void ContextAxon::zeroState(void) {
	LinearAxon::zeroState();
	delayedReference->networkReset();
}
ContextAxon::~ContextAxon(void) {
	delayedReference->attachTo(NULL,0);
	delete delayedReference;
}
// Author: Wesley Curtis Lefebvre

void SigmoidContextAxon::fireMap(void) {
	register NSFloat *pos=&V(0), *endPos=pos+V(), *delayPos=&Vd(0), *weightPos=&w(0);
	while (pos<endPos) {
		*pos = 1.0f/(1.0f+(NSFloat)exp(-*pos));
		*pos++ = m_fBeta * (*pos + *weightPos++ * *delayPos++);
		//              *pos = m_fBeta * (*pos + *weightPos++ * *delayPos++);  //Changed 9/9/97
		//              *pos++ = 1.0f/(1.0f+(NSFloat)exp(-*pos));
	}
}
// Author: Wesley Curtis Lefebvre

void TanhContextAxon::fireMap(void) {
	register NSFloat *pos=&V(0), *endPos=pos+V(), *delayPos=&Vd(0), *weightPos=&w(0);
	while (pos<endPos) {
		*pos = (NSFloat)tanh(*pos);
		*pos++ = m_fBeta * (*pos + *weightPos++ * *delayPos++);
		//              *pos = m_fBeta * (*pos + *weightPos++ * *delayPos++);  // Changed 9/9/97
		//              *pos++ = (NSFloat)tanh(*pos);
	}
}
// Author: Wesley Curtis Lefebvre

void IntegratorAxon::fireMap(void) {
	register NSFloat *pos=&V(0), *endPos=pos+V(), *delayPos=&Vd(0), *weightPos=&w(0);
	while (pos<endPos) {
		*pos++ = m_fBeta * ((1.0f-*weightPos) * *pos + *weightPos * *delayPos++);
		weightPos++;
	}
}

// Author: Wesley Curtis Lefebvre

void SigmoidIntegratorAxon::fireMap(void) {
	register NSFloat *pos=&V(0), *endPos=pos+V(), *delayPos=&Vd(0), *weightPos=&w(0);
	while (pos<endPos) {
		*pos = 1.0f/(1.0f+(NSFloat)exp(-*pos));
		*pos++ = m_fBeta * ((1.0f-*weightPos) * *pos + *weightPos * *delayPos++);
		//              *pos = m_fBeta * ((1.0f-*weightPos) * *pos + *weightPos * *delayPos++);   // Changed 9/9/97
		//              *pos++ = 1.0f/(1.0f+(NSFloat)exp(-*pos));
		weightPos++;
	}
}
// Author: Wesley Curtis Lefebvre

void TanhIntegratorAxon::fireMap(void) {
	register NSFloat *pos=&V(0), *endPos=pos+V(), *delayPos=&Vd(0), *weightPos=&w(0);
	while (pos<endPos) {
		*pos = (NSFloat)tanh(*pos);
		*pos++ = m_fBeta * ((1.0f-*weightPos) * *pos + *weightPos * *delayPos++);
		//              *pos = m_fBeta * ((1.0f-*weightPos) * *pos + *weightPos * *delayPos++);
		//              *pos++ = (NSFloat)tanh(*pos);
		weightPos++;
	}
}

// Author: Wesley Curtis Lefebvre

Synapse::Synapse() {
	//      initializationAccess = NULL;
	Last = Feed = NULL;
	somaTemporalDimension = 0;
}
Synapse::~Synapse() {
	Last = Feed = NULL;
	cells = 0;
	cell = t = T = NULL;
}
void Synapse::constructDLL(void) {
	if (dllStruct->constructor && Last && Last->rows() && Last->cols() && Feed && Feed->rows() && Feed->cols()) {
		if (m_pCallingObject)
			dllStruct->instance = (*(Synapse_Constructor_CSW)dllStruct->constructor)(dllStruct->instance, Last->rows(), Last->cols(), Feed->rows(), Feed->cols(), m_pCallingObject);
		else
			dllStruct->instance = (*(Synapse_Constructor)dllStruct->constructor)(dllStruct->instance, Last->rows(), Last->cols(), Feed->rows(), Feed->cols());
	}
}
void Synapse::allocateSynapticLinks(void) {
	if (Last && Feed)
		initializeDLL();
}
void Synapse::detachYourself(void) {
	Soma::setOutputCells(cells = 0);
	cell = t = T = NULL;
	activityAltered();
}
const char *Synapse::dllFunctionName()
{
	return "performSynapse";
}
void Synapse::fireDLL(void)
{
	if (m_pCallingObject)
        (*(Synapse_MAP_CSW)dllStruct->function)(dllStruct->instance, &V(0), Last->rows(), Last->cols(), &(Feed->V(0)), Feed->rows(), Feed->cols(), m_pCallingObject);
	else
        (*(Synapse_MAP)dllStruct->function)(dllStruct->instance, &V(0), Last->rows(), Last->cols(), &(Feed->V(0)), Feed->rows(), Feed->cols());
}
void Synapse::fire(void) {
	if (activityAccess)
		activityAccess->reportSpatialAccess(t);
	if (dllStruct)
		fireDLL();
	else
		fireFeed();
	Soma::fire();
}
void Synapse::fireFeed(void) {
	register NSFloat *pos=&V(0), *feedPos=&(Feed->V(0)), *endPos=pos+(V()<U()? V(): U());
	while (pos<endPos)
		*feedPos++ += *pos++;
}
void Synapse::loadStateVariables(FILE *file, unsigned short int version)
{
	if ((version > 123) && temporalDimension()) {
		int elementCount;
		NSFloat dummy;
		fscanf(file,"%d", &elementCount);
		for (int i=0; i<elementCount; i++)
			fscanf(file,"%f", i<V() ? &V(i) : &dummy);
	}
	Soma::loadStateVariables(file, version);
}
void Synapse::saveStateVariables(FILE *file)
{
    if (temporalDimension()) {
        fprintf(file,"%d ", V());
        for (int i=0; i<V(); i++)
            fprintf(file,"%21.15e ", V(i));
        fprintf(file,"\n");
    }
    Soma::saveStateVariables(file);
}
void Synapse::removeLast(Soma *aSoma) {
	Last = NULL;
	detachYourself();
	deleteLinks();
}
void Synapse::retryConnections() {
	Soma::setInputCells(Feed->U());
	attachTo((Soma*)Last, temporalDimension());
	if (temporalDimension())
		Last->removeNext(this);
	if (Last && Feed)
		allocateSynapticLinks();
}
void Synapse::setLast(Soma *aSoma) {
	Last = (Axon*)aSoma;
	attachTo((Soma*)Last, temporalDimension());
	if (temporalDimension())
		Last->removeNext(this);
	if (Last && Feed)
		allocateSynapticLinks();
}
void Synapse::removeNext(Soma *aSoma) {
	Next = NULL;
	Feed = NULL;
	Soma::setInputCells(0);
	deleteLinks();
}
void Synapse::setDLLFunction(DLLStruct *aDLL)
{
	Soma::setDLLFunction(aDLL);
	//      if (V() && U())
	//              allocateSynapticLinks();
}
/*void Synapse::setInitializationAccess(NAccess* aNSAccess) {
if (initializationAccess = aNSAccess)
initializationAccess->setDataLength(V())->setDataWidth(1);
}*/
void Synapse::setNext(Soma *aSoma) {
	Next = aSoma;
	Feed = (Axon*)aSoma;
	Soma::setInputCells(Feed->U());
	if (Last && Feed)
		allocateSynapticLinks();
}
void Synapse::setTemporalDimension(int T) {
	Soma::setTemporalDimension(T);
	if (Last)
		setLast((Soma*)Last);
}
void Synapse::zeroState()
{
	Soma::zeroState();
	if (temporalDimension())
		Soma::networkReset();
}

// Author: Wesley Curtis Lefebvre

void FullSynapse::allocateSynapticLinks(void) {
	allocateLinks(U()*V());
	Synapse::allocateSynapticLinks();
}
const char *FullSynapse::dllFunctionName()
{
	return "performFullSynapse";
}
void FullSynapse::linksAltered(void) {
	if (linkAccess)
		linkAccess->setData(&(w(0)), U(), V());
	if (dllLinkAccess)
		dllLinkAccess->setData(&(w(0))+U()*V(), dllLinks);
}
void FullSynapse::fireDLL(void)
{
	if (m_pCallingObject)
        (*(FullSynapse_MAP_CSW)dllStruct->function)(dllStruct->instance, &V(0), Last->rows(), Last->cols(), &(Feed->V(0)), Feed->rows(), Feed->cols(), &w(0), m_pCallingObject);
	else
        (*(FullSynapse_MAP)dllStruct->function)(dllStruct->instance, &V(0), Last->rows(), Last->cols(), &(Feed->V(0)), Feed->rows(), Feed->cols(), &w(0));
}
void FullSynapse::fireFeed(void) {
    register NSFloat *pos=&V(0), *feedPos=&(Feed->V(0)), *endPos=pos+V(), *endFeedPos=feedPos+U(), *weightPos=&w(0);
    while (feedPos<endFeedPos) {
        pos=&V(0);
        while (pos<endPos)
			*feedPos += *weightPos++ * *pos++;
        feedPos++;
    }
}
// SVMOutputSynapse
SVMOutputSynapse::SVMOutputSynapse()
{
	m_bRunningFirstEpoch = TRUE;
	m_fpDesiredMatrix = NULL;
	m_fSVMBiasVector = NULL;
}
void SVMOutputSynapse::epochEnded()
{
	m_bRunningFirstEpoch = FALSE;
	FullSynapse::epochEnded();
}
void SVMOutputSynapse::fireFeed(void)
{
	register NSFloat *pos=&V(0), *feedPos=&(Feed->V(0)), *endPos=pos+V(), *endFeedPos=feedPos+U(), *weightPos=&w(0), *matrixPos = m_fpDesiredMatrix, *biasPos = m_fSVMBiasVector;
	if (m_bRunningFirstEpoch) {
		while (feedPos<endFeedPos)
			*feedPos++ = 0.0f;
	}
	else {
		if (matrixPos) {
			while (feedPos<endFeedPos) {
				pos=&V(0);
				while (pos<endPos) {
					*feedPos += *weightPos++ * *matrixPos++ * *pos++;
				}
				*feedPos++ += *biasPos++;
			}
		}
	}
}
void SVMOutputSynapse::linksAltered(void)
{
	if (m_fpDesiredMatrix)
		free(m_fpDesiredMatrix);
	m_fpDesiredMatrix = (Feed && V()*Feed->V()) ? (NSFloat *)calloc(V()*Feed->V(),sizeof(NSFloat)) : NULL;
	if (m_fSVMBiasVector)
		free(m_fSVMBiasVector);
	m_fSVMBiasVector = (Feed && Feed->V()) ? (NSFloat *)calloc(Feed->V(),sizeof(NSFloat)) : NULL;
	FullSynapse::linksAltered();
}
void SVMOutputSynapse::loadStateVariables(FILE *file, unsigned short int version)
{
    if (version > 217) {
		int elementCount,outputCount;
		NSFloat aFloat;
        fscanf(file,"%d", &elementCount);
		int k=0;
		for (int i=0; i<U(); i++) {
			for (int j=0; j<V(); j++) {
				if (k<elementCount) {
					fscanf(file,"%f", &aFloat);
					setDesiredElement(i,j,aFloat);
				}
				k++;
			}
		}
		fscanf(file,"%d", &outputCount);
		for (int l=0; l<outputCount; l++) {
			fscanf(file,"%f", &aFloat);
			if (l<U())
				setBiasElement(l,aFloat);
		}
    }
	if (version > 219)
		fscanf(file,"%d", &m_bRunningFirstEpoch);
	FullSynapse::loadStateVariables(file, version);
}
void SVMOutputSynapse::networkReset()
{
	if (Feed)
		for (int i=0; i<Feed->V(); i++)
			*(m_fSVMBiasVector+i) = 0.1f;
		m_bRunningFirstEpoch = TRUE;
		FullSynapse::networkReset();
}
void SVMOutputSynapse::saveStateVariables(FILE *file)
{
	fprintf(file,"%d ",U()*V());
	for (int i=0; i<U(); i++)
		for (int j=0; j<V(); j++)
			fprintf(file,"%21.15e ", desiredElement(i,j));
		fprintf(file,"\n");
		fprintf(file,"%d ",U());
		for (int k=0; k<U(); k++)
			fprintf(file,"%21.15e ", biasElement(k));
		fprintf(file,"\n%d ",m_bRunningFirstEpoch);
		fprintf(file,"\n");
		FullSynapse::saveStateVariables(file);
}
void SVMOutputSynapse::setDesiredElement(int outIndex, int exemplar, NSFloat aFloat)
{
	if (m_fpDesiredMatrix)
		*(m_fpDesiredMatrix+outIndex*V()+exemplar) = aFloat;
}
// Author: Wesley Curtis Lefebvre
ArbitrarySynapse::ArbitrarySynapse()
{
	connections = NULL;
}
void ArbitrarySynapse::loadWeights(FILE *file, unsigned short int version)
{       int                             i,j,fromIndex,toIndex,toCount,connCount;
//printf("ArbitrarySynapse::loadWeights\n");
connections->empty();
fscanf(file,"%d",&connCount);
if ((connCount < 0) || (connCount > V())) {
	fprintf(stderr,"number of connections out of range. Read %i, valid range is 0-%i.",connCount,V());
	exit(0);
}
for(i=0;i<connCount;i++){
	fscanf(file,"%d",&fromIndex);
	if ((fromIndex < 0) || (fromIndex > V()-1)) {
		fprintf(stderr,"\"from\" index out of range. Read %i, valid range is 0-%i.",fromIndex,V()-1);
		exit(0);
	}
	fscanf(file,"%d",&toCount);
	if ((toCount < 1) || (toCount > U())) {
		fprintf(stderr,"number of \"to\" connections out of range. Read %i, valid range is 1-%i.",toCount,U());
		exit(0);
	}
	for(j=0;j<toCount;j++){
		fscanf(file,"%d",&toIndex);
		if ((toIndex < 0) || (toIndex > U()-1)) {
			fprintf(stderr,"\"to\" index out of range. Read %i, valid range is 0-%i.",toIndex,U()-1);
			exit(0);
		}
		connections->addConnection(fromIndex, toIndex);
	}
}
allocateLinks(connections->weights());
Soma::loadWeights(file, version);
}

void ArbitrarySynapse::saveWeights(FILE *file, BOOL addLineFeed)
{
	fprintf(file,"%d\n",connections->count());
	for(int i=0,j;i<connections->count();i++){
		Connection *aConnection = connections->connectionAt(i);
		fprintf(file,"%d ",aConnection->feedingIndex());
		fprintf(file,"%d ",aConnection->count());
		int *toConnections = aConnection->connections();
		for(j=0;j<aConnection->count();j++)
			fprintf(file,"%d ",toConnections[j]);
		fprintf(file,"\n");
	}
	Soma::saveWeights(file, addLineFeed);
}

const char *ArbitrarySynapse::dllFunctionName()
{
	return NULL;
}
void ArbitrarySynapse::fireFeed(void) {
	for (register int i=0,j,k=0; i<connections->count(); i++) {
		Connection *aConnection = connections->connectionAt(i);
		int *indices = aConnection->connections();
		for (j=0; j<aConnection->count(); j++,k++)
			Feed->V(indices[j]) += w(k)*V(aConnection->feedingIndex());
	}
	/*      register NSFloat *weightPos=&w(0);
	Connection **pos=&(connections->connectionAt(0)), **endPos=pos+connections->count();
	while (pos<endPos) {
	register int *indexPos=(*pos)->connections(), *endIndexPos=indexPos+(*pos)->count();
	while (indexPos<endIndexPos)
	Feed->V(*indexPos++) += *weightPos++ * V((*pos)->feedingIndex());
	pos++;
}       */
}
void ArbitrarySynapse::freeConnectionList()
{
	delete connections;
	connections = NULL;
}
// Author: Wesley Curtis Lefebvre
// Name: FunctionIO.h

FunctionIO::FunctionIO()
{
	m_readWrite = READ;
	m_pFunction = NULL;
	m_pCallingObject = NULL;
}
void FunctionIO::reportSpatialData()
{	NSFloat *functionData = data;
if (m_bAccessEnabled) {
	if (m_bPerformNormalization && !m_readWrite) {
		denormalizeData();
		functionData = m_fDenormalizedData;
	}
	if (m_pFunction) {
		if (m_pCallingObject)
			(*(FunctionIO_MAP_CSW)m_pFunction)(functionData, rows(), cols(), m_pCallingObject);
		else
			(*(FunctionIO_MAP)m_pFunction)(functionData, rows(), cols());
	}
	if (m_bPerformNormalization && m_readWrite)
		normalizeData();
}
}
void FunctionIO::setFunction(void *aFunction, void *callingObject)
{
	m_pFunction = aFunction;
	m_pCallingObject = callingObject;
}
void FunctionIO::setMode(RW readWrite)
{
	m_readWrite = readWrite;
}
// Author: Wesley Curtis Lefebvre
//
BackAxon::BackAxon()
{
	Dual = NULL;
	gradientsEnabled = YES;
	m_fWeightingVector = NULL;
	m_nWeightingVectorSize = 0;
	m_nCurrentExemplar = 0;
}
void BackAxon::activityAltered(void) {
	if (preActivityAccess)
		preActivityAccess->setData(t, Dual ? Dual->rows() : rows(), Dual ? Dual->cols() : cols());
	if (activityAccess)
		activityAccess->setData(t, Dual ? Dual->rows() : rows(), Dual ? Dual->cols() : cols());
	//      Axon::activityAltered();
}
void BackAxon::backpropComplete()
{
	Dual->recallBufPosition();
}
void BackAxon::backpropStarting()
{
	Dual->storeBufPosition();
	Dual->decrementTime();
}
void BackAxon::constructDLL(void) {
	if (Dual && dllStruct->constructor) {
		if (m_pCallingObject)
			dllStruct->instance = (DLLData*)(*(BackAxon_Constructor_CSW)dllStruct->constructor)(dllStruct->instance,Dual->dllStruct? Dual->dllStruct->instance: NULL,Dual->rows(), Dual->cols(), m_pCallingObject);
		else
			dllStruct->instance = (DLLData*)(*(BackAxon_Constructor)dllStruct->constructor)(dllStruct->instance,Dual->dllStruct? Dual->dllStruct->instance: NULL,Dual->rows(), Dual->cols());
	}
}
const char *BackAxon::dllFunctionName()
{
	return "performBackAxon";
}
void BackAxon::fireDLL(void)
{
	if (m_pCallingObject)
        (*(BackAxon_MAP_CSW)dllStruct->function)(dllStruct->instance, Dual->dllStruct? Dual->dllStruct->instance: NULL, &(Dual->V(0)), Dual->rows(), Dual->cols(), &V(0), m_pCallingObject);
	else
        (*(BackAxon_MAP)dllStruct->function)(dllStruct->instance, Dual->dllStruct? Dual->dllStruct->instance: NULL, &(Dual->V(0)), Dual->rows(), Dual->cols(), &V(0));
}
void BackAxon::fire()
{
	Axon::fire();
	if (linkAccess)
		linkAccess->reportSpatialAccess();
}
void BackAxon::incrementTime()
{
	if (gradientsEnabled)
		Dual->decrementTime();
	Axon::incrementTime();
}
void BackAxon::linksAltered() {
	Axon::linksAltered();
	//wclnow        if (linkAccess)
	//              linkAccess->setSpatialName("Weight Gradients");
}
void BackAxon::networkRandomize()
{
	Axon::networkRandomize();
	register NSFloat *pos=&w(0), *endPos=pos+W();
	while (pos<endPos)
		*pos++ = 0.0f;
	//      for (int i=0; i<w(); i++)
	//              w(i) = (NSFloat)0;
	if (linkAccess)
		linkAccess->reportSpatialAccess();
	if (dllLinkAccess)
		dllLinkAccess->reportSpatialAccess();
}
void BackAxon::setBackpropOffset(int anOffset)
{
	if (Dual)
		if (anOffset < temporalDimension())
			Dual->setTemporalDimension(temporalDimension());
		else
			Dual->setTemporalDimension(anOffset);
}
void BackAxon::setCurrentExemplar(int exemplar)
{
	m_nCurrentExemplar = exemplar;
}
void BackAxon::setDual(Soma *aSoma)
{
	Dual = (Axon*)aSoma;
	if (Dual && ((U() != Dual->V()) || (V() != Dual->U()))) {
		Soma::setInputCells(Dual->V());
		Soma::setOutputCells(Dual->U());
		m_iRows = Dual->rows();
		m_iCols = Dual->cols();
		initialize();
	}
}
void BackAxon::setGradientsEnabled(BOOL aFlag)
{
	gradientsEnabled = aFlag;
}
void BackAxon::setWeightingVector(NSFloat *aVector, int vectorSize)
{
	m_fWeightingVector = aVector;
	m_nWeightingVectorSize = vectorSize;
}
void BackAxon::updateExemplarCounter()
{
	m_nCurrentExemplar++;
	if (m_nCurrentExemplar >= m_nWeightingVectorSize)
		m_nCurrentExemplar = 0;
}
//BackFuzzyAxon

void BackFuzzyAxon::activityAltered(void) {
	if (Dual && !((FuzzyAxon *)Dual)->m_bOldFuzzyFormat) {
		if (preActivityAccess)
			preActivityAccess->setData(&V(0), (int)pow(((FuzzyAxon *)Dual)->m_nMFCount,V()-1), ((FuzzyAxon *)Dual)->m_nMFCount);
		if (activityAccess)
			activityAccess->setData(t, Dual ? Dual->rows() : rows(), Dual ? Dual->cols() : cols());
	}
}
NSFloat BackFuzzyAxon::computeDerivative(int weightIndex, int outputIndex, BOOL dllActive)
{
	if (dllActive) {
		FuzzyAxon *fuzzyAxon = (FuzzyAxon *)Dual;
		int paramCount = fuzzyAxon->MFParameters();
		int winner = fuzzyAxon->winningMF(outputIndex)*paramCount;
		NSFloat x = fuzzyAxon->winningInput(outputIndex);
		NSFloat newValue = 0.0f;
		if ((weightIndex >= winner) && (weightIndex < (winner + paramCount))) {
			if (m_pCallingObject)
				(*(BackFuzzyAxon_MAP_CSW)dllStruct->function)(dllStruct->instance, Dual->dllStruct? Dual->dllStruct->instance: NULL, &(Dual->V(0)), Dual->rows(), Dual->cols(), &V(0), &(fuzzyAxon->w(0)), weightIndex, winner, x, &newValue, m_pCallingObject);
			else
				(*(BackFuzzyAxon_MAP)dllStruct->function)(dllStruct->instance, Dual->dllStruct? Dual->dllStruct->instance: NULL, &(Dual->V(0)), Dual->rows(), Dual->cols(), &V(0), &(fuzzyAxon->w(0)), weightIndex, winner, x, &newValue);
		}
		return newValue;
	}
	else
		return derivative(weightIndex, outputIndex);
}
const char *BackFuzzyAxon::dllFunctionName()
{
    return "performBackFuzzyAxon";
}
void BackFuzzyAxon::fireDLL(void)
{
	fireMap();
}
void BackFuzzyAxon::fireMap()
{
	BOOL dllActive = (dllStruct) ? TRUE : FALSE;
    if (gradientsEnabled) {
        register NSFloat *pos=&V(0), *endPos=pos+U(), *weightPos=&w(0), *endWeightPos=weightPos+w();
        if (m_nWeightingVectorSize) {
			int j = 0;
			while (pos<endPos) {
				int i = 0;
				weightPos=&w(0);
				while (weightPos<endWeightPos)
					*weightPos++ += *pos * computeDerivative(i++,j,dllActive) * *(m_fWeightingVector + m_nCurrentExemplar);
				pos++;
				j++;
			}
			updateExemplarCounter();
        }
        else {
			int j = 0;
			while (pos<endPos) {
				int i = 0;
				weightPos=&w(0);
				while (weightPos<endWeightPos)
					*weightPos++ += *pos * computeDerivative(i++,j,dllActive);
				pos++;
				j++;
			}
        }
	}
/*	else {
		 register NSFloat *pos=&V(0), *endPos=pos+U();
		 int j = 0;
		 while (pos<endPos) {
		 int i = 0;
		 while (i<w())
		 *pos *= derivative(i++,j);
		 pos++;
		 j++;
		 }
} */
}
void BackFuzzyAxon::initialize() {
	FuzzyAxon *fuzzyAxon = (FuzzyAxon *)Dual;
    allocateLinks(V() * fuzzyAxon->m_nMFCount * fuzzyAxon->MFParameters());
    BackAxon::initialize();
}
//BackBellFuzzyAxon
NSFloat BackBellFuzzyAxon::derivative(int weightIndex, int outputIndex)
{	NSFloat returnValue = 0.0f;
FuzzyAxon *fuzzyAxon = (FuzzyAxon *)Dual;
int paramCount = fuzzyAxon->MFParameters();
int winner = fuzzyAxon->winningMF(outputIndex)*paramCount;
NSFloat x = fuzzyAxon->winningInput(outputIndex);
if ((weightIndex >= winner) && (weightIndex < (winner + paramCount))) {
	NSFloat a = fuzzyAxon->w(winner);
	if (a == 0.0f)
		returnValue = 0.0f;
	NSFloat b = fuzzyAxon->w(winner + 1);
	NSFloat c = fuzzyAxon->w(winner + 2);
	NSFloat tmp1 = (x - c)/a;
	NSFloat tmp2 = tmp1 == 0 ? 0 : (NSFloat)pow(tmp1*tmp1, b);
	NSFloat denom = (1 + tmp2)*(1 + tmp2);
	if (weightIndex == winner)
		returnValue = (2*b*tmp2/(a*denom));
	if (weightIndex == (winner + 1)) {
		if (tmp1 == 0)
			returnValue = 0.0f;
		else
			returnValue = ((NSFloat)-log(tmp1*tmp1)*tmp2/denom);
	}
	if (weightIndex == (winner + 2)) {
		if (x == c)
			returnValue = 0.0f;
		else
			returnValue = (2*b*tmp2/((x - c)*(denom)));
	}
}
return returnValue;
}
//BackGaussianFuzzyAxon
NSFloat BackGaussianFuzzyAxon::derivative(int weightIndex, int outputIndex) {
	NSFloat returnValue = 0.0f;
	FuzzyAxon *fuzzyAxon = (FuzzyAxon *)Dual;
	int paramCount = fuzzyAxon->MFParameters();
	int winner = fuzzyAxon->winningMF(outputIndex)*paramCount;
	NSFloat x = fuzzyAxon->winningInput(outputIndex);
	if ((weightIndex >= winner) && (weightIndex < (winner + paramCount))) {
		NSFloat c = fuzzyAxon->w(winner);
		NSFloat sigma = fuzzyAxon->w(winner + 1);
		if (sigma == 0.0f)
			returnValue = 0.0f;
		NSFloat exp_fraction = (x - c) / sigma;
		NSFloat exp_final = (NSFloat)((exp_fraction*exp_fraction) / -2.0);
		NSFloat fwrd_activation = (NSFloat)exp (exp_final);
		if (weightIndex == winner)
		{
			NSFloat deriv_fwrd = (NSFloat)((x - c) / (sigma*sigma));
			returnValue = fwrd_activation * deriv_fwrd;
		}
		if (weightIndex == (winner + 1)) {
			NSFloat deriv_fwrd = (NSFloat)(((x - c)*(x - c)) / pow (sigma, 3.0));
			returnValue = fwrd_activation * deriv_fwrd;
		}
	}
	return returnValue;
}
//BackCombinerAxon
void BackCombinerAxon::fireMap()
{
    register NSFloat *pos1=&V(0), *pos2=pos1+U(), *endPos1=pos1+U(), *dualPos1=&(Dual->V(0)), *dualPos2=&(Dual->V(U()));
    while (pos1<endPos1) {
		*pos2++ = (*pos1++ * ((*dualPos2 == 0.0f) ? 1.0f  : (*dualPos1 / *dualPos2)));
		dualPos1++;
		dualPos2++;
	}
	pos1=&V(0);
	pos2=pos1+U();
	dualPos2=&(Dual->V(U()));
    while (pos1<endPos1)
		*pos1++ *= *dualPos2++;
}

//BackNormalizedAxon
void BackNormalizedAxon::fireMap()
{
    register NSFloat *pos=&V(0), *endPos=pos+U(), *dualWF=((NormalizedAxon *)Dual)->weightingFactorArray();
    while (pos<endPos)
		*pos++ *= *dualWF++;
}

// Author: Wesley Curtis Lefebvre

const char *BackBiasAxon::dllFunctionName()
{
	return "performBackBiasAxon";
}
void BackBiasAxon::fireDLL(void)
{
	if (m_pCallingObject)
        (*(BackBiasAxon_MAP_CSW)dllStruct->function)(dllStruct->instance, Dual->dllStruct? Dual->dllStruct->instance: NULL, &(Dual->V(0)), Dual->rows(), Dual->cols(), &V(0), &w(0), m_pCallingObject);
	else
        (*(BackBiasAxon_MAP)dllStruct->function)(dllStruct->instance, Dual->dllStruct? Dual->dllStruct->instance: NULL, &(Dual->V(0)), Dual->rows(), Dual->cols(), &V(0), &w(0));
}
void BackBiasAxon::fireMap() {
	if (gradientsEnabled) {
		register NSFloat *pos=&V(0), *endPos=pos+U(), *weightPos=&w(0);
		if (m_nWeightingVectorSize) {
			while (pos<endPos)
				*weightPos++ += *pos++ * *(m_fWeightingVector + m_nCurrentExemplar);
			updateExemplarCounter();
		}
		else {
			while (pos<endPos)
				*weightPos++ += *pos++;
		}
	}
}
void BackBiasAxon::initialize() {
	allocateLinks(U());
	BackAxon::initialize();
}

// Author: Wesley Curtis Lefebvre

BackLinearAxon::BackLinearAxon() {
	m_fOffset = 0.0f;
}
const char *BackLinearAxon::dllFunctionName()
{
	return "performBackLinearAxon";
}
void BackLinearAxon::fireDLL(void)
{
	if (m_pCallingObject)
        (*(BackLinearAxon_MAP_CSW)dllStruct->function)(dllStruct->instance, Dual->dllStruct? Dual->dllStruct->instance: NULL, &(Dual->V(0)), Dual->rows(), Dual->cols(), &V(0), &w(0), linearAxon()->beta(), m_pCallingObject);
	else
        (*(BackLinearAxon_MAP)dllStruct->function)(dllStruct->instance, Dual->dllStruct? Dual->dllStruct->instance: NULL, &(Dual->V(0)), Dual->rows(), Dual->cols(), &V(0), &w(0), linearAxon()->beta());
}
void BackLinearAxon::fireMap()
{
	if (gradientsEnabled) {
		register NSFloat *pos=&V(0), *endPos=pos+U(), *weightPos=&w(0), beta=linearAxon()->beta();
		if (m_nWeightingVectorSize) {
			while (pos<endPos)
				*weightPos++ += *pos++ *= beta * *(m_fWeightingVector + m_nCurrentExemplar);
			updateExemplarCounter();
		}
		else {
			while (pos<endPos)
				*weightPos++ += *pos++ *= beta;
		}
	} else {
		register NSFloat *pos=&V(0), *endPos=pos+U(), beta=linearAxon()->beta();
		while (pos<endPos)
			*pos++ *= beta;
	}
}
// Author: Wesley Curtis Lefebvre

void BackSigmoidAxon::fireMap()
{
	if (gradientsEnabled) {
		register NSFloat *pos=&V(0), *endPos=pos+U(), *dualPos=&(Dual->V(0)), *weightPos=&w(0), beta=linearAxon()->beta();
		if (m_nWeightingVectorSize) {
			while (pos<endPos)
				*weightPos++ += *pos++ *= beta * (*dualPos*(1.0f - *dualPos++) + m_fOffset) * *(m_fWeightingVector + m_nCurrentExemplar);
			updateExemplarCounter();
		}
		else {
			while (pos<endPos)
				*weightPos++ += *pos++ *= beta * (*dualPos*(1.0f - *dualPos++) + m_fOffset);
		}
	} else {
		register NSFloat *pos=&V(0), *endPos=pos+U(), *dualPos=&(Dual->V(0)), *weightPos=&w(0), beta=linearAxon()->beta();
		while (pos<endPos)
			*pos++ *= beta * (*dualPos*(1.0f - *dualPos++) + m_fOffset);
	}
	/*      for (register int i=0; i<V(); i++) {
	fireSensitivity(i);
	V(i) *= linearAxon()->beta();
	}
	if (gradientsEnabled)
	for (i=0; i<V(); i++)
	fireGradient(i); */
}
//BackNormalizedSigmoidAxon
void BackNormalizedSigmoidAxon::fireMap()
{
    register NSFloat *pos=&V(0), *endPos=pos+U(), *dualWF=((NormalizedSigmoidAxon *)Dual)->weightingFactorArray();
    while (pos<endPos)
		*pos++ *= *dualWF++;
	BackSigmoidAxon::fireMap();
}
// Author: Wesley Curtis Lefebvre

void BackTanhAxon::fireMap()
{
	if (gradientsEnabled) {
		register NSFloat *pos=&V(0), *endPos=pos+U(), *dualPos=&(Dual->V(0)), *weightPos=&w(0), beta=linearAxon()->beta();
		if (m_nWeightingVectorSize) {
			while (pos<endPos) {
				*weightPos++ += (*pos++ *= beta * (1.0f - *dualPos * *dualPos + m_fOffset)) * *(m_fWeightingVector + m_nCurrentExemplar);
				dualPos++;
			}
			updateExemplarCounter();
		}
		else {
			while (pos<endPos) {
				*weightPos++ += *pos++ *= beta * (1.0f - *dualPos * *dualPos + m_fOffset);
				dualPos++;
			}
		}
	} else {
		register NSFloat *pos=&V(0), *endPos=pos+U(), *dualPos=&(Dual->V(0)), beta=linearAxon()->beta();
		while (pos<endPos) {
			*pos++ *= beta * (1.0f - *dualPos * *dualPos + m_fOffset);
			dualPos++;
		}
	}
}
// Author: Wesley Curtis Lefebvre
BackTDNNAxon::BackTDNNAxon() {
	memoryTaps = 1;
	delayedReference = new Soma;
}
void BackTDNNAxon::activityAltered(void) {
	if (preActivityAccess) {
		preActivityAccess->setData(&V(0), taps(), rows()*cols());
	}
	if (activityAccess)     {
		activityAccess->setData(&V(0), rows(), cols());
	}
}
void BackTDNNAxon::backpropComplete()
{
	BackAxon::backpropComplete();
	register NSFloat *pos=&V(0), *endPos=pos+U()-V(), *delayPos=&Vd(V());
	while (pos<endPos) {
		pos++;
		*delayPos++ = 0.0f;
	}
}
void BackTDNNAxon::constructDLL(void) {
	if (dllStruct->constructor) {
		if (m_pCallingObject)
			dllStruct->instance = (*(BackTDNNAxon_Constructor_CSW)dllStruct->constructor)(dllStruct->instance,Dual->dllStruct? Dual->dllStruct->instance: NULL, Dual->rows(), Dual->cols(), memoryTaps, m_pCallingObject);
		else
			dllStruct->instance = (*(BackTDNNAxon_Constructor)dllStruct->constructor)(dllStruct->instance,Dual->dllStruct? Dual->dllStruct->instance: NULL, Dual->rows(), Dual->cols(), memoryTaps);
	}
}
const char *BackTDNNAxon::dllFunctionName()
{
	return "performBackTDNNAxon";
}
void BackTDNNAxon::fireDLL(void)
{
	if (m_pCallingObject)
        (*(BackTDNNAxon_MAP_CSW)dllStruct->function)(dllStruct->instance, Dual->dllStruct? Dual->dllStruct->instance: NULL, &V(0), Dual->rows(), Dual->cols(), &Vd(0), memoryTaps, &(Dual->V(0)), m_pCallingObject);
	else
        (*(BackTDNNAxon_MAP)dllStruct->function)(dllStruct->instance, Dual->dllStruct? Dual->dllStruct->instance: NULL, &V(0), Dual->rows(), Dual->cols(), &Vd(0), memoryTaps, &(Dual->V(0)));
}
void BackTDNNAxon::fireMap(void) {
	register NSFloat *pos=&V(0), *endPos=pos+U()-V(), *delayPos=&Vd(V());
	while (pos<endPos)
		*pos++ += *delayPos++;
}
void BackTDNNAxon::incrementTime(void) {
	Dual->decrementTime();
	delayedReference->incrementTime();
	Axon::incrementTime();
}
void BackTDNNAxon::initialize(void) {
	BackAxon::initialize();
	delayedReference->attachTo(this, 1);
}
void BackTDNNAxon::loadStateVariables(FILE *file, unsigned short int version)
{
	if (version > 123) {
		int elementCount;
		NSFloat dummy;
		fscanf(file,"%d", &elementCount);
		for (int i=0; i<elementCount; i++)
			fscanf(file,"%f", i<V() ? &Vd(i) : &dummy);
	}
	BackAxon::loadStateVariables(file, version);
}
void BackTDNNAxon::saveStateVariables(FILE *file)
{
    fprintf(file,"%d ", V());
    for (int i=0; i<V(); i++)
		fprintf(file,"%21.15e ", Vd(i));
    fprintf(file,"\n");
    BackAxon::saveStateVariables(file);
}
void BackTDNNAxon::setMemoryTaps(int anInt) {
	memoryTaps = anInt;
	if (memoryTaps < 1)
		memoryTaps = 1;
}
void BackTDNNAxon::zeroState(void) {
	BackAxon::zeroState();
	delayedReference->networkReset();
}
BackTDNNAxon::~BackTDNNAxon(void) {
	delayedReference->attachTo(NULL,0);
	delete delayedReference;
}
// Author: Wesley Curtis Lefebvre

const char *BackGammaAxon::dllFunctionName()
{
	return "performBackGammaAxon";
}
void BackGammaAxon::fireDLL(void)
{
	if (m_pCallingObject)
        (*(BackGammaAxon_MAP_CSW)dllStruct->function)(dllStruct->instance, Dual->dllStruct? Dual->dllStruct->instance: NULL, &V(0), Dual->rows(), Dual->cols(), &Vd(0), memoryTaps, &(Dual->V(0)), &(Dual->w(0)), &w(0), m_pCallingObject);
	else
        (*(BackGammaAxon_MAP)dllStruct->function)(dllStruct->instance, Dual->dllStruct? Dual->dllStruct->instance: NULL, &V(0), Dual->rows(), Dual->cols(), &Vd(0), memoryTaps, &(Dual->V(0)), &(Dual->w(0)), &w(0));
}
void BackGammaAxon::fireMap(void) {
	register NSFloat
		*pos=&V(0),
		*endPos=pos+U()-V(),
		*delayPos=&Vd(V()),
		*tapPos=&V(V()),
		*dualWeightPos=&(Dual->w(0)),
		*endDualWeightPos=dualWeightPos+w();
	if (gradientsEnabled) {
		register NSFloat
			*dualPos=&(Dual->V(0)),
			*dualTapPos=&(Dual->V(V())),
			*weightPos=&w(0);
		if (m_nWeightingVectorSize) {
			while (pos<endPos) {
				while (dualWeightPos<endDualWeightPos) {
					*pos++ += *dualWeightPos * *delayPos;
					*tapPos++ += (1.0f - *dualWeightPos++) * *delayPos;
					*weightPos++ += *delayPos++ * (*dualPos++ - *dualTapPos++) * *(m_fWeightingVector + m_nCurrentExemplar);
				}
				weightPos = &w(0);
				dualWeightPos = &(Dual->w(0));
			}
			updateExemplarCounter();
		}
		else {
			while (pos<endPos) {
				while (dualWeightPos<endDualWeightPos) {
					*pos++ += *dualWeightPos * *delayPos;
					*tapPos++ += (1.0f - *dualWeightPos++) * *delayPos;
					*weightPos++ += *delayPos++ * (*dualPos++ - *dualTapPos++);
				}
				weightPos = &w(0);
				dualWeightPos = &(Dual->w(0));
			}
		}
	} else
		while (pos<endPos) {
			while (dualWeightPos<endDualWeightPos) {
				*pos++ += *dualWeightPos * *delayPos;
				*tapPos++ += (1.0f - *dualWeightPos++) * *delayPos++;
			}
			dualWeightPos = &(Dual->w(0));
		}
}
void BackGammaAxon::initialize(void) {
	allocateLinks(V());
	BackTDNNAxon::initialize();
}

// Author: Wesley Curtis Lefebvre
// Name:        Connection.m

Connection::Connection() {
	fromIndex = -1;
	toCount = toAllocated = 0;
	toIndices = NULL;
}
Connection::~Connection() {
	free((char*)toIndices);
	toIndices = NULL;
}
void Connection::addConnection(int toIndex) {
	if ((toCount < toAllocated) && (toIndices[toCount-1] < toIndex)) {
		toIndices[toCount++] = toIndex;
		return;
	}
	for (register int i=toCount-1; i>=0; i--) {
		if (toIndices[i] < toIndex) {
			insertConnectionAt(toIndex, i+1);
			return;
		}
		if (toIndices[i] == toIndex)
			return;
	}
	insertConnectionAt(toIndex, 0);
}
void Connection::allocateConnections(int number) {
	toAllocated = number;
	toIndices = (int *)calloc(toAllocated, sizeof(int));
}
void Connection::empty(void) {
	toCount = 0;
}
void Connection::insertConnectionAt(int toIndex, int index) {
	int *tempIndices = toIndices;
	if (toCount >= toAllocated) {
		allocateConnections(toAllocated? toAllocated * 2 :128);
		for (register int i=0; i<index; i++)
			toIndices[i] = tempIndices[i];
	}
	for (register int i=toCount++; i>index; i--)
		toIndices[i] = tempIndices[i-1];
	toIndices[index] = toIndex;
	if (tempIndices && (tempIndices != toIndices))
		free((char*)tempIndices);
}
void Connection::removeConnection(int toIndex) {
	BOOL found=NO;
	for (int i=0; i<toCount; i++) {
		if (toIndices[i] == toIndex) {
			found = YES;
			toCount--;
		}
		if (found)
			toIndices[i] = toIndices[i+1];
	}
	if (found)
		toIndices[toCount] = -1;
}
Connection &Connection::operator=(Connection &aConnection) {
	fromIndex = aConnection.fromIndex;
	toCount = aConnection.toCount;
	toAllocated = aConnection.toAllocated;
	if (toIndices)
		free((char*)toIndices);
	toIndices = (int *)calloc(toAllocated, sizeof(int));
	for (int i=0; i<toCount; i++)
		toIndices[i] = aConnection.toIndices[i];
	return *this;
}

// Author: Wesley Curtis Lefebvre

ConnectionList::ConnectionList() {
	synapseCount = synapseAllocated = 0;
	synapse = NULL;
}
ConnectionList::~ConnectionList() {
	for (int i=0; i<synapseAllocated; i++)
		delete synapse[i];
	free((char*)synapse);
	synapse = NULL;
}
void ConnectionList::addConnection(int fromIndex, int toIndex) {
	for (register int i=synapseCount-1; i>=0; i--) {
		if (synapse[i]->feedingIndex() == fromIndex) {
			synapse[i]->addConnection(toIndex);
			return;
		}
		if (synapse[i]->feedingIndex() < fromIndex) {
			insertConnectionAt(fromIndex, toIndex, i+1);
			return;
		}
	}
	insertConnectionAt(fromIndex, toIndex, 0);
}
void ConnectionList::allocateSynapse(int number) {
	synapseAllocated = number;
	synapse = (Connection**)calloc(synapseAllocated, sizeof(Connection*));
	for(register int i=0; i<synapseAllocated; i++)
		synapse[i] = new Connection;
}
void ConnectionList::empty(void) {
	synapseCount = 0;
}
void ConnectionList::insertConnectionAt(int fromIndex, int toIndex, int index) {
	Connection **tempSynapse = synapse;
	int oldAllocated = synapseAllocated;
	register int i;
	if (synapseCount >= synapseAllocated) {
		allocateSynapse(synapseAllocated? synapseAllocated * 2 :128);
		for (i=0; i<index; i++)
			*synapse[i] = *tempSynapse[i];
	}
	for (i=synapseCount++; i>index; i--)
		*synapse[i] = *tempSynapse[i-1];
	synapse[index]->setIndex(fromIndex);
	synapse[index]->empty();
	synapse[index]->addConnection(toIndex);
	if (tempSynapse && (tempSynapse != synapse))    {
		for (i=0; i<oldAllocated; i++)
			delete tempSynapse[i];
		free((char*)tempSynapse);
	}
}
void ConnectionList::removeConnection(int fromIndex, int toIndex) {
	BOOL foundAndEmpty=NO;
	for (int i=0; i<synapseCount; i++) {
		if (synapse[i]->feedingIndex() == fromIndex) {
			synapse[i]->removeConnection(toIndex);
			if (foundAndEmpty = synapse[i]->count()? NO :YES)
				synapseCount--;
		}
		if (foundAndEmpty)
			*synapse[i] = *synapse[i+1];
	}
	if (foundAndEmpty)
		synapse[synapseCount]->empty();
}
int ConnectionList::weights(void) {
	int total, i;
	for (total=0, i=0; i<synapseCount; i++)
		total += synapse[i]->count();
	return total;
}
// Author: Wesley Curtis Lefebvre

void BackLaguarreAxon::fireMap(void) {
    if (gradientsEnabled) {
        if (m_nWeightingVectorSize) {
			for (register int i=0,j,k; i<V(); i++) {
				NSFloat gain = (NSFloat)sqrt(1-(Dual->w(i)*Dual->w(i)));
				for (j=memoryTaps-1; j>=0; j--) {
					k = i + j*V();
					if (j==0) {
						V(k) += Dual->w(i)*Vd(k)/gain;
						V(k) *= gain;
						w(i) += (Dual->V(k)*Vd(k)/gain) * *(m_fWeightingVector + m_nCurrentExemplar);
					}
					else {
						V(k) += Dual->w(i)*Vd(k);
						V(k-V()) += Vd(k) - Dual->w(i)*V(k);
						w(i) += (Vd(k)*Dual->V(k) - V(k)*Dual->V(k-V())) * *(m_fWeightingVector + m_nCurrentExemplar);
					}
				}
			}
            updateExemplarCounter();
        }
        else {
			for (register int i=0,j,k; i<V(); i++) {
				NSFloat gain = (NSFloat)sqrt(1-(Dual->w(i)*Dual->w(i)));
				for (j=memoryTaps-1; j>=0; j--) {
					k = i + j*V();
					if (j==0) {
						V(k) += Dual->w(i)*Vd(k)/gain;
						V(k) *= gain;
						w(i) += Dual->V(k)*Vd(k)/gain;
					}
					else {
						V(k) += Dual->w(i)*Vd(k);
						V(k-V()) += Vd(k) - Dual->w(i)*V(k);
						w(i) += Vd(k)*Dual->V(k) - V(k)*Dual->V(k-V());
					}
				}
			}
		}
    }
    else {
        for (register int i=0,j,k; i<V(); i++) {
            NSFloat gain = (NSFloat)sqrt(1-(Dual->w(i)*Dual->w(i)));
            for (j=memoryTaps-1; j>=0; j--) {
                k = i + j*V();
                if (j==0) {
					V(k) += Dual->w(i)*Vd(k)/gain;
					V(k) *= gain;
                }
                else {
					V(k) += Dual->w(i)*Vd(k);
					V(k-V()) += Vd(k) - Dual->w(i)*V(k);
                }
            }
        }
    }
}
void BackLaguarreAxon::initialize(void) {
	allocateLinks(V());
	BackTDNNAxon::initialize();
}

// Author: Wesley Curtis Lefebvre

BackContextAxon::BackContextAxon() {
	delayedReference = new Soma;
	m_fOffset = 0.1f;
}
const char *BackContextAxon::dllFunctionName()
{
	return "performBackContextAxon";
}
void BackContextAxon::fireDLL(void)
{
	if (m_pCallingObject)
        (*(BackContextAxon_MAP_CSW)dllStruct->function)(dllStruct->instance, Dual->dllStruct? Dual->dllStruct->instance: NULL, &V(0), Dual->rows(), Dual->cols(), &Vd(0), &Vd(0), &(Dual->w(0)), linearAxon()->beta(), &w(0), m_pCallingObject);
	else
        (*(BackContextAxon_MAP)dllStruct->function)(dllStruct->instance, Dual->dllStruct? Dual->dllStruct->instance: NULL, &V(0), Dual->rows(), Dual->cols(), &Vd(0), &Vd(0), &(Dual->w(0)), linearAxon()->beta(), &w(0));
}
void BackContextAxon::fireMap(void) {
	register NSFloat
		*pos=&V(0),
		*endPos=pos+V(),
		*delayPos=&Vd(0),
		*weightPos=&(Dual->w(0)),
		beta=linearAxon()->beta();
	if (gradientsEnabled) {
		register NSFloat *gradPos=&w(0), *dualPos=&(Dual->V(0));
		if (m_nWeightingVectorSize) {
			while (pos<endPos) {
				*pos++ = beta * (*pos + *weightPos++ * *delayPos);
				*gradPos++ += *delayPos++ * beta * *dualPos++ * *(m_fWeightingVector + m_nCurrentExemplar);
			}
			updateExemplarCounter();
		}
		else {
			while (pos<endPos) {
				*pos++ = beta * (*pos + *weightPos++ * *delayPos);
				*gradPos++ += *delayPos++ * beta * *dualPos++;
			}
		}
	} else
		while (pos<endPos)
			*pos++ = beta * (*pos + *weightPos++ * *delayPos++);
}
void BackContextAxon::incrementTime(void) {
	Dual->decrementTime();
	delayedReference->incrementTime();
	Axon::incrementTime();
}
void BackContextAxon::initialize(void) {
	BackAxon::initialize();
	allocateLinks(V());
	delayedReference->attachTo(this, 1);
}
void BackContextAxon::loadStateVariables(FILE *file, unsigned short int version)
{
	if (version > 123) {
		int elementCount;
		NSFloat dummy;
		fscanf(file,"%d", &elementCount);
		for (int i=0; i<elementCount; i++)
			fscanf(file,"%f", i<V() ? &Vd(i) : &dummy);
	}
	BackLinearAxon::loadStateVariables(file, version);
}
void BackContextAxon::saveStateVariables(FILE *file)
{
    fprintf(file,"%d ", V());
    for (int i=0; i<V(); i++)
        fprintf(file,"%21.15e ", Vd(i));
    fprintf(file,"\n");
    BackLinearAxon::saveStateVariables(file);
}
void BackContextAxon::zeroState(void) {
	BackAxon::zeroState();
	delayedReference->networkReset();
}
BackContextAxon::~BackContextAxon(void) {
	delayedReference->attachTo(NULL,0);
	delete delayedReference;
}
// Author: Wesley Curtis Lefebvre

void BackSigmoidContextAxon::fireMap() {
	register NSFloat
		*pos=&V(0),
		*endPos=pos+V(),
		*delayPos=&Vd(0),
		*weightPos=&(Dual->w(0)),
		*dualPos=&(Dual->V(0)),
		beta=linearAxon()->beta();
	if (gradientsEnabled) {
		register NSFloat *gradPos=&w(0), *dualPos=&(Dual->V(0));
		if (m_nWeightingVectorSize) {
			while (pos<endPos) {
				*pos *= *dualPos * (1.0f - *dualPos) + m_fOffset;
				*pos++ = beta * (*pos + *weightPos++ * *delayPos);
				*gradPos++ += *delayPos++ * beta * *dualPos++ * *(m_fWeightingVector + m_nCurrentExemplar);
			}
			updateExemplarCounter();
		}
		else {
			while (pos<endPos) {
				*pos *= *dualPos * (1.0f - *dualPos) + m_fOffset;
				*pos++ = beta * (*pos + *weightPos++ * *delayPos);
				*gradPos++ += *delayPos++ * beta * *dualPos++;
			}
		}
	} else
		while (pos<endPos) {
			*pos *= *dualPos * (1.0f - *dualPos) + m_fOffset;
			*pos++ = beta * (*pos + *weightPos++ * *delayPos++);
			dualPos++;
		}
}

// Author: Wesley Curtis Lefebvre

void BackTanhContextAxon::fireMap() {
	register NSFloat
		*pos=&V(0),
		*endPos=pos+V(),
		*delayPos=&Vd(0),
		*weightPos=&(Dual->w(0)),
		*dualPos=&(Dual->V(0)),
		beta=linearAxon()->beta();
	if (gradientsEnabled) {
		register NSFloat *gradPos=&w(0);
		if (m_nWeightingVectorSize) {
			while (pos<endPos) {
				*pos *= 1.0f - *dualPos * *dualPos + m_fOffset;
				*pos++ = beta * (*pos + *weightPos++ * *delayPos);
				*gradPos++ += *delayPos++ * beta * *dualPos++ * *(m_fWeightingVector + m_nCurrentExemplar);
			}
			updateExemplarCounter();
		}
		else {
			while (pos<endPos) {
				*pos *= 1.0f - *dualPos * *dualPos + m_fOffset;
				*pos++ = beta * (*pos + *weightPos++ * *delayPos);
				*gradPos++ += *delayPos++ * beta * *dualPos++;
			}
		}
	} else
		while (pos<endPos) {
			*pos *= 1.0f - *dualPos * *dualPos + m_fOffset;
			*pos++ = beta * (*pos + *weightPos++ * *delayPos++);
			dualPos++;
		}
}
// Author: Wesley Curtis Lefebvre

void BackIntegratorAxon::fireMap(void) {
	register NSFloat
		*pos=&V(0),
		*endPos=pos+V(),
		*delayPos=&Vd(0),
		*weightPos=&(Dual->w(0)),
		beta=linearAxon()->beta();
	if (gradientsEnabled) {
		register NSFloat *gradPos=&w(0), *dualPos=&(Dual->V(0));
		if (m_nWeightingVectorSize) {
			while (pos<endPos) {
				*pos++ = beta * (*pos * (1.0f-*weightPos) + *weightPos++ * *delayPos);
				*gradPos++ += *delayPos++ * beta * *dualPos++ * *(m_fWeightingVector + m_nCurrentExemplar);
			}
			updateExemplarCounter();
		}
		else {
			while (pos<endPos) {
				*pos++ = beta * (*pos * (1.0f-*weightPos) + *weightPos++ * *delayPos);
				*gradPos++ += *delayPos++ * beta * *dualPos++;
			}
		}
	} else
		while (pos<endPos)
			*pos++ = beta * (*pos + *weightPos++ * *delayPos++);
}

// Author: Wesley Curtis Lefebvre

void BackSigmoidIntegratorAxon::fireMap() {
	register NSFloat
		*pos=&V(0),
		*endPos=pos+V(),
		*delayPos=&Vd(0),
		*weightPos=&(Dual->w(0)),
		*dualPos=&(Dual->V(0)),
		beta=linearAxon()->beta();
	if (gradientsEnabled) {
		register NSFloat *gradPos=&w(0);
		if (m_nWeightingVectorSize) {
			while (pos<endPos) {
				*pos *= *dualPos * (1.0f - *dualPos) + m_fOffset;
				*pos++ = beta * (*pos * (1.0f-*weightPos) + *weightPos++ * *delayPos);
				*gradPos++ += *delayPos++ * beta * *dualPos++ * *(m_fWeightingVector + m_nCurrentExemplar);
			}
			updateExemplarCounter();
		}
		else {
			while (pos<endPos) {
				*pos *= *dualPos * (1.0f - *dualPos) + m_fOffset;
				*pos++ = beta * (*pos * (1.0f-*weightPos) + *weightPos++ * *delayPos);
				*gradPos++ += *delayPos++ * beta * *dualPos++;
			}
		}
	} else
		while (pos<endPos) {
			*pos *= *dualPos * (1.0f - *dualPos) + m_fOffset;
			*pos++ = beta * (*pos * (1.0f-*weightPos) + *weightPos++ * *delayPos++);
			dualPos++;
		}
}

// Author: Wesley Curtis Lefebvre

void BackTanhIntegratorAxon::fireMap() {
	register NSFloat
		*pos=&V(0),
		*endPos=pos+V(),
		*delayPos=&Vd(0),
		*weightPos=&(Dual->w(0)),
		*dualPos=&(Dual->V(0)),
		beta=linearAxon()->beta();
	if (gradientsEnabled) {
		register NSFloat *gradPos=&w(0);
		if (m_nWeightingVectorSize) {
			while (pos<endPos) {
				*pos *= 1.0f - *dualPos * *dualPos + m_fOffset;
				*pos++ = beta * (*pos * (1.0f-*weightPos) + *weightPos++ * *delayPos);
				*gradPos++ += *delayPos++ * beta * *dualPos++ * *(m_fWeightingVector + m_nCurrentExemplar);
			}
			updateExemplarCounter();
		}
		else {
			while (pos<endPos) {
				*pos *= 1.0f - *dualPos * *dualPos + m_fOffset;
				*pos++ = beta * (*pos * (1.0f-*weightPos) + *weightPos++ * *delayPos);
				*gradPos++ += *delayPos++ * beta * *dualPos++;
			}
		}
	} else
		while (pos<endPos) {
			*pos *= 1.0f - *dualPos * *dualPos + m_fOffset;
			*pos++ = beta * (*pos * (1.0f-*weightPos) + *weightPos++ * *delayPos++);
			dualPos++;
		}
}

// Author: Wesley Curtis Lefebvre

BackSynapse::BackSynapse()
{
	Dual = NULL;
	gradientsEnabled = YES;
	m_fWeightingVector = NULL;
	m_nWeightingVectorSize = 0;
	m_nCurrentExemplar = 0;
}
void BackSynapse::activityAltered(void) {
	Synapse::activityAltered();
	//      if (activityAccess)
	//wclnow                activityAccess->setSpatialName("Activity Gradients");
}
void BackSynapse::constructDLL(void) {
	if (Dual && dllStruct->constructor && Last && Feed) {
		int lrows = ((BackAxon *)Last)->Dual ? ((BackAxon *)Last)->Dual->rows() : ((BackAxon *)Last)->rows();
		int lcols = ((BackAxon *)Last)->Dual ? ((BackAxon *)Last)->Dual->cols() : ((BackAxon *)Last)->cols();
		int frows = ((BackAxon *)Feed)->Dual ? ((BackAxon *)Feed)->Dual->rows() : ((BackAxon *)Feed)->rows();
		int fcols = ((BackAxon *)Feed)->Dual ? ((BackAxon *)Feed)->Dual->cols() : ((BackAxon *)Feed)->cols();
		if (lrows && lcols && frows && fcols) {
			if (m_pCallingObject)
                dllStruct->instance = (*(BackSynapse_Constructor_CSW)dllStruct->constructor)(dllStruct->instance, Dual->dllStruct? Dual->dllStruct->instance: NULL, lrows, lcols, frows, fcols, m_pCallingObject);
			else
                dllStruct->instance = (*(BackSynapse_Constructor)dllStruct->constructor)(dllStruct->instance, Dual->dllStruct? Dual->dllStruct->instance: NULL, lrows, lcols, frows, fcols);
		}
	}
}
const char *BackSynapse::dllFunctionName()
{
	return "performBackSynapse";
}
void BackSynapse::fireDLL(void)
{
	if (m_pCallingObject)
		(*(BackSynapse_MAP_CSW)dllStruct->function)(dllStruct->instance, Dual->dllStruct? Dual->dllStruct->instance: NULL, &V(0), ((BackAxon *)Last)->Dual->rows(), ((BackAxon *)Last)->Dual->cols(), &(Feed->V(0)), ((BackAxon *)Feed)->Dual->rows(), ((BackAxon *)Feed)->Dual->cols(), &(Dual->V(0)), m_pCallingObject);
	else
		(*(BackSynapse_MAP)dllStruct->function)(dllStruct->instance, Dual->dllStruct? Dual->dllStruct->instance: NULL, &V(0), ((BackAxon *)Last)->Dual->rows(), ((BackAxon *)Last)->Dual->cols(), &(Feed->V(0)), ((BackAxon *)Feed)->Dual->rows(), ((BackAxon *)Feed)->Dual->cols(), &(Dual->V(0)));
}
void BackSynapse::linksAltered(void) {
	Synapse::linksAltered();
	//      if (linkAccess)
	//wclnow                linkAccess->setSpatialName("Weight Gradients");
}
void BackSynapse::backpropComplete()
{
	Dual->recallBufPosition();
}
void BackSynapse::backpropStarting()
{
	Dual->storeBufPosition();
	Dual->decrementTime();
}
void BackSynapse::fire()
{
	Synapse::fire();
	if (linkAccess)
		linkAccess->reportSpatialAccess();
}
void BackSynapse::incrementTime()
{
	if (gradientsEnabled)
		Dual->decrementTime();
	Synapse::incrementTime();
}
void BackSynapse::networkRandomize()
{
	Synapse::networkRandomize();
	register NSFloat *pos=&w(0), *endPos=pos+W();
	if (pos)
		while (pos<endPos)
			*pos++ = 0.0f;
		//      for (int i=0; i<links; i++)
		//              w(i) = (NSFloat)0;
        if (linkAccess)
			linkAccess->reportSpatialAccess();
        if (dllLinkAccess)
			dllLinkAccess->reportSpatialAccess();
}
void BackSynapse::setCurrentExemplar(int exemplar)
{
	m_nCurrentExemplar = exemplar;
}
void BackSynapse::setDual(Soma *aSoma)
{
	Dual = (Synapse*)aSoma;
	if (Dual)
		setTemporalDimension(Dual->temporalDimension());
}
void BackSynapse::setGradientsEnabled(BOOL aFlag)
{
	gradientsEnabled = aFlag;
}
void BackSynapse::setWeightingVector(NSFloat *aVector, int vectorSize)
{
	m_fWeightingVector = aVector;
	m_nWeightingVectorSize = vectorSize;
}
void BackSynapse::updateExemplarCounter()
{
	m_nCurrentExemplar++;
	if (m_nCurrentExemplar >= m_nWeightingVectorSize)
		m_nCurrentExemplar = 0;
}
// Author: Wesley Curtis Lefebvre

void BackFullSynapse::allocateSynapticLinks()
{
	allocateLinks(U()*V());
	BackSynapse::allocateSynapticLinks();
}
const char *BackFullSynapse::dllFunctionName()
{
	return "performBackFullSynapse";
}
void BackFullSynapse::fireDLL(void)
{
	if (m_pCallingObject)
		(*(BackFullSynapse_MAP_CSW)dllStruct->function)(dllStruct->instance, Dual->dllStruct? Dual->dllStruct->instance: NULL, &V(0), ((BackAxon *)Last)->Dual->rows(), ((BackAxon *)Last)->Dual->cols(), &(Feed->V(0)), ((BackAxon *)Feed)->Dual->rows(), ((BackAxon *)Feed)->Dual->cols(), &(Dual->V(0)), &(Dual->w(0)), &w(0), m_pCallingObject);
	else
		(*(BackFullSynapse_MAP)dllStruct->function)(dllStruct->instance, Dual->dllStruct? Dual->dllStruct->instance: NULL, &V(0), ((BackAxon *)Last)->Dual->rows(), ((BackAxon *)Last)->Dual->cols(), &(Feed->V(0)), ((BackAxon *)Feed)->Dual->rows(), ((BackAxon *)Feed)->Dual->cols(), &(Dual->V(0)), &(Dual->w(0)), &w(0));
}
void BackFullSynapse::fireFeed()
{
	if (gradientsEnabled) {
		register NSFloat
			*pos=&V(0),
			*endPos=pos+V(),
			*dualPos,
			*feedPos=&(Feed->V(0)),
			*endFeedPos=feedPos+U(),
			*weightPos=&w(0),
			*dualWeightPos=&(Dual->w(0));
		if (m_nWeightingVectorSize) {
			while (pos<endPos) {
				dualPos=&(Dual->V(0));
				feedPos=&(Feed->V(0));
				while (feedPos<endFeedPos) {
					*feedPos++ += *dualWeightPos++ * *pos;
					*weightPos++ += *pos * *dualPos++ * *(m_fWeightingVector + m_nCurrentExemplar);
				}
				pos++;
			}
			updateExemplarCounter();
		}
		else {
			while (pos<endPos) {
				dualPos=&(Dual->V(0));
				feedPos=&(Feed->V(0));
				while (feedPos<endFeedPos) {
					*feedPos++ += *dualWeightPos++ * *pos;
					*weightPos++ += *pos * *dualPos++;
				}
				pos++;
			}
		}
	} else {
		register NSFloat
			*pos=&V(0),
			*endPos=pos+V(),
			*feedPos=&(Feed->V(0)),
			*endFeedPos=feedPos+U(),
			*dualWeightPos=&(Dual->w(0));
		while (pos<endPos) {
			feedPos=&(Feed->V(0));
			while (feedPos<endFeedPos)
				*feedPos++ += *dualWeightPos++ * *pos;
			pos++;
		}
	}
	/*      for (register int i=0,j; i<U(); i++)
	for (j=0; j<V(); j++)
	Feed->V(i) += W(i,j)*V(j);
	if (gradientsEnabled)
	for (i=0; i<U(); i++)
	for (j=0; j<V(); j++)
	w(i+j*U()) += V(j)*Dual->V(i);  */
}
void BackFullSynapse::linksAltered() {
	BackSynapse::linksAltered();
	if (linkAccess)
		linkAccess->setData(&w(0), V(), U());
}

// Author: Wesley Curtis Lefebvre

const char *BackArbitrarySynapse::dllFunctionName()
{
	return NULL;
}
void BackArbitrarySynapse::allocateSynapticLinks(void) {
	connections = Dual? ((ArbitrarySynapse *)(void *)Dual)->connectionList() :NULL;
	int weights = connections? connections->weights() :0;
	allocateLinks(weights);
}
void BackArbitrarySynapse::fireFeed(void) {
	register int i,j,k;
	for (i=0,j,k=0; i<connections->count(); i++) {
		Connection *aConnection = connections->connectionAt(i);
		int *indices = aConnection->connections();
		for (j=0; j<aConnection->count(); j++,k++)
			Feed->V(aConnection->feedingIndex()) += Dual->w(k)*V(indices[j]);
	}
	if (gradientsEnabled) {
		if (m_nWeightingVectorSize) {
			for (i=0,j,k=0; i<connections->count(); i++) {
				Connection *aConnection = connections->connectionAt(i);
				int *indices = aConnection->connections();
				for (j=0; j<aConnection->count(); j++,k++)
					w(k) += V(indices[j]) * Dual->V(aConnection->feedingIndex()) * *(m_fWeightingVector + m_nCurrentExemplar);;
			}
			updateExemplarCounter();
		}
		else {
			for (i=0,j,k=0; i<connections->count(); i++) {
				Connection *aConnection = connections->connectionAt(i);
				int *indices = aConnection->connections();
				for (j=0; j<aConnection->count(); j++,k++)
					w(k) += V(indices[j]) * Dual->V(aConnection->feedingIndex());
			}
		}
	}
}
void BackArbitrarySynapse::setDual(Soma *aSoma) {
	BackSynapse::setDual(aSoma);
	connections = Dual? ((ArbitrarySynapse *)(void *)Dual)->connectionList() :NULL;
}

// Author: Wesley Curtis Lefebvre

BackCriteriaControl::BackCriteriaControl() {
	streamFixed = NO;
}
const char *BackCriteriaControl::dllFunctionName()
{
	return NULL;
}
void BackCriteriaControl::fireMap(void) {
	register NSFloat *pos=&V(0), *endPos=pos+V(), *dualPos=&(Dual->V(0));
	while (pos<endPos)
		*pos++ = *dualPos++;
	//      for (int i=0; i<V(); i++)
	//              V(i) = Dual->V(i);
}
void BackCriteriaControl::incrementTime(void) {
	if (streamFixed)
		Axon::incrementTime();
	else
		BackAxon::incrementTime();
}
// Author: Wesley Curtis Lefebvre

Criterion::Criterion()
{
    tempE = E = runningE = m_fRunningMSE = m_fPercentError = 0.0f;
    tempCounter = 0;
    testing = m_bSensitivity = NO;
    costCounter = m_iSensitivityCounter = 0;
    costAccess = runningCostAccess = desiredAccess = performanceMeasuresAccess = correlationAccess = NULL;
    confusionAccess = confusionAsPercentAccess = rOCAccess = NULL;
    m_pSensitivity = m_pRawSensitivity = m_pSensitivityTotals = NULL;
    m_pFixedOutput = new SpatialData;
    desired = NULL;
    m_dfSumXY = NULL;
    m_dfSumX = NULL;
    m_dfSumY = NULL;
    m_dfSumXX = NULL;
    m_dfSumYY = NULL;
    m_dfTempSumXY = NULL;
    m_dfTempSumX = NULL;
    m_dfTempSumY = NULL;
    m_dfTempSumXX = NULL;
    m_dfTempSumYY = NULL;
    m_fCorrelation = NULL;
	m_nTempROCCounters = NULL;
	m_fROC = NULL;
	m_nROCCounters = NULL;
	m_fTempConfusion = NULL;
	m_fConfusion = NULL;
	m_fPerformanceMeasures = NULL;
	m_fConfusionAsPercent = NULL;
    m_fWeightingVector = NULL;
	m_fOffsetVector = NULL;
	m_nWeightingVectorSize = 0;
	m_nCurrentExemplar = 0;
	m_bSensitivityDataLocal = FALSE;
	m_bRawSensitivityDataLocal = FALSE;
	m_bSensitivityTotalsDataLocal = FALSE;
	m_bAutoSave = FALSE;
	m_iSaveCounter = 0;
	m_nTotalNetworkWeights = 0;
	m_iCheckCostEvery = 25;
	m_bOnIncrease = TRUE;
	m_fLastReportedTestBestCost = 1e9f;
	m_fLastReportedTrainBestCost = 1e9f;
	m_fLastCost = 1e9f;
	m_fLastUncheckedCost = 1e9f;
	m_fBestCost = 1e9f;
	m_fCorrelationAverage = 0.0f;
	m_fNMSE = 0.0f;
	m_fAIC = 0.0f;
	m_fMDL = 0.0f;
    m_fLastReportedTrainCost = 0.0f;
    m_fLastReportedTestCost = 0.0f;
	m_bTrainBestCostUpdated = FALSE;
	m_bIgnoreNextError = FALSE;
	m_fConfusionThreshold = 0.0f;
	m_nROCThresholds = 100;
	m_nROCChannel = 0;
	m_nROCMinValue = -1.0f;
	m_nROCMaxValue = 1.0f;
}
Criterion::~Criterion()
{
    if (desired)
		free((char*)desired);
    if (m_dfSumXY)
		free((char*)m_dfSumXY);
    if (m_dfSumX)
		free((char*)m_dfSumX);
    if (m_dfSumY)
		free((char*)m_dfSumY);
    if (m_dfSumXX)
		free((char*)m_dfSumXX);
    if (m_dfSumYY)
		free((char*)m_dfSumYY);
    if (m_dfTempSumXY)
		free((char*)m_dfTempSumXY);
    if (m_dfTempSumX)
		free((char*)m_dfTempSumX);
    if (m_dfTempSumY)
		free((char*)m_dfTempSumY);
    if (m_dfTempSumXX)
		free((char*)m_dfTempSumXX);
    if (m_dfTempSumYY)
		free((char*)m_dfTempSumYY);
    if (m_fCorrelation)
		free((char*)m_fCorrelation);
    if (m_nTempROCCounters)
		free((char*)m_nTempROCCounters);
    if (m_fROC)
		free((char*)m_fROC);
    if (m_nROCCounters)
		free((char*)m_nROCCounters);
	if (m_fTempConfusion)
		free((char*)m_fTempConfusion);
    if (m_fConfusion)
		free((char*)m_fConfusion);
    if (m_fPerformanceMeasures)
		free((char*)m_fPerformanceMeasures);
    if (m_fConfusionAsPercent)
		free((char*)m_fConfusionAsPercent);
    free(m_pFixedOutput);
    if (m_pSensitivity && m_bSensitivityDataLocal)
		delete m_pSensitivity;
    if (m_pRawSensitivity && m_bRawSensitivityDataLocal)
		delete m_pRawSensitivity;
    if (m_pSensitivityTotals && m_bSensitivityTotalsDataLocal)
		delete m_pSensitivityTotals;
}
BOOL Criterion::checkForSaveBest(BOOL trainTest, NSFloat cost, BOOL force)
{
	return checkForSaveBest(cost,force);
}
BOOL Criterion::checkForSaveBest(NSFloat cost, BOOL force)
{
	BOOL returnValue = FALSE;
	BOOL check = m_bAutoSave && !isTesting();
	if (check) {
		BOOL increase = FALSE;
		check = ++m_iSaveCounter >= m_iCheckCostEvery;
		if (!check && m_bOnIncrease)
			check = increase = (cost >= m_fLastCost);
		check = check || force;
		m_fLastUncheckedCost = !check ? cost : 1e9f;
		m_fLastCost = cost;
		if (check) {
			if ((cost < m_fBestCost) && (cost > 1e-20f)) {
				m_fBestCost = cost;
				returnValue = TRUE;
			}
			if (!increase)
				m_iSaveCounter = 0;
		}
	}
	return returnValue;
}
#define confusionMatrix(i,j)			m_fConfusion[j+i*m_nConfusionRows]
#define confusionAsPercentMatrix(i,j)	m_fConfusionAsPercent[j+i*m_nConfusionRows]
#define ROCMatrix(i,j)					m_fROC[j+i*3]
#define ROCCountersMatrix(i,j)			m_nROCCounters[j+i*4]
#define TempROCCountersMatrix(i,j)		m_nTempROCCounters[j+i*4]
void Criterion::computePerformanceStats()
{
	if (correlationAccess || performanceMeasuresAccess) {
		register double *yyPos = m_dfSumYY, *yPos = m_dfSumY, *xPos = m_dfSumX, *xyPos = m_dfSumXY, *xxPos = m_dfSumXX;
        register NSFloat *offset = m_fOffsetVector, *pos=desired, *endPos=pos+V(), *activityPos=&V(0);
        while (pos<endPos) {
            register NSFloat x = *pos - *activityPos;
            m_fRunningMSE += x * x;
			NSFloat myDesired = *pos - (offset ? *offset++ : 0.0f);
			if (myDesired != 0)
				m_fPercentError += ((NSFloat)fabs(x / myDesired)) * 100.0f;
            *xPos++ += *pos;
            *yPos++ += *activityPos;
            *xyPos++ += *pos * *activityPos;
            *xxPos++ += *pos * *pos;
            *yyPos++ += *activityPos * *activityPos;
			pos++;
			activityPos++;
		}
	}
	if (rOCAccess) {
		int currentOutputClass;
		int currentDesiredClass;
		NSFloat increment = (m_nROCMaxValue - m_nROCMinValue) / (NSFloat)(m_nROCThresholds-1);
		NSFloat desiredThresh = ((m_nROCMaxValue - m_nROCMinValue) / 2) + m_nROCMinValue;
		NSFloat curThreshold = m_nROCMinValue;
		int curIndex = 0;
		while (curIndex < m_nROCThresholds) {
			currentOutputClass = (V(m_nROCChannel) < curThreshold) ? 0 : 1;
			currentDesiredClass = (desired[m_nROCChannel] <= desiredThresh) ? 0 : 1;
			ROCCountersMatrix(curIndex,((currentDesiredClass == 1) ? 2 : 3))++;
			if (currentOutputClass == 1)
				ROCCountersMatrix(curIndex,((currentDesiredClass == 1) ? 0 : 1))++;
			curIndex++;
			curThreshold += increment;
		}
	}
	if (confusionAccess || confusionAsPercentAccess) {
		int currentOutputClass;
		int currentDesiredClass;
		if (V() == 1) {
			currentOutputClass = (V(0) < m_fConfusionThreshold) ? 0 : 1;
			currentDesiredClass = (desired[0] < m_fConfusionThreshold) ? 0 : 1;
		}
		else {
			currentOutputClass = 0;
            int i;
			for (i=1; i<V(); i++)
				if (V(i) > V(currentOutputClass))
					currentOutputClass = i;
				currentDesiredClass = 0;
				for (i=1; i<V(); i++)
					if (desired[i] > desired[currentDesiredClass])
						currentDesiredClass = i;
		}
		confusionMatrix(currentDesiredClass, currentOutputClass) += 1;
	}
}
void Criterion::determineCost()
{
    if (m_nWeightingVectorSize && !testing)
        updateWeightedCost();
    E += runningE;
    runningE /= V();
    if (runningCostAccess)
        runningCostAccess->reportSpatialAccess();
    runningE = 0.0f;
}
const char *Criterion::dllFunctionName()
{
	return "performCriterion";
}
void Criterion::epochEnded()
{
	Axon::epochEnded();
	tempE = 0.0f;
    tempCounter = 0;
}
void Criterion::fireDLL(void)
{
    if (sensitivity())
        fireSensitivity();
    else {
        if (desiredAccess)
            desiredAccess->reportSpatialAccess();
		computePerformanceStats();
		if (m_pCallingObject)
			runningE = (*(Criterion_MAP_CSW)dllStruct->function)(dllStruct->instance, &V(0), m_iRows, m_iCols, &V(0), desired, testing, m_pCallingObject);
		else
			runningE = (*(Criterion_MAP)dllStruct->function)(dllStruct->instance, &V(0), m_iRows, m_iCols, &V(0), desired, testing);
        if (m_bAutoSave || costAccess || runningCostAccess || correlationAccess || performanceMeasuresAccess || confusionAccess || confusionAsPercentAccess || rOCAccess) {
			costCounter++;
			Criterion::determineCost();
		}
    }
}
void Criterion::fireGetReady()
{
}
void Criterion::fireMap()
{
    if (sensitivity())
        fireSensitivity();
    else {
        if (desiredAccess)
            desiredAccess->reportSpatialAccess();
		computePerformanceStats();
        if (m_bAutoSave || costAccess || runningCostAccess || correlationAccess || performanceMeasuresAccess || confusionAccess || confusionAsPercentAccess || rOCAccess) {
            costCounter++;
            determineCost();
        }
        determineCostDerivative();
    }
}
void Criterion::fireSensitivity()
{
	if (m_iSensitivityCounter>=0) {
		register NSFloat
			*pos=&(m_pSensitivity->S(m_iSensitivityCounter++,0)),
			*endPos=pos+U(),
			*currentPos=&V(0),
			*fixedPos=&(m_pFixedOutput->S(0));
		while (pos<endPos)
			*pos++ += (NSFloat)fabs(*currentPos++ - *fixedPos++);
		if (m_iSensitivityCounter >= m_pSensitivity->rows())
			m_iSensitivityCounter = -1;
	} else {
		m_pFixedOutput->copyDataFromDouble(&V(0));
		m_iSensitivityCounter = 0;
	}
}
void Criterion::incrementTime()
{
	register NSFloat *pos=desired, *endPos=pos+V();
	while (pos<endPos)
		*pos++ = 0.0f;
	Axon::incrementTime();
}
void Criterion::ignoreNextError()
{
	m_bIgnoreNextError = TRUE;
}
void Criterion::initialize()
{
    Axon::initialize();
    if (desired)
		free((char*)desired);
    desired = (NSFloat*)calloc(V(), sizeof(NSFloat));
    if (desiredAccess)
		desiredAccess->setData(desired, rows(), cols());
    if (m_dfSumXY)
		free((char*)m_dfSumXY);
    m_dfSumXY = (double*)calloc(V(), sizeof(double));
    if (m_dfSumX)
		free((char*)m_dfSumX);
    m_dfSumX = (double*)calloc(V(), sizeof(double));
    if (m_dfSumY)
		free((char*)m_dfSumY);
    m_dfSumY = (double*)calloc(V(), sizeof(double));
    if (m_dfSumXX)
		free((char*)m_dfSumXX);
    m_dfSumXX = (double*)calloc(V(), sizeof(double));
    if (m_dfSumYY)
		free((char*)m_dfSumYY);
    m_dfSumYY = (double*)calloc(V(), sizeof(double));
    if (m_dfTempSumXY)
		free((char*)m_dfTempSumXY);
    m_dfTempSumXY = (double*)calloc(V(), sizeof(double));
    if (m_dfTempSumX)
		free((char*)m_dfTempSumX);
    m_dfTempSumX = (double*)calloc(V(), sizeof(double));
    if (m_dfTempSumY)
		free((char*)m_dfTempSumY);
    m_dfTempSumY = (double*)calloc(V(), sizeof(double));
    if (m_dfTempSumXX)
		free((char*)m_dfTempSumXX);
    m_dfTempSumXX = (double*)calloc(V(), sizeof(double));
    if (m_dfTempSumYY)
		free((char*)m_dfTempSumYY);
    m_dfTempSumYY = (double*)calloc(V(), sizeof(double));
    if (m_fCorrelation)
		free((char*)m_fCorrelation);
    m_fCorrelation = (NSFloat*)calloc(V(), sizeof(NSFloat));
    if (correlationAccess)
        correlationAccess->setData(m_fCorrelation, rows(), cols());
	m_nConfusionRows = (V() == 1) ? 2 : V();
    if (m_nTempROCCounters)
		free((char*)m_nTempROCCounters);
    m_nTempROCCounters = (int*)calloc(m_nROCThresholds*4, sizeof(int));
    if (m_fROC)
		free((char*)m_fROC);
    m_fROC = (NSFloat*)calloc(m_nROCThresholds*3, sizeof(NSFloat));
    if (m_nROCCounters)
		free((char*)m_nROCCounters);
    m_nROCCounters = (int*)calloc(m_nROCThresholds*4, sizeof(int));
    if (m_fTempConfusion)
		free((char*)m_fTempConfusion);
    m_fTempConfusion = (NSFloat*)calloc(m_nConfusionRows*m_nConfusionRows, sizeof(NSFloat));
    if (m_fConfusion)
		free((char*)m_fConfusion);
    m_fConfusion = (NSFloat*)calloc(m_nConfusionRows*m_nConfusionRows, sizeof(NSFloat));
    if (rOCAccess)
        rOCAccess->setData(m_fROC, m_nROCThresholds, 3);
    if (confusionAccess)
        confusionAccess->setData(m_fConfusion, m_nConfusionRows, m_nConfusionRows);
    if (m_fPerformanceMeasures)
		free((char*)m_fPerformanceMeasures);
    m_fPerformanceMeasures = (NSFloat*)calloc(6, sizeof(NSFloat));
    if (performanceMeasuresAccess)
        performanceMeasuresAccess->setData(m_fPerformanceMeasures, 6);
    if (m_fConfusionAsPercent)
		free((char*)m_fConfusionAsPercent);
    m_fConfusionAsPercent = (NSFloat*)calloc(m_nConfusionRows*m_nConfusionRows, sizeof(NSFloat));
    if (confusionAsPercentAccess)
        confusionAsPercentAccess->setData(m_fConfusionAsPercent, m_nConfusionRows, m_nConfusionRows);
}
void Criterion::networkRandomize()
{
    Axon::networkRandomize();
	m_fLastReportedTestBestCost = 1e9f;
	m_fLastReportedTrainBestCost = 1e9f;
}
void Criterion::networkReset()
{
	m_fLastCost = 1e9f;
	m_fLastReportedTestCost = 0.0f;
	m_fLastReportedTrainCost = 0.0f;
	m_fLastReportedTestBestCost = 1e9f;
	m_fLastReportedTrainBestCost = 1e9f;
	E = tempE = m_fRunningMSE = m_fPercentError = m_fNMSE = m_fAIC = m_fMDL = m_fCorrelationAverage = 0.0f;
	register double *yyPos = m_dfSumYY, *yPos = m_dfSumY, *xPos = m_dfSumX, *xyPos = m_dfSumXY, *xxPos = m_dfSumXX;
    register double *endPos=xPos+V();
    while (xPos<endPos) {
        *xPos++ = 0.0f;
        *yPos++ = 0.0f;
        *xyPos++ = 0.0f;
        *xxPos++ = 0.0f;
        *yyPos++ = 0.0f;
	}
	register NSFloat *pos=m_fConfusion, *endPos2=m_fConfusion+m_nConfusionRows*m_nConfusionRows, *pos2 = m_fConfusionAsPercent;
	while (pos<endPos2)
		*pos++ = *pos2++ = 0.0f;
	NSFloat increment = (m_nROCMaxValue - m_nROCMinValue) / (NSFloat)(m_nROCThresholds-1);
	NSFloat curThreshold = m_nROCMinValue;
	int curIndex = 0;
	while (curIndex < m_nROCThresholds) {
		ROCMatrix(curIndex,0) = curThreshold;
		ROCMatrix(curIndex,1) = 0.0f;
		ROCMatrix(curIndex,2) = 0.0f;
		ROCCountersMatrix(curIndex,0) = 0;
		ROCCountersMatrix(curIndex,1) = 0;
		ROCCountersMatrix(curIndex,2) = 0;
		ROCCountersMatrix(curIndex,3) = 0;
		curIndex++;
		curThreshold += increment;
	}
	m_iSaveCounter = 0;
	tempCounter = costCounter = 0;
	SpatialAccess::m_bRefreshAccess = TRUE;
	if (costAccess)
		costAccess->reportSpatialAccess();
	if (runningCostAccess)
		runningCostAccess->reportSpatialAccess();
	if (correlationAccess)
		correlationAccess->reportSpatialAccess();
	if (performanceMeasuresAccess)
		reportPerformanceAccess();
	if (rOCAccess)
		rOCAccess->reportSpatialAccess();
	if (confusionAccess)
		confusionAccess->reportSpatialAccess();
	if (confusionAsPercentAccess)
		confusionAsPercentAccess->reportSpatialAccess();
	SpatialAccess::m_bRefreshAccess = FALSE;
	Axon::networkReset();
}
void Criterion::phaseEnded(int counter)
{
	if (correlationAccess || performanceMeasuresAccess) {
		register double *yyPos = m_dfSumYY, *yPos = m_dfSumY, *xPos = m_dfSumX, *xyPos = m_dfSumXY, *xxPos = m_dfSumXX;
		register NSFloat *pos = m_fCorrelation, *endPos=pos+V();
		m_fCorrelationAverage = 0.0f;
		m_fNMSE = 0.0f;
		while (pos<endPos) {
			*xPos++ = 0.0f;
			*yPos++ = 0.0f;
			*xyPos++ = 0.0f;
			*xxPos++ = 0.0f;
			*yyPos++ = 0.0f;
			pos++;
		}
		if (performanceMeasuresAccess) {
			m_fRunningMSE = 0.0f;
			m_fPercentError = 0.0f;
		}
	}
	if (rOCAccess) {
		int curIndex = 0;
		while (curIndex < m_nROCThresholds) {
			ROCCountersMatrix(curIndex,0) = 0;
			ROCCountersMatrix(curIndex,1) = 0;
			ROCCountersMatrix(curIndex,2) = 0;
			ROCCountersMatrix(curIndex,3) = 0;
			curIndex++;
		}
	}
	if (confusionAccess || confusionAsPercentAccess) {
		register NSFloat *pos=m_fConfusion, *endPos=m_fConfusion+m_nConfusionRows*m_nConfusionRows;
		while (pos<endPos)
			*pos++ = 0.0f;
	}
}
NSFloat Criterion::reportCost(BOOL reportSpatial, BOOL resetCounter)
{
    if (costCounter) {
		if (testing) {
			m_fLastReportedTestCost = E / (costCounter * V());
			// CG Update calls reportCost with resetCounter = False, don't want to change E if CG Update
			// calls reportCost or it will be miscalculated (divided twice) the second time
			// (the one that counts)
			if (resetCounter)
				E = m_fLastReportedTestCost;
			if (m_bTrainBestCostUpdated) {
				m_bTrainBestCostUpdated = FALSE;
				m_fLastReportedTestBestCost = m_fLastReportedTestCost;
			}
		}
		else {
			m_fLastReportedTrainCost = E / (costCounter * V());
			if (resetCounter)
				E = m_fLastReportedTrainCost;
			if (m_fLastReportedTrainCost < 0)
				m_fLastReportedTrainCost=1e10;
			if (m_fLastReportedTrainCost < m_fLastReportedTrainBestCost) {
				m_bTrainBestCostUpdated = TRUE;
				m_fLastReportedTrainBestCost = m_fLastReportedTrainCost;
			}
		}
		if (reportSpatial) {
			//			if (costAccess)
			//				costAccess->reportSpatialAccess();
			if (correlationAccess || performanceMeasuresAccess) {
				register double *yyPos = m_dfSumYY, *yPos = m_dfSumY, *xPos = m_dfSumX, *xyPos = m_dfSumXY, *xxPos = m_dfSumXX;
				register NSFloat *pos = m_fCorrelation, *endPos=pos+V();
				m_fCorrelationAverage = 0.0f;
				m_fNMSE = 0.0f;
				while (pos<endPos) {
					*pos = (NSFloat)(((costCounter * *xyPos - *xPos * *yPos) / (NSFloat)sqrt((NSFloat)(costCounter * *xxPos - *xPos * *xPos) * (costCounter * *yyPos - *yPos * *yPos))));
					m_fCorrelationAverage += *pos++;
					if (performanceMeasuresAccess)
						m_fNMSE += (NSFloat)((costCounter * *xxPos - *xPos * *xPos) / costCounter);
					*xPos++ = 0.0f;
					*yPos++ = 0.0f;
					*xyPos++ = 0.0f;
					*xxPos++ = 0.0f;
					*yyPos++ = 0.0f;
				}
				m_fCorrelationAverage /= V();
				if (correlationAccess)
					correlationAccess->reportSpatialAccess();
				if (performanceMeasuresAccess) {
					m_fNMSE = m_fRunningMSE / m_fNMSE;
					m_fRunningMSE /= costCounter * V();
					m_fPercentError /= costCounter * V();
					m_fAIC = costCounter * (NSFloat)log(m_fRunningMSE) + 2.0f * (NSFloat)m_nTotalNetworkWeights;
					m_fMDL = costCounter * (NSFloat)log(m_fRunningMSE) + 0.5f * (NSFloat)m_nTotalNetworkWeights * (NSFloat)log(costCounter);
					reportPerformanceAccess();
					m_fRunningMSE = 0.0f;
					m_fPercentError = 0.0f;
				}
			}
			if (rOCAccess) {
				NSFloat increment = (m_nROCMaxValue - m_nROCMinValue) / (NSFloat)(m_nROCThresholds-1);
				NSFloat curThreshold = m_nROCMinValue;
				int curIndex = 0;
				while (curIndex < m_nROCThresholds) {
					ROCMatrix(curIndex,0) = curThreshold;
					ROCMatrix(curIndex,1) = ROCCountersMatrix(curIndex,2) ? (NSFloat)ROCCountersMatrix(curIndex,0)/(NSFloat)ROCCountersMatrix(curIndex,2) : 0.0f;
					ROCMatrix(curIndex,2) = ROCCountersMatrix(curIndex,3) ? (NSFloat)ROCCountersMatrix(curIndex,1)/(NSFloat)ROCCountersMatrix(curIndex,3) : 0.0f;
					ROCCountersMatrix(curIndex,0) = 0;
					ROCCountersMatrix(curIndex,1) = 0;
					ROCCountersMatrix(curIndex,2) = 0;
					ROCCountersMatrix(curIndex,3) = 0;
					curIndex++;
					curThreshold += increment;
				}
				rOCAccess->reportSpatialAccess();
			}
			if (confusionAccess || confusionAsPercentAccess) {
				if (confusionAccess)
					confusionAccess->reportSpatialAccess();
				if (confusionAsPercentAccess) {
					//					register NSFloat *pos=m_fConfusion, *endPos=m_fConfusion+m_nConfusionRows*m_nConfusionRows, *destPos = m_fConfusionAsPercent;
					for (int i=0; i<m_nConfusionRows; i++) {
						int desiredCount = 0;
                        int j;
						for (j=0; j<m_nConfusionRows; j++)
							desiredCount += (int)(confusionMatrix(i, j));
						for (j=0; j<m_nConfusionRows; j++)
							confusionAsPercentMatrix(i, j) = desiredCount ? (confusionMatrix(i, j) / (NSFloat)desiredCount) * 100.0f : 0;
					}
					confusionAsPercentAccess->reportSpatialAccess();
				}
				register NSFloat *pos=m_fConfusion, *endPos=m_fConfusion+m_nConfusionRows*m_nConfusionRows;
				while (pos<endPos)
					*pos++ = 0.0f;
			}
		}
    }
	if (m_bIgnoreNextError) {
		m_fLastReportedTestCost = m_fLastReportedTestBestCost;
		m_fLastReportedTrainCost = m_fLastReportedTrainBestCost;
		E = testing ? m_fLastReportedTestCost : m_fLastReportedTrainCost;
	}
	if (reportSpatial && costCounter && costAccess)
		costAccess->reportSpatialAccess();
	// when called by CGUpdate, don't want to reset costCounter or we can't report next time
	if (resetCounter){
		E = 0.0f;
		costCounter = 0;
	}
    return testing ? m_fLastReportedTestCost : m_fLastReportedTrainCost;
}
void Criterion::reportPerformanceAccess()
{
	*m_fPerformanceMeasures = m_fRunningMSE;
	*(m_fPerformanceMeasures+1) = m_fNMSE;
	*(m_fPerformanceMeasures+2) = m_fCorrelationAverage;
	*(m_fPerformanceMeasures+3) = m_fPercentError;
	*(m_fPerformanceMeasures+4) = m_fAIC;
	*(m_fPerformanceMeasures+5) = m_fMDL;
	performanceMeasuresAccess->reportSpatialAccess();
}
void Criterion::setCostAccess(SpatialAccess *anId)
{
    costAccess = anId;
    if (costAccess)
        costAccess->setData(&E, 1);
}
void Criterion::setConfusionAccess(SpatialAccess* anId)
{
    confusionAccess = anId;
    if (confusionAccess)
        confusionAccess->setData(m_fConfusion, m_nConfusionRows, m_nConfusionRows);
}
void Criterion::setConfusionAsPercentAccess(SpatialAccess* anId)
{
    confusionAsPercentAccess = anId;
    if (confusionAsPercentAccess)
        confusionAsPercentAccess->setData(m_fConfusionAsPercent, m_nConfusionRows, m_nConfusionRows);
}
void Criterion::setCorrelationAccess(SpatialAccess* anId)
{
    correlationAccess = anId;
    if (correlationAccess)
        correlationAccess->setData(m_fCorrelation, rows(), cols());
}
void Criterion::setCurrentExemplar(int exemplar)
{
	m_nCurrentExemplar = exemplar;
}
void Criterion::setDesiredAccess(SpatialAccess* anId)
{
	desiredAccess = anId;
	if (desiredAccess)
		desiredAccess->setData(desired, rows(), cols());
}
void Criterion::setPerformanceMeasuresAccess(SpatialAccess *anId)
{
    performanceMeasuresAccess = anId;
    if (performanceMeasuresAccess)
        performanceMeasuresAccess->setData(m_fPerformanceMeasures, 6);
}
void Criterion::setRunningCostAccess(SpatialAccess *anId)
{
	runningCostAccess = anId;
	if (runningCostAccess)
		runningCostAccess->setData(&runningE, 1);
}
void Criterion::setROCAccess(SpatialAccess* anId)
{
    rOCAccess = anId;
    if (rOCAccess)
        rOCAccess->setData(m_fROC, m_nROCThresholds, 3);
}
void Criterion::setSensitivityData(SpatialData *anId)
{
	if (!anId || (anId != m_pSensitivity)) {
		if (m_pSensitivity && m_bSensitivityDataLocal)
			delete m_pSensitivity;
		if (anId) {
			m_pSensitivity = anId;
			m_bSensitivityDataLocal = FALSE;
		}
		else {
			m_pSensitivity = new SpatialData;
			m_bSensitivityDataLocal = TRUE;
		}
	}
}
void Criterion::setRawSensitivityData(SpatialData *anId)
{
	if (!anId || (anId != m_pRawSensitivity)) {
		if (m_pRawSensitivity && m_bRawSensitivityDataLocal)
			delete m_pRawSensitivity;
		if (anId) {
			m_pRawSensitivity = anId;
			m_bRawSensitivityDataLocal = FALSE;
		}
		else {
			m_pRawSensitivity = new SpatialData;
			m_bRawSensitivityDataLocal = TRUE;
		}
	}
}
void Criterion::setSensitivityTotalsData(SpatialData *anId)
{
	if (!anId || (anId != m_pSensitivityTotals)) {
		if (m_pSensitivityTotals && m_bSensitivityTotalsDataLocal)
			delete m_pSensitivityTotals;
		if (anId) {
			m_pSensitivityTotals = anId;
			m_bSensitivityTotalsDataLocal = FALSE;
		}
		else {
			m_pSensitivityTotals = new SpatialData;
			m_bSensitivityTotalsDataLocal = TRUE;
		}
	}
}
NSFloat Criterion::setTesting(BOOL aFlag)
{
    NSFloat returnValue = 0.0f;
    if (aFlag) {
        tempCounter = costCounter;
        tempE = E;
        E = 0.0f;
        costCounter = 0;
		if (correlationAccess || performanceMeasuresAccess) {
			m_fTempRunningMSE = m_fRunningMSE;
			m_fTempPercentError = m_fPercentError;
			m_fRunningMSE = 0.0f;
			m_fPercentError = 0.0f;
			register double *yyPos = m_dfSumYY, *yPos = m_dfSumY, *tempyPos = m_dfTempSumY, *tempyyPos = m_dfTempSumYY, *xPos = m_dfSumX, *xyPos = m_dfSumXY, *xxPos = m_dfSumXX, *tempxPos = m_dfTempSumX, *tempxyPos = m_dfTempSumXY, *tempxxPos = m_dfTempSumXX;
			register double *endPos=xPos+V();
			while (xPos<endPos) {
				*tempxPos++ = *xPos;
				*tempyPos++ = *yPos;
				*tempxyPos++ = *xyPos;
				*tempxxPos++ = *xxPos;
				*tempyyPos++ = *yyPos;
				*xPos++ = 0.0f;
				*yPos++ = 0.0f;
				*xyPos++ = 0.0f;
				*xxPos++ = 0.0f;
				*yyPos++ = 0.0f;
			}
		}
		if (rOCAccess) {
			int curIndex = 0;
			while (curIndex < m_nROCThresholds) {
				TempROCCountersMatrix(curIndex,0) = ROCCountersMatrix(curIndex,0);
				TempROCCountersMatrix(curIndex,1) = ROCCountersMatrix(curIndex,1);
				TempROCCountersMatrix(curIndex,2) = ROCCountersMatrix(curIndex,2);
				TempROCCountersMatrix(curIndex,3) = ROCCountersMatrix(curIndex,3);
				ROCCountersMatrix(curIndex,0) = 0;
				ROCCountersMatrix(curIndex,1) = 0;
				ROCCountersMatrix(curIndex,2) = 0;
				ROCCountersMatrix(curIndex,3) = 0;
				curIndex++;
			}
		}
		if (confusionAccess || confusionAsPercentAccess) {
			register NSFloat *pos=m_fConfusion, *endPos=m_fConfusion+m_nConfusionRows*m_nConfusionRows, *tempPos = m_fTempConfusion;
			while (pos<endPos) {
				*tempPos++ = *pos;
				*pos++ = 0.0f;
			}
		}
    } else {
        returnValue = reportCost();
        costCounter = tempCounter;
        E = tempE;
		if (correlationAccess || performanceMeasuresAccess) {
			m_fRunningMSE = m_fTempRunningMSE;
			m_fPercentError = m_fTempPercentError;
			register double *yyPos = m_dfSumYY, *yPos = m_dfSumY, *tempyPos = m_dfTempSumY, *tempyyPos = m_dfTempSumYY, *xPos = m_dfSumX, *xyPos = m_dfSumXY, *xxPos = m_dfSumXX, *tempxPos = m_dfTempSumX, *tempxyPos = m_dfTempSumXY, *tempxxPos = m_dfTempSumXX;
			register double *endPos=xPos+V();
			while (xPos<endPos) {
				*xPos++ = *tempxPos++;
				*yPos++ = *tempyPos++;
				*xyPos++ = *tempxyPos++;
				*xxPos++ = *tempxxPos++;
				*yyPos++ = *tempyyPos++;
			}
		}
		if (rOCAccess) {
			int curIndex = 0;
			while (curIndex < m_nROCThresholds) {
				ROCCountersMatrix(curIndex,0) = TempROCCountersMatrix(curIndex,0);
				ROCCountersMatrix(curIndex,1) = TempROCCountersMatrix(curIndex,1);
				ROCCountersMatrix(curIndex,2) = TempROCCountersMatrix(curIndex,2);
				ROCCountersMatrix(curIndex,3) = TempROCCountersMatrix(curIndex,3);
				curIndex++;
			}
		}
		if (confusionAccess || confusionAsPercentAccess) {
			register NSFloat *pos=m_fConfusion, *endPos=m_fConfusion+m_nConfusionRows*m_nConfusionRows, *tempPos = m_fTempConfusion;
			while (pos<endPos)
				*pos++ = *tempPos++;
		}
    }
	testing = aFlag;
    return returnValue;
}
void Criterion::setSensitivity(BOOL aFlag)
{
	m_iSensitivityCounter = -1;
	if (!m_pSensitivity)
		setSensitivityData(NULL);
	if (!m_pSensitivityTotals)
		setSensitivityTotalsData(NULL);
	if (!m_pRawSensitivity)
		setRawSensitivityData(NULL);
	if (m_bSensitivity = aFlag)     {
		m_pSensitivity->zero();
		m_pRawSensitivity->zero();
		m_pSensitivityTotals->zero();
	} else {
		for (int col=0; col<m_pSensitivity->cols(); col++) {
			NSFloat total=0.0f;
			int row;
			for (row=0; row<m_pSensitivity->rows(); row++) {
				m_pSensitivityTotals->S(row) += m_pSensitivity->S(row,col);
				total += m_pSensitivity->S(row,col);
			}
			for (row=0; row<m_pSensitivity->rows(); row++) {
				m_pRawSensitivity->S(row,col) = m_pSensitivity->S(row,col);
				m_pSensitivity->S(row,col) /= total/100;
			}
		}
		NSFloat total=0.0f;
		int i;
		for (i=0; i<m_pSensitivityTotals->S(); i++)
			total += m_pSensitivityTotals->S(i);
		for (i=0; i<m_pSensitivityTotals->S(); i++)
			m_pSensitivityTotals->S(i) /= total/100;
		m_pSensitivity->reportSpatialAccess();
		m_pRawSensitivity->reportSpatialAccess();
		m_pSensitivityTotals->reportSpatialAccess();
	}
}
void Criterion::setSensitivityInputs(int anInt)
{
	int size = anInt? anInt: 1;
	if (!m_pSensitivityTotals)
		setSensitivityTotalsData(NULL);
	if (!m_pSensitivity)
		setSensitivityData(NULL);
	if (!m_pRawSensitivity)
		setRawSensitivityData(NULL);
	m_pSensitivity->setSpatialDimension(size, U());
	m_pRawSensitivity->setSpatialDimension(size, U());
	m_pSensitivityTotals->setSpatialDimension(size);
	m_pFixedOutput->setSpatialDimension(size? U(): 0);
}
void Criterion::setWeightingVector(NSFloat *aVector, int vectorSize)
{
	m_fWeightingVector = aVector;
	m_nWeightingVectorSize = vectorSize;
}
void Criterion::updateWeightedCost()
{
	runningE *= *(m_fWeightingVector + m_nCurrentExemplar);
	m_nCurrentExemplar++;
	if (m_nCurrentExemplar >= m_nWeightingVectorSize)
		m_nCurrentExemplar = 0;
}
// Author: Wesley Curtis Lefebvre

void L1Criterion::determineCost()
{
	register NSFloat *pos=desired, *endPos=pos+V(), *activityPos=&V(0);
	while (pos<endPos)
		runningE += (NSFloat)fabs(*pos++ - *activityPos++);
	Criterion::determineCost();
}
void L1Criterion::determineCostDerivative()
{
	register NSFloat *pos=desired, *endPos=pos+V(), *activityPos=&V(0);
	NSFloat error;
	while (pos<endPos) {
		error = *pos++ - *activityPos;
		*activityPos++ = error < 0.0f ? -1.0f : error > 0.0f ? 1.0f : 0.0f;
	}
}
// Author: Wesley Curtis Lefebvre

void L2Criterion::determineCost()
{
	register NSFloat *pos=desired, *endPos=pos+V(), *activityPos=&V(0);
	while (pos<endPos) {
		register NSFloat x = *pos++ - *activityPos++;
		runningE += 0.5f * x * x;
	}
	Criterion::determineCost();
}
void L2Criterion::determineCostDerivative()
{
	register NSFloat *pos=desired, *endPos=pos+V(), *activityPos=&V(0);
	while (pos<endPos)
		*activityPos++ = *pos++ - *activityPos;
}
// SVML2Criterion
SVML2Criterion::SVML2Criterion()
{
	m_bRunningFirstEpoch = TRUE;
	m_nExemplarCount = 0;
	m_pOutputSynapse = NULL;
}
void SVML2Criterion::determineCost()
{
	register NSFloat *pos=desired, *endPos=pos+V(), *activityPos=&V(0);
	while (pos<endPos) {
		register NSFloat x = *pos++ - (NSFloat)tanh(*activityPos++);
		runningE += 0.5f * x * x;
	}
	Criterion::determineCost();
}
void SVML2Criterion::determineCostDerivative()
{
    register NSFloat *pos=desired, *endPos=pos+V(), *activityPos=&V(0);
    while (pos<endPos)
		*activityPos++ = *pos++ * *activityPos;
	if (m_bRunningFirstEpoch && !testing && m_pOutputSynapse && (m_nExemplarCount < m_pOutputSynapse->V())) {
		pos = desired;
		int outputIndex = 0;
		while (pos<endPos)
			m_pOutputSynapse->setDesiredElement(outputIndex++, m_nExemplarCount, *pos++);
		++m_nExemplarCount;
	}
}
void SVML2Criterion::epochEnded()
{
	m_bRunningFirstEpoch = FALSE;
	m_nExemplarCount = 0;
	L2Criterion::epochEnded();
}
void SVML2Criterion::loadStateVariables(FILE *file, unsigned short int version)
{
	if (version > 219)
		fscanf(file,"%d", &m_bRunningFirstEpoch);
	if (version > 221)
		fscanf(file,"%d", &m_nExemplarCount);
	L2Criterion::loadStateVariables(file, version);
}
void SVML2Criterion::networkReset()
{
	m_bRunningFirstEpoch = TRUE;
	m_nExemplarCount = 0;
	L2Criterion::networkReset();
}
void SVML2Criterion::saveStateVariables(FILE *file)
{
	fprintf(file,"%d\n",m_bRunningFirstEpoch);
	fprintf(file,"%d\n",m_nExemplarCount);
    L2Criterion::saveStateVariables(file);
}
// L2TemporalCriterion
L2TemporalCriterion::L2TemporalCriterion()
{
	m_bDataPreDifferenced = FALSE;
	m_nMode = 0;
	m_bRecencyOfObservation = FALSE;
	m_bDirectionOfChange = FALSE;
	m_bMagnitudeOfChange = FALSE;
	m_fSmallChangeScale = 0.5f;
	m_fLargeChangeScale = 1.5f;
	m_fRightDirectionScale = 0.5f;
	m_fWrongDirectionScale = 1.5f;
	m_fSmallChangeRightDirectionScale = 0.5f;
	m_fLargeChangeRightDirectionScale = 0.8f;
	m_fSmallChangeWrongDirectionScale = 1.2f;
	m_fLargeChangeWrongDirectionScale = 1.5f;
	m_fOverallScale = NULL;
	m_fDiscountRate = 6;
	m_nExemplarNumber = 0;
	m_nNumberOfExemplars = 0;
	m_nActiveNumberOfExemplars = 0;
	m_nCVNumberOfExemplars = 0;
	m_fLastDesired = NULL;
	m_fLastActivity = NULL;
	m_bMemoryNeedsReallocating = TRUE;
}
L2TemporalCriterion::~L2TemporalCriterion()
{
	if (m_fLastDesired)
		delete m_fLastDesired;
	if (m_fLastActivity)
		delete m_fLastActivity;
	if (m_fOverallScale)
		delete m_fOverallScale;
}
void L2TemporalCriterion::determineCost()
{
	// Determine and save the overall scale factor for each output and update the cost.
	// The computed scale factors are used again in "determineCostDerivative".
    register NSFloat x, *pos=desired, *endPos=pos + V(), *activityPos=&V(0), *overallScalePos = m_fOverallScale, *lastDesiredPos = m_fLastDesired, *lastActivityPos = m_fLastActivity, *standardDevPos = &w(0), *normalizationOffsetPos = standardDevPos + V();
	NSFloat recencyScale;
	switch(m_nMode)
	{
	case 0:
		// Normal L2
		while (pos<endPos)
		{
			*overallScalePos++ = 1.0f;
			x = *pos++ - *activityPos++;
			runningE += 0.5f * x * x;
		}
		break;
	case 1:
		// Recency Scale only
		recencyScale = determineRecencyScale();
		while (pos<endPos)
		{
			*overallScalePos = recencyScale;
			x = *pos++ - *activityPos++;
			runningE += 0.5f * (*overallScalePos++) * x * x;
		}
		break;
	case 2:
		// Direction Scale only
		while (pos<endPos)
		{
			*overallScalePos = determineDirectionScale(*pos, *lastDesiredPos++, *activityPos, *lastActivityPos++, *normalizationOffsetPos++);
			x = *pos++ - *activityPos++;
			runningE += 0.5f * (*overallScalePos++) * x * x;
		}
		break;
	case 3:
		// Recency Scale and Direction Scale
		recencyScale = determineRecencyScale();
		while (pos<endPos)
		{
			*overallScalePos = recencyScale * determineDirectionScale(*pos, *lastDesiredPos++, *activityPos, *lastActivityPos++, *normalizationOffsetPos++);
			x = *pos++ - *activityPos++;
			runningE += 0.5f * (*overallScalePos++) * x * x;
		}
		break;
	case 4:
		// Magnitude Scale only
		while (pos<endPos)
		{
			*overallScalePos = determineMagnitudeScale(*pos, *lastDesiredPos++, *standardDevPos++);
			x = *pos++ - *activityPos++;
			runningE += 0.5f * (*overallScalePos++) * x * x;
		}
		break;
	case 5:
		// Recency Scale and Magnitude Scale
		recencyScale = determineRecencyScale();
		while (pos<endPos)
		{
			*overallScalePos = recencyScale * determineMagnitudeScale(*pos, *lastDesiredPos++, *standardDevPos++);
			x = *pos++ - *activityPos++;
			runningE += 0.5f * (*overallScalePos++) * x * x;
		}
		break;
	case 6:
		// Direction Scale and Magnitude Scale = Directional Profit
		while (pos<endPos)
		{
			*overallScalePos = determineDirectionalProfitScale(*pos, *lastDesiredPos++, *activityPos, *lastActivityPos++, *standardDevPos++, *normalizationOffsetPos++);
			x = *pos++ - *activityPos++;
			runningE += 0.5f * (*overallScalePos++) * x * x;
		}
		break;
	case 7:
		// Recency Scale, Direction Scale, and Magnitude Scale = Recency Scale and DirectionalProfit
		recencyScale = determineRecencyScale();
		while (pos<endPos)
		{
			*overallScalePos = recencyScale * determineDirectionalProfitScale(*pos, *lastDesiredPos++, *activityPos, *lastActivityPos++, *standardDevPos++, *normalizationOffsetPos++);
			x = *pos++ - *activityPos++;
			runningE += 0.5f * (*overallScalePos++) * x * x;
		}
		break;
	}
	if (!m_bDataPreDifferenced)
	{
		// Keep track of the last desired value if using Direction Scale or Magnitude Scale
		if (m_bDirectionOfChange || m_bMagnitudeOfChange)
			for (int i = 0; i < V(); i++)
				m_fLastDesired[i] = desired[i];
			// Keep track of the last activity value if using Direction Scale
			if (m_bDirectionOfChange)
				for (int i = 0; i < V(); i++)
					m_fLastActivity[i] = V(i);
	}
	// Update exemplar number
	m_nExemplarNumber++;
	Criterion::determineCost();
}
void L2TemporalCriterion::determineCostDerivative()
{
	// This function assumes that determineCost is always called before determineCostDerivative
	// so that m_fOverallScale does not need to be determined twice.
	register NSFloat *pos=desired, *endPos=pos+V(), *activityPos=&V(0), *overallScalePos = m_fOverallScale;
	while (pos<endPos)
		*activityPos++ = (*overallScalePos++) * (*pos++ - *activityPos);
}
NSFloat L2TemporalCriterion::determineDirectionalProfitScale(NSFloat desired, NSFloat lastDesired, NSFloat activity, NSFloat lastActivity, NSFloat standardDeviation, NSFloat normalizationOffset)
{
	int rightDirection, largeChange;
	if (m_bDataPreDifferenced)
	{
		rightDirection = ((desired - normalizationOffset) * (activity - normalizationOffset) > 0) ? 1 : 0;
		largeChange = (fabs((double) desired) > standardDeviation) ? 1 : 0;
	}
	else
	{
		if (m_nExemplarNumber == 0)
		{
			// Can't compute directional profit scale for 1st exemplar because previous desired output
			// is unknown.  Use normal L2.
			return 1.0f;
		}
		else
		{
			rightDirection = ((desired - lastDesired) * (activity - lastActivity) > 0) ? 1 : 0;
			largeChange = (fabs((double) (desired - lastDesired)) > standardDeviation) ? 1 : 0;
		}
	}
	switch((largeChange * 1) + (rightDirection * 2))
	{
	case 0:
		// Small change, wrong direction
		return m_fSmallChangeWrongDirectionScale;
		break;
	case 1:
		// Large change, wrong direction
		return m_fLargeChangeWrongDirectionScale;
		break;
	case 2:
		// Small change, right direction
		return m_fSmallChangeRightDirectionScale;
		break;
	case 3:
		// Large change, right direction
		return m_fLargeChangeRightDirectionScale;
		break;
	default:
		return 1.0f;
		break;
	}
}
NSFloat L2TemporalCriterion::determineDirectionScale(NSFloat desired, NSFloat lastDesired, NSFloat activity, NSFloat lastActivity, NSFloat normalizationOffset)
{
	if (m_bDataPreDifferenced)
	{
		if ((desired - normalizationOffset) * (activity - normalizationOffset) > 0)
			return m_fRightDirectionScale;
		else
			return m_fWrongDirectionScale;
	}
	else
	{
		if (m_nExemplarNumber == 0)
		{
			// Can't compute direction scale for 1st exemplar because previous desired output
			// is unknown.  Use normal L2.
			return 1.0f;
		}
		else
		{
			if ((desired - lastDesired) * (activity - lastActivity) > 0)
				return m_fRightDirectionScale;
			else
				return m_fWrongDirectionScale;
		}
	}
}
NSFloat L2TemporalCriterion::determineMagnitudeScale(NSFloat desired, NSFloat lastDesired, NSFloat standardDeviation)
{
	if (m_bDataPreDifferenced)
	{
		if (fabs((double) desired) > standardDeviation)
			return m_fLargeChangeScale;
		else
			return m_fSmallChangeScale;
	}
	else
	{
		if (m_nExemplarNumber == 0)
			// Can't compute magnitude scale for 1st exemplar because previous desired output
			// is unknown.  Use normal L2.
			return 1.0f;
		else
		{
			if (fabs((double) (desired - lastDesired)) > standardDeviation)
				return m_fLargeChangeScale;
			else
				return m_fSmallChangeScale;
		}
	}
}
NSFloat L2TemporalCriterion::determineRecencyScale()
{
	return (1.0f / (1.0f + (NSFloat) exp(m_fDiscountRate - ((2 * m_fDiscountRate * (m_nExemplarNumber + 1)) / m_nNumberOfExemplars))));
}
void L2TemporalCriterion::epochEnded()
{
	m_nExemplarNumber = 0;
	Criterion::epochEnded();
}
void L2TemporalCriterion::fireGetReady()
{
	if (m_bMemoryNeedsReallocating)
	{
		// If scaling error according to "Direction Of Change" or "Magnitude Of Change" then
		// previous desired values are necessary so allocate memory for them.
		if (m_bDirectionOfChange || m_bMagnitudeOfChange)
		{
			if (m_fLastDesired)
				delete m_fLastDesired;
			m_fLastDesired = new NSFloat[V()];
		}
		// If scaling error according to "Direction Of Change" then previous activity values
		// are necessary so allocate memory for them.
		if (m_bDirectionOfChange)
		{
			if (m_fLastActivity)
				delete m_fLastActivity;
			m_fLastActivity = new NSFloat[V()];
		}
		// Allocate memory to hold scale factors
		if (m_fOverallScale)
			delete m_fOverallScale;
		m_fOverallScale = new NSFloat[V()];
		m_bMemoryNeedsReallocating  = false;
	}
	// Set the current number of exemplars (used by Recency Scaling)
	m_nNumberOfExemplars = m_nActiveNumberOfExemplars;
	Criterion::fireGetReady();
}
void L2TemporalCriterion::initialize(void)
{
	allocateLinks(U()*2);
	Criterion::initialize();
}
void L2TemporalCriterion::linksAltered(void)
{
	m_bMemoryNeedsReallocating  = true;
	Criterion::linksAltered();
}
void L2TemporalCriterion::networkReset()
{
	m_nExemplarNumber = 0;
	Criterion::networkReset();
}
NSFloat L2TemporalCriterion::setTesting(BOOL aFlag)
{
	if (aFlag)
	{
		m_nNumberOfExemplars = m_nCVNumberOfExemplars;
		m_nExemplarNumber = 0;
	}
	else
		m_nNumberOfExemplars = m_nActiveNumberOfExemplars;
	return Criterion::setTesting(aFlag);
}
// Author: Wesley Curtis Lefebvre

LpCriterion::LpCriterion()
{
	m_nP = 5;
}
void LpCriterion::determineCost()
{
	register NSFloat *pos=desired, *endPos=pos+V(), *activityPos=&V(0);
	while (pos<endPos)
		runningE += (NSFloat)fabs((NSFloat)pow((double)(*pos++ - *activityPos++), (double)m_nP));
	runningE *= 1.0f/(NSFloat)m_nP;
	Criterion::determineCost();
	//      for (register int i=0; i<V(); i++)
	//              E += (NSFloat)pow(desired[i]-V(i), (double)p);
}
void LpCriterion::determineCostDerivative()
{
	register NSFloat *pos=desired, *endPos=pos+V(), *activityPos=&V(0);
	while (pos<endPos) {
		*activityPos = *pos++ - *activityPos;
		*activityPos++ *= (NSFloat)fabs((NSFloat)pow((double)*activityPos, (double)m_nP-2));
	}
	//      for (register int i=0; i<V(); i++) {
	//              V(i) = desired[i]-V(i);
	//              V(i) *= (NSFloat)pow(V(i), (double)p-2);
	//      }
}
void LpCriterion::setP(int anInt) {
	if (anInt != p()) {
		m_nP = anInt;
		if (p() < 2)
			m_nP = 2;
	}
}

// Author: Wesley Curtis Lefebvre

void LinfinityCriterion::determineCost()
{
	register NSFloat *pos=desired, *endPos=pos+V(), *activityPos=&V(0);
	while (pos<endPos)
		runningE += (NSFloat)fabs(tan(*pos++ - *activityPos++));
	Criterion::determineCost();
}
void LinfinityCriterion::determineCostDerivative()
{
	register NSFloat *pos=desired, *endPos=pos+V(), *activityPos=&V(0);
	while (pos<endPos)
		*activityPos++ = (NSFloat)tan(*pos++ - *activityPos);
}
// Author: Wesley Curtis Lefebvre

Gradient::Gradient() {
	errorSoma = NULL;
	forwardSoma = NULL;
	linksAllocated = 0;
	maxWeight = 1e4f;
	m_pCallingObject = NULL;
	dllStruct = NULL;
	m_Gradient = NULL;
	m_Criterion = NULL;
	m_nGradientCount = 0;
}
Gradient::~Gradient() {
	if (m_Gradient) {
		free(m_Gradient);
		m_Gradient = NULL;
	}
}
void Gradient::allocateGradientPointers(int anInt)
{
	if (m_nGradientCount != anInt) {
		if (m_Gradient)
			free(m_Gradient);
		m_Gradient = (Gradient **)malloc(anInt * sizeof(Gradient *));
		m_nGradientCount = anInt;
	}
}
void Gradient::allocateLinks(int anInt) {
	linksAllocated = anInt;
	if (linksAllocated < 1)
		linksAllocated = 1;
	initializeDLL();
}
const char *Gradient::dllFunctionName()
{
	return NULL;
}
void Gradient::constructDLL(void) {
	if (dllStruct->constructor) {
		if (m_pCallingObject)
			dllStruct->instance = (*(DLL_Constructor_CSW)dllStruct->constructor)(dllStruct->instance, m_pCallingObject);
		else
			dllStruct->instance = (*(DLL_Constructor)dllStruct->constructor)(dllStruct->instance);
	}
}
void Gradient::destroyDLL(DLLData *instance) {
	if (dllStruct->destructor) {
		if (m_pCallingObject)
			(*(DLL_Destructor_CSW)dllStruct->destructor)(instance, m_pCallingObject);
		else
			(*(DLL_Destructor)dllStruct->destructor)(instance);
	}
}
void Gradient::fireGetReady()
{
}
void Gradient::initializeDLL(void) {
	if (dllStruct) {
		DLLData *oldInstance = dllStruct->instance;
		constructDLL();
		destroyDLL(oldInstance);
	}
}
void Gradient::loadWeights(FILE *file, unsigned short int version)
{
	//printf("Gradient::loadWeights\n");
	loadStateVariables(file, version);
}
void Gradient::loadStateVariables(FILE *file, unsigned short int version)
{
}
void Gradient::saveWeights(FILE *file)
{
	saveStateVariables(file);
    fprintf(file,"\n");
}
void Gradient::saveStateVariables(FILE *file)
{
}
void Gradient::networkRandomize(void) {
	networkReset();
}
void Gradient::setCriterion(Criterion *aCriterion) {
	m_Criterion = aCriterion;
}
void Gradient::setDLL(void *function, void *constructor, void *destructor, void *instance, void *callingObject)
{
	dllStruct = new DLLStruct;
	dllStruct->function = function;
	dllStruct->constructor = constructor;
	dllStruct->destructor = destructor;
	dllStruct->instance = (DLLData*)instance;
	m_pCallingObject = callingObject;
	initializeDLL();
}
void Gradient::setDLLFunction(DLLStruct *aDLL)
{
	dllStruct = aDLL;
	initializeDLL();
}
void Gradient::setErrorSoma(Soma *aSoma) {
	errorSoma = aSoma;
	allocateLinks(errorSoma? errorSoma->W(): 0);
}
void Gradient::setGradient(Gradient *aGradient, int index) {
	if (m_Gradient) {
		//		ASSERT(index < m_nGradientCount);
		*(m_Gradient+index) = aGradient;
	}
}
void Gradient::setWeightLimit(NSFloat aFloat) {
	maxWeight = aFloat;
}
void Gradient::update(Soma *aSoma) {
}
void Gradient::updateDLL(Soma *aSoma) {
}
void Gradient::updateWeights(Soma *aSoma) {
	if (dllStruct)
		updateDLL(aSoma);
	else
		update(aSoma);
	if (errorSoma->linkAccess)
		errorSoma->linkAccess->reportSpatialAccess();
	aSoma->rangeLinks();
}
void Gradient::networkReset(void) {
	register NSFloat *pos=&(errorSoma->w(0)), *endPos=pos+errorSoma->W();
	while (pos<endPos)
		*pos++ = 0.0f;
	if (errorSoma->linkAccess)
		errorSoma->linkAccess->reportSpatialAccess();
}

// Author: Wesley Curtis Lefebvre
Step::Step() {
    m_fDefaultStepSize = (NSFloat)0.1;
    stepAccess = nil;
    stepSize = NULL;
    stepCount = 0;
    individualSteps = NO;
	m_bDecayWeights = FALSE;
	m_fDecayRate = 0.01f;
	m_nStepDivisor = 1;
}
Step::~Step(void) {
    if (stepSize) {
        free((char*)stepSize);
		stepSize = NULL;
	}
}
void Step::addToStepSize(NSFloat aFloat) {
	for (int i=0; i<step(); i++)
		step(i) += aFloat;
	if (stepAccess)
		stepAccess->reportSpatialAccess();
}
void Step::allocateLinks(int anInt) {
	Gradient::allocateLinks(anInt);
	if (stepSize)
		free((char*)stepSize);
	stepCount = (individualSteps && linksAllocated) ? linksAllocated: 1;
	stepSize = (NSFloat*)calloc(stepCount, sizeof(NSFloat));
	setDefaultStepSize(m_fDefaultStepSize);
	setStepAccess(stepAccess);
}
void Step::constructDLL(void) {
	if (dllStruct->constructor) {
		if (m_pCallingObject)
			dllStruct->instance = (*(Step_Constructor_CSW)dllStruct->constructor)(dllStruct->instance, errorSoma? errorSoma->W(): 0, individualSteps, m_pCallingObject);
		else
			dllStruct->instance = (*(Step_Constructor)dllStruct->constructor)(dllStruct->instance, errorSoma? errorSoma->W(): 0, individualSteps);
	}
}
const char *Step::dllFunctionName()
{
	return "performStep";
}
void Step::multByStepSize(NSFloat aFloat) {
	for (int i=0; i<step(); i++)
		step(i) *= aFloat;
	if (stepAccess)
		stepAccess->reportSpatialAccess();
}
void Step::networkReset(void) {
	if (!individualSteps)
		setDefaultStepSize(m_fDefaultStepSize);
	Gradient::networkReset();
}
void Step::setIndividualSteps(BOOL aFlag) {
	individualSteps = aFlag;
	allocateLinks(linksAllocated);
}
void Step::setDefaultStepSize(NSFloat aFloat) {
	m_fDefaultStepSize = aFloat;
	for (int i=0; i<step(); i++)
		step(i) = m_fDefaultStepSize;
	if (stepAccess)
		stepAccess->reportSpatialAccess();
}
void Step::setStepAccess(SpatialAccess* anNSAccess) {
	if (stepAccess = anNSAccess)
		stepAccess->setData(stepSize, step());
}
void Step::update(Soma *aSoma) {
    register NSFloat *pos=&(aSoma->w(0)), *endPos=pos+aSoma->W(), *errorPos=&(errorSoma->w(0)), *s=&step(0);
    if (individualSteps) {
		if (m_bDecayWeights) {
            while (pos<endPos) {
                *pos++ += (*s++/(NSFloat)m_nStepDivisor) * *errorPos - m_fDecayRate * *pos;
                *errorPos++ = 0.0f;
            }
		}
		else {
            while (pos<endPos) {
                *pos++ += (*s++/(NSFloat)m_nStepDivisor) * *errorPos;
                *errorPos++ = 0.0f;
            }
		}
	}
    else {
		NSFloat tempStep = *s/(NSFloat)m_nStepDivisor;
		if (m_bDecayWeights) {
            while (pos<endPos) {
                *pos++ += tempStep * *errorPos - m_fDecayRate * *pos;
                *errorPos++ = 0.0f;
            }
		}
		else {
            while (pos<endPos) {
                *pos++ += tempStep * *errorPos;
                *errorPos++ = 0.0f;
            }
		}
	}
}
void Step::updateDLL(Soma *aSoma)
{
	if (m_pCallingObject)
		(*(Step_MAP_CSW)dllStruct->function)(dllStruct->instance, &(aSoma->w(0)), aSoma->W(),&(errorSoma->w(0)),&step(0),individualSteps,m_nStepDivisor,m_bDecayWeights,m_fDecayRate,m_pCallingObject);
	else
		(*(Step_MAP)dllStruct->function)(dllStruct->instance, &(aSoma->w(0)), aSoma->W(),&(errorSoma->w(0)),&step(0),individualSteps,m_nStepDivisor,m_bDecayWeights,m_fDecayRate);
    register NSFloat *pos=&(errorSoma->w(0)), *endPos=pos+aSoma->W();
    while (pos<endPos)
		*pos++ = 0.0f;
}
// SVMStep
SVMStep::SVMStep()
{
	m_nExemplarCount = 0;
	m_bRunningFirstEpoch = TRUE;
}
void SVMStep::epochEnded()
{
	m_nExemplarCount = 0;
	m_bRunningFirstEpoch = FALSE;
}
void SVMStep::loadStateVariables(FILE *file, unsigned short int version)
{
	if (version > 219)
		fscanf(file,"%d", &m_bRunningFirstEpoch);
	if (version > 221)
		fscanf(file,"%d", &m_nExemplarCount);
	Step::loadStateVariables(file, version);
}
void SVMStep::networkReset()
{
	m_nExemplarCount = 0;
	m_bRunningFirstEpoch = TRUE;
	Step::networkReset();
}
void SVMStep::saveStateVariables(FILE *file)
{
	fprintf(file,"%d\n",m_bRunningFirstEpoch);
	fprintf(file,"%d\n",m_nExemplarCount);
    Step::saveStateVariables(file);
}
void SVMStep::update(Soma *aSoma) {
	if (!m_bRunningFirstEpoch && (m_nExemplarCount < aSoma->V())) {
		for (int i=0; i<aSoma->U(); i++) {
			int weightIndex = m_nExemplarCount+aSoma->V()*i;
			NSFloat dAlpha = step(individualSteps ? weightIndex : 0) * (1 - errorSoma->w(weightIndex));
			if (aSoma->w(weightIndex)+dAlpha > 0.0f) {
				aSoma->w(weightIndex) += dAlpha;
				((SVMOutputSynapse *)aSoma)->updateSVMBiasElement(i, m_nExemplarCount, dAlpha);
			}
		}
		m_nExemplarCount++;
	}
    register NSFloat *errorPos=&(errorSoma->w(0)), *endPos=errorPos+aSoma->W();
    while (errorPos<endPos)
		*errorPos++ = 0.0f;
}
// CGGlobals
void CGGlobals::initializeGlobals(void){
	//	Sigma=0.0001f;
	//	Lambda=0.01f;
	//	LambdaBar=0.0f;
	//	MSE=0.0f;
	//	DeltaK=0.0f;
	Sigma=0.0001;
	Lambda=0.01;
	LambdaBar=0.0;
	MSE=0.0;
	DeltaK=0.0;
	MSEOld=1e10;
	checkMSE=FALSE;
	Success=FALSE;
	Initialized=FALSE;
	EpochNumber=0;
	WeightsRandomized=FALSE;
	weightsStoredOnFireConclude = FALSE;
	NumWeights=0;
}
// CGVars
CGVars::CGVars() {
	weight= NULL;
	gradient = NULL;
	Pvec = NULL;
	Gvec = NULL;
	SKvec = NULL;
	StoredWeightOnFireConclude = NULL;
	StoredWeight = NULL;
	linksAllocated = 0;
}
//void CGVars::operator=(CGVars var1){
//	weight=var1.weight;
//	gradient=var1.gradient;
//	StoredGradient=var1.StoredGradient;
//	StoredWeight=var1.StoredWeight;
//	Pvec=var1.Pvec;
//	Gvec=var1.Gvec;
//	SKvec=var1.SKvec;
//}
void CGVars::operator++(void){
	weight++;
	gradient++;
	StoredWeight++;
	StoredWeightOnFireConclude++;
	Pvec++;
	Gvec++;
	SKvec++;
}
void CGVars::store_info(Gradient *g_ptr) {
	if (g_ptr->forwardSoma && g_ptr->errorSoma) {
		weight= &(g_ptr->forwardSoma->w(0));
		gradient = &(g_ptr->errorSoma->w(0));
		Pvec = ((ConjugateGradient *)g_ptr)->Pvec;
		Gvec = ((ConjugateGradient *)g_ptr)->Gvec;
		SKvec = ((ConjugateGradient *)g_ptr)->SKvec;
		StoredWeight = ((ConjugateGradient *)g_ptr)->StoredWeights;
		StoredWeightOnFireConclude = ((ConjugateGradient *)g_ptr)->StoredWeightsOnFireConclude;
		linksAllocated = ((ConjugateGradient *)g_ptr)->linksAllocated;
	}
}
void CGVars::store_weight(void){
	*StoredWeight= *weight;
}
void CGVars::store_weight_conclude(void){
	*StoredWeightOnFireConclude= *weight;
}
void CGVars::reload_weight(void){
	*weight = *StoredWeight;
}
void CGVars::reload_weight_conclude(void){
	*weight = *StoredWeightOnFireConclude;
}
//ConjugateGradient
ConjugateGradient::ConjugateGradient() {
	//Reset parameters for all other CGs
	m_bMasterGradient = FALSE;
	StoredWeightsOnFireConclude = NULL;
	StoredWeights = NULL;
	Pvec = NULL;
	Gvec = NULL;
	SKvec = NULL;
	forwardSoma = NULL;
	m_nActiveNumberOfExemplars = 0;
	cgg = NULL;
	grad_comp_num = 0;
	//	resetCG();
}
ConjugateGradient::~ConjugateGradient(void) {
	//Reset parameters for all other CGs
	forwardSoma = NULL;
	if (StoredWeights) {
		free(StoredWeightsOnFireConclude);
		free(StoredWeights);
		free(Pvec);
		free(Gvec);
		free(SKvec);
		StoredWeightsOnFireConclude = NULL;
		StoredWeights = NULL;
		Pvec = NULL;
		Gvec = NULL;
		SKvec = NULL;
	}
	if (m_bMasterGradient && cgg) {
		delete cgg;
		cgg = NULL;
		m_bMasterGradient = FALSE;
	}
	//	resetCG();
	// RESET ALL OTHER PARAMETERS (write routine)
}
void ConjugateGradient::allocateLinks(int anInt) {
	Gradient::allocateLinks(anInt);
	if (StoredWeightsOnFireConclude)
		free(StoredWeightsOnFireConclude);
	StoredWeightsOnFireConclude = (NSFloat*) calloc(linksAllocated, sizeof(NSFloat));
	if (StoredWeights)
		free(StoredWeights);
	StoredWeights = (NSFloat*) calloc(linksAllocated, sizeof(NSFloat));
	if (Pvec)
		free(Pvec);
	//	Pvec = (NSFloat*) calloc(linksAllocated, sizeof(NSFloat));
	Pvec = (double *) calloc(linksAllocated, sizeof(double));
	if (Gvec)
		free(Gvec);
	//	Gvec = (NSFloat*) calloc(linksAllocated, sizeof(NSFloat));
	Gvec = (double *) calloc(linksAllocated, sizeof(double));
	if (SKvec)
		free(SKvec);
	//	SKvec = (NSFloat*) calloc(linksAllocated, sizeof(NSFloat));
	SKvec = (double *) calloc(linksAllocated, sizeof(double));
	if (cgg && m_bMasterGradient)
		cgg->Initialized = FALSE;
	//		cgg->initializeGlobals();
	//	resetCG();
	// RESET ALL OTHER PARAMETERS (write routine)
	//allocate memory here
	//	TRACE("Allocate: %x %x %x %x %x %x %d\n", this, StoredGradients, StoredWeights, Pvec, Gvec, SKvec, linksAllocated);
}
/*
void ConjugateGradient::constructDLL(void) {
if (dllStruct->constructor) {
if (m_pCallingObject)
dllStruct->instance = (*(ConjugateGradient_Constructor_CSW)dllStruct->constructor)(dllStruct->instance, errorSoma? errorSoma->W(): 0, individualSteps, m_pCallingObject);
else
dllStruct->instance = (*(ConjugateGradient_Constructor)dllStruct->constructor)(dllStruct->instance, errorSoma? errorSoma->W(): 0, individualSteps);
}
}
  const char *ConjugateGradient::dllFunctionName()
  {
  return "performConjugateGradient";
  }
*/

void ConjugateGradient::fireGetReady()
{
	if (cgg==NULL) {
		cgg=(CGGlobals *)new CGGlobals;
		cgg->initializeGlobals();
		resetCG();
	}
	else if (cgg->weightsStoredOnFireConclude){
		reloadOldWeightsOnFireConclude();
		cgg->weightsStoredOnFireConclude=FALSE;
	}
}
void ConjugateGradient::fireConclude()
{
	if (cgg!=NULL) {
		storeOldWeightsOnFireConclude();
		reloadOldWeights();
		cgg->weightsStoredOnFireConclude=TRUE;
	}
}
void ConjugateGradient::networkReset(void) {
	Gradient::networkReset();
	if (m_bMasterGradient && cgg)
		cgg->initializeGlobals();
	//Reset parameters here
}
void ConjugateGradient::networkRandomize(void) {
	if (m_bMasterGradient){
		if (cgg) {
			cgg->initializeGlobals();
			cgg->WeightsRandomized=TRUE;
		}
	}
}
CGVars *ConjugateGradient::first_var(Gradient *g_ptr) {
	m_firstvar_cgvars.store_info(g_ptr);
	return(&m_firstvar_cgvars);
}
CGVars *ConjugateGradient::first_weight(void) {
	Gradient *g_ptr;
	grad_comp_num=0;
	g_ptr= *(m_Gradient);
	m_firstweight_cgvars.store_info(g_ptr);
	return(&m_firstweight_cgvars);
}
void ConjugateGradient::loadStateVariables(FILE *file, unsigned short int version)
{
    if (version > 215) {
		int linksAllocated;
		fscanf(file,"%d", &linksAllocated);
		if ((linksAllocated == linksAllocated) && StoredWeightsOnFireConclude) {
            int i;
			for (i=0; i<linksAllocated; i++)
				fscanf(file,"%f", (StoredWeightsOnFireConclude+i));
			for (i=0; i<linksAllocated; i++)
				fscanf(file,"%f", (StoredWeights+i));
			for (i=0; i<linksAllocated; i++)
				fscanf(file,"%lf", (Pvec+i));
			for (i=0; i<linksAllocated; i++)
				fscanf(file,"%lf", (Gvec+i));
			for (i=0; i<linksAllocated; i++)
				fscanf(file,"%lf", (SKvec+i));
			BOOL isMaster;
			fscanf(file,"%d", &isMaster);
			setMasterGradient(isMaster);
			if (m_bMasterGradient) {
				fscanf(file,"%lf", &cgg->Sigma);
				fscanf(file,"%lf", &cgg->Lambda);
				fscanf(file,"%lf", &cgg->LambdaBar);
				fscanf(file,"%lf", &cgg->SigmaK);
				fscanf(file,"%lf", &cgg->MSE);
				fscanf(file,"%lf", &cgg->DeltaK);
				fscanf(file,"%lf", &cgg->MSEOld);
				fscanf(file,"%d", &cgg->checkMSE);
				fscanf(file,"%d", &cgg->Success);
				fscanf(file,"%d", &cgg->Initialized);
				fscanf(file,"%d", &cgg->EpochNumber);
				fscanf(file,"%d", &cgg->WeightsRandomized);
				fscanf(file,"%d", &cgg->weightsStoredOnFireConclude);
				fscanf(file,"%d", &cgg->NumWeights);
			}
		}
		else
			printf("Error loading weights for Conjugate Gradient.\n");
    }
    Gradient::loadStateVariables(file, version);
}
void ConjugateGradient::saveStateVariables(FILE *file)
{
	fprintf(file,"%d\n",linksAllocated);
    int i;
	for (i=0; i<linksAllocated; i++)
		fprintf(file,"%21.15e ", *(StoredWeightsOnFireConclude+i));
	fprintf(file,"\n");
	for (i=0; i<linksAllocated; i++)
		fprintf(file,"%21.15e ", *(StoredWeights+i));
	fprintf(file,"\n");
	for (i=0; i<linksAllocated; i++)
		fprintf(file,"%21.15e ", *(Pvec+i));
	fprintf(file,"\n");
	for (i=0; i<linksAllocated; i++)
		fprintf(file,"%21.15e ", *(Gvec+i));
	fprintf(file,"\n");
	for (i=0; i<linksAllocated; i++)
		fprintf(file,"%21.15e ", *(SKvec+i));
	fprintf(file,"\n");
	BOOL isMaster = (masterGradient() && cgg);
	fprintf(file,"%d\n",(int)isMaster);
	if (isMaster) {
		fprintf(file,"%21.15e\n", cgg->Sigma);
		fprintf(file,"%21.15e\n", cgg->Lambda);
		fprintf(file,"%21.15e\n", cgg->LambdaBar);
		fprintf(file,"%21.15e\n", cgg->SigmaK);
		fprintf(file,"%21.15e\n", cgg->MSE);
		fprintf(file,"%21.15e\n", cgg->DeltaK);
		fprintf(file,"%21.15e\n", cgg->MSEOld);
		fprintf(file,"%d\n", (int)cgg->checkMSE);
		fprintf(file,"%d\n", (int)cgg->Success);
		fprintf(file,"%d\n", (int)cgg->Initialized);
		fprintf(file,"%d\n", (int)cgg->EpochNumber);
		fprintf(file,"%d\n", (int)cgg->WeightsRandomized);
		fprintf(file,"%d\n", (int)cgg->weightsStoredOnFireConclude);
		fprintf(file,"%d\n", (int)cgg->NumWeights);
	}
	fprintf(file,"\n");
    Gradient::saveStateVariables(file);
}
void ConjugateGradient::setMasterGradient(BOOL aBool)
{
/*
if (m_bMasterGradient) {
if (cgg) {
delete cgg;
cgg = NULL;
}
}
	*/
    m_bMasterGradient = aBool;
	if (m_bMasterGradient) {
		if (!cgg){
			cgg = new CGGlobals;
			cgg->initializeGlobals();
			resetCG();
		}
	}
}
void ConjugateGradient::resetCG(void) {
	CGVars *cgvar;
	if (m_Gradient && m_bMasterGradient && cgg) {
		//		cgg->initializeGlobals();
		// RESET ALL WEIGHTS, ETC.
		cgvar=first_weight();
		while (cgvar->weight) {
			*cgvar->SKvec = 0.0f;
			*cgvar->Pvec = 0.0f;
			*cgvar->Gvec = 0.0f;
			cgvar=next_weight(cgvar);
		}
	}
}
CGVars *ConjugateGradient::next_weight(CGVars *cgin) {
	Gradient *g_ptr;
	CGVars *cgvars;
	cgvars = cgin;
	//	ASSERT(grad_comp_num < m_nGradientCount);
	g_ptr= *(m_Gradient+grad_comp_num);
	if (g_ptr->forwardSoma) {
		//		ASSERT(g_ptr->forwardSoma->W() <= g_ptr->linksAllocated);
		//		ASSERT(cgvars->linksAllocated == g_ptr->linksAllocated);
		//		ASSERT(cgvars->weight >= &(g_ptr->forwardSoma->w(0)));
		if (cgvars->weight < &(g_ptr->forwardSoma->w(0))+g_ptr->forwardSoma->W()-1)
			++(*cgvars);
		else {
			++(grad_comp_num);
			if (grad_comp_num < m_nGradientCount) {
				g_ptr= *(m_Gradient+grad_comp_num);
				cgvars = first_var(g_ptr);
			}
			else
				cgvars->weight=NULL;
		}
	}
	return(cgvars);
}
// CALLED EVERY OTHER TIME  (1,3,5...)
void ConjugateGradient::phaseEnded(int counter)
{
	CGVars *cgvar;
	//	NSFloat norm,norm2;
	double norm,norm2;
	if (cgg && cgg->Initialized && !cgg->WeightsRandomized){
		cgg->MSE = m_Criterion->lastReportedTrainCost() * m_nActiveNumberOfExemplars;
		// Finish Step 2 after computing E'(w+sigma*p)
		if (cgg->Success) {
			cgg->DeltaK=0.0f;
			cgvar=first_weight();
			while (cgvar->weight) {
				*cgvar->SKvec = (*cgvar->Gvec - *cgvar->gradient)/cgg->SigmaK;
				cgg->DeltaK+= *cgvar->Pvec * *cgvar->SKvec;
				cgvar=next_weight(cgvar);
			}
		}
		// STEP 3
		norm=0.0f;
		cgvar=first_weight();
		while (cgvar->weight) {
			norm+= *cgvar->Pvec * *cgvar->Pvec;
			cgvar=next_weight(cgvar);
		}
		norm=(NSFloat) sqrt(norm);
		norm2=norm*norm;
		cgg->MuK=0;
		cgvar=first_weight();
		while (cgvar->weight) {
			*cgvar->SKvec += (cgg->Lambda - cgg->LambdaBar)* *cgvar->Pvec;
			cgg->MuK += *cgvar->Pvec * *cgvar->Gvec;
			cgvar=next_weight(cgvar);
		}
		cgg->DeltaK += (cgg->Lambda - cgg->LambdaBar)*norm2;
		// STEP 4
		if (cgg->DeltaK<0) {
			cgvar=first_weight();
			while (cgvar->weight) {
				*cgvar->SKvec += (cgg->Lambda - 2*cgg->DeltaK/norm2) * *cgvar->Pvec;
				cgvar=next_weight(cgvar);
			}
			cgg->LambdaBar=2*(cgg->Lambda - cgg->DeltaK/norm2);
			cgg->DeltaK = - cgg->DeltaK + cgg->Lambda*norm2;
			cgg->Lambda = cgg->LambdaBar;
		}
		// STEP 5
		cgg->AlphaK = cgg->MuK / cgg->DeltaK;
		cgvar=first_weight();
		while (cgvar->weight) {
			*cgvar->weight = (NSFloat)(*cgvar->StoredWeight + cgg->AlphaK * *cgvar->Pvec);
			cgvar=next_weight(cgvar);
		}
		cgg->checkMSE=TRUE;
	}
	resetGradients();
	cgg->WeightsRandomized=FALSE;
}

// CALLED EVERY OTHER TIME (2,4,6)
void ConjugateGradient::epochEnded(int counter)
{
	CGVars *cgvar;
	//	NSFloat sum, capdelta,betak,norm;
	double sum, capdelta,betak,norm;

	if (cgg && !cgg->WeightsRandomized) { //can't do anything since weights are not equal to gradient etc.
		cgg->EpochNumber=counter;
		cgg->MSE = m_Criterion->lastReportedTrainCost() * m_nActiveNumberOfExemplars;
		//		if ((!cgg->Initialized) || (cgg->EpochNumber % cgg->NumWeights == 0)) {
		if (!cgg->Initialized) {
			// STEP 1	INITIALIZE
			cgg->initializeGlobals();
			resetCG();
			cgg->Success=TRUE;
			//			cgg->Success=FALSE;
			cgg->Initialized=TRUE;
			cgg->NumWeights=0;
			cgvar=first_weight();
			cgg->MSE = m_Criterion->lastReportedTrainCost() * m_nActiveNumberOfExemplars;
			while (cgvar->weight) {
				++cgg->NumWeights;
				*cgvar->Pvec = *cgvar->gradient;
				*cgvar->Gvec = *cgvar->gradient;
				cgvar=next_weight(cgvar);
			}
		} else if (cgg->Lambda>1e6) {
			// if lambda gets too large than nothing happens, follow gradient descent and try again
			cgg->initializeGlobals();
			resetCG();
			cgg->Success=TRUE;
			//			cgg->Success=FALSE;
			cgg->Initialized=TRUE;
			cgg->NumWeights=0;
			cgvar=first_weight();
			cgg->MSE = m_Criterion->lastReportedTrainCost() * m_nActiveNumberOfExemplars;
			//			cgg->MSE = cgg->MSEOld; // this is necessary to get the update accepted
			while (cgvar->weight) {
				++cgg->NumWeights;
				//				*cgvar->Pvec = *cgvar->gradient;
				//				*cgvar->Gvec = *cgvar->gradient;
				*cgvar->Pvec = 0.01*cgg->NumWeights;
				*cgvar->Gvec = 0.01*cgg->NumWeights;
				*cgvar->weight = (NSFloat)(*cgvar->weight + *cgvar->weight*((NSFloat) rand()/RAND_MAX-.5)*.01);
				cgvar=next_weight(cgvar);
			}
		}
		else {
			// CONTINUATION OF STAGE 2 (Steps 6 & 7)
			// STEP 6
			sum = 0.0f;
			cgvar=first_weight();
			while (cgvar->weight) {
				sum += *cgvar->Pvec * *cgvar->Gvec;
				cgvar=next_weight(cgvar);
			}
			capdelta=2*(cgg->MSEOld - cgg->MSE)/sum/cgg->AlphaK;
			// STEP 7
			/*			if ((capdelta>=0) || (cgg->Lambda >1)){
			if (cgg->Lambda > 1) {	 // IF LAMBDA IS GETTING BIG, RESET AND USE GRAD DESCENT
			cgvar=first_weight();
			while (cgvar.weight) {
			*cgvar.Pvec = *cgvar.Gvec;
			cgvar=next_weight(cgvar);
			}
			cgg->Lambda=0.01f;
			}
			*/
			if (capdelta>=0) {
				cgg->LambdaBar=0.0f;
				sum=0.0f; norm=0.0f;
				if (!cgg->NumWeights || (cgg->EpochNumber % cgg->NumWeights == 0)) {
					cgg->NumWeights=0;
					cgvar=first_weight();
					while (cgvar->weight) {
						++cgg->NumWeights;
						*cgvar->Pvec = *cgvar->gradient;
						*cgvar->Gvec = *cgvar->gradient;
						cgvar=next_weight(cgvar);
					}
				}
				else {
					cgvar=first_weight();
					while (cgvar->weight) {
						norm += *cgvar->gradient * *cgvar->gradient;
						sum += *cgvar->gradient * *cgvar->Gvec;
						cgvar=next_weight(cgvar);
					}
					norm = (NSFloat) sqrt(norm);
					betak=(norm*norm-sum)/cgg->MuK;
					cgvar=first_weight();
					while (cgvar->weight) {
						*cgvar->Pvec= *cgvar->Gvec + betak * *cgvar->Pvec;
						*cgvar->Gvec= *cgvar->gradient;
						cgvar=next_weight(cgvar);
					}
				}
				if (capdelta>=0.75)
					cgg->Lambda=0.5f * cgg->Lambda;
				cgg->Success=TRUE;
			}
			else {	  // capdelta <0   STILL STEP 7
				cgg->LambdaBar= cgg->Lambda;
				cgg->Success=FALSE;
				// ERROR IS RISING, RELOAD OLD WEIGHTS
				reloadOldWeights();
			}
			// STEP 8
			if (capdelta<0.25f)
				cgg->Lambda=4.0f * cgg->Lambda;
		}
		// Now that we have finished stage 2 or initialization, start step 2
		// START OF STEP 2
		if (cgg->Success) {
			norm=0.0f;
			cgvar=first_weight();
			while (cgvar->weight) {
				norm+=*cgvar->Pvec * *cgvar->Pvec;
				cgvar=next_weight(cgvar);
			}
			norm=(NSFloat) sqrt(norm);
			cgg->SigmaK= cgg->Sigma/norm;
			cgg->MSEOld = cgg->MSE;
		}  // If not success, can probably rewrite to not take the extra pass through for sigmak
		storeOldWeights();
		cgvar=first_weight();
		while (cgvar->weight) {
			*cgvar->weight = (NSFloat)(*cgvar->StoredWeight +cgg->SigmaK * *cgvar->Pvec);
			cgvar=next_weight(cgvar);
		}
	}
	resetGradients();
}

void ConjugateGradient::storeOldWeights(){
	CGVars *cgvar;
	cgvar=first_weight();
	while (cgvar->weight) {
		//TRACE("Store Weights: %x %x %x %f\n", this, cgvar.StoredWeight, cgvar.weight, *cgvar.weight);
		cgvar->store_weight();
		cgvar=next_weight(cgvar);
	}
}
void ConjugateGradient::storeOldWeightsOnFireConclude(){
	CGVars *cgvar;
	cgvar=first_weight();
	while (cgvar->weight) {
		//TRACE("Store Weights Conclude: %x %x %x %f\n", this, cgvar.StoredWeight, cgvar.weight, *cgvar.weight);
		cgvar->store_weight_conclude();
		cgvar=next_weight(cgvar);
	}
}
void ConjugateGradient::reloadOldWeights(){
	CGVars *cgvar;
	cgvar=first_weight();
	while (cgvar->weight) {
		//TRACE("Reload Weights: %x %x %x\n", this, cgvar.weight, cgvar.StoredWeight);
		cgvar->reload_weight();
		cgvar=next_weight(cgvar);
	}
}
void ConjugateGradient::reloadOldWeightsOnFireConclude(){
	CGVars *cgvar;
	cgvar=first_weight();
	while (cgvar->weight) {
		//TRACE("Reload Weights Conclude: %x %x %x\n", this, cgvar.weight, cgvar.StoredWeight);
		cgvar->reload_weight_conclude();
		cgvar=next_weight(cgvar);
	}
}
void ConjugateGradient::resetGradients(){
	CGVars *cgvar;
	cgvar=first_weight();
	while (cgvar->weight) {
		*cgvar->gradient = 0.0f;
		cgvar=next_weight(cgvar);
	}
}
void ConjugateGradient::update(Soma *aSoma) {
	NSFloat new_cost;
	forwardSoma = aSoma;
	if (m_bMasterGradient && cgg) {
		if (cgg->checkMSE) {
			m_Criterion->reportCost(FALSE,FALSE);
			new_cost=m_Criterion->lastReportedTrainCost() * m_nActiveNumberOfExemplars;
			if (new_cost<0)
				new_cost = 1e10;
			if (new_cost>=cgg->MSEOld)
				m_Criterion->ignoreNextError();
			cgg->checkMSE=FALSE;
		}
	}
}
/*
void ConjugateGradient::updateDLL(Soma *aSoma)
{
if (m_pCallingObject)
(*(ConjugateGradient_MAP_CSW)dllStruct->function)(dllStruct->instance, &(aSoma->w(0)), aSoma->W(),&(errorSoma->w(0)),&step(0),individualSteps,m_pCallingObject);
else
(*(ConjugateGradient_MAP)dllStruct->function)(dllStruct->instance, &(aSoma->w(0)), aSoma->W(),&(errorSoma->w(0)),&step(0),individualSteps);
register NSFloat *pos=&(errorSoma->w(0)), *endPos=pos+aSoma->W();
while (pos<endPos)
*pos++ = 0.0f;
}
*/
// Author: Wesley Curtis Lefebvre

Momentum::Momentum() {
	defaultMomentum = (NSFloat)0.5;
	deltaWAccess = momentumAccess = nil;
	momentumSize = delta = NULL;
	individualMomentums = NO;
	momentumCount = 0;
}
void Momentum::allocateLinks(int anInt) {
	Step::allocateLinks(anInt);
	if (delta)
		free((char*)delta);
	delta = (NSFloat*)calloc(linksAllocated, sizeof(NSFloat));
	setDeltaWAccess(deltaWAccess);
	if (momentumSize)
		free((char*)momentumSize);
	momentumCount = individualMomentums? linksAllocated: 1;
	momentumSize = (NSFloat*)calloc(momentumCount, sizeof(NSFloat));
	setDefaultMomentum(defaultMomentum);
	setMomentumAccess(momentumAccess);
}
const char *Momentum::dllFunctionName()
{
	return "performMomentum";
}
void Momentum::setDeltaWAccess(SpatialAccess *anNSAccess) {
	deltaWAccess = anNSAccess;
	if (deltaWAccess)
		deltaWAccess->setData(delta, linksAllocated);
}
void Momentum::addToMomentum(NSFloat aFloat) {
	for (int i=0; i<momentum(); i++)
		momentum(i) += aFloat;
	if (momentumAccess)
		momentumAccess->reportSpatialAccess();
}
void Momentum::multByMomentum(NSFloat aFloat) {
	for (int i=0; i<momentum(); i++)
		momentum(i) *= aFloat;
	if (momentumAccess)
		momentumAccess->reportSpatialAccess();
}
void Momentum::loadStateVariables(FILE *file, unsigned short int version)
{
	if (version > 123) {
		int elementCount;
		NSFloat dummy;
		fscanf(file,"%d", &elementCount);
		for (int i=0; i<elementCount; i++)
			fscanf(file,"%f", i<errorSoma->W() ? &dW(i) : &dummy);
	}
	Step::loadStateVariables(file, version);
}
void Momentum::saveStateVariables(FILE *file)
{
    fprintf(file,"%d ",errorSoma->W());
    for (int i=0; i<errorSoma->W(); i++)
		fprintf(file,"%21.15e ", dW(i));
    fprintf(file,"\n");
    Step::saveStateVariables(file);
}
void Momentum::setDefaultMomentum(NSFloat aFloat) {
	defaultMomentum = aFloat;
	for (int i=0; i<momentum(); i++)
		momentum(i) = defaultMomentum;
	if (momentumAccess)
		momentumAccess->reportSpatialAccess();
}
void Momentum::setIndividualMomentums(BOOL aFlag) {
	individualMomentums = aFlag;
	allocateLinks(linksAllocated);
}
void Momentum::setMomentumAccess(SpatialAccess* anNSAccess) {
	if (momentumAccess = anNSAccess)
		momentumAccess->setData(momentumSize, momentum());
}
void Momentum::update(Soma *aSoma) {
    register NSFloat
		*pos=&(aSoma->w(0)),
		*endPos=pos+aSoma->W(),
		*errorPos=&(errorSoma->w(0)),
		*s=&step(0),
		m=momentum(0),
		*dWPos=&dW(0);
	if (individualSteps) {
		if (m_bDecayWeights) {
			while (pos<endPos) {
				*dWPos = m * *dWPos + (*s++/(NSFloat)m_nStepDivisor) * *errorPos;
                *pos++ += *dWPos++ - m_fDecayRate * *pos;
                *errorPos++ = 0.0f;
            }
		}
		else {
			while (pos<endPos) {
                *pos++ += *dWPos++ = m * *dWPos + (*s++/(NSFloat)m_nStepDivisor) * *errorPos;
                *errorPos++ = 0.0f;
            }
		}
	}
    else {
		NSFloat tempStep = *s/(NSFloat)m_nStepDivisor;
		if (m_bDecayWeights) {
			while (pos<endPos) {
				*dWPos = m * *dWPos + tempStep * *errorPos;
                *pos++ += *dWPos++ - m_fDecayRate * *pos;
                *errorPos++ = 0.0f;
			}
		}
		else {
			while (pos<endPos) {
                *pos++ += *dWPos++ = m * *dWPos + tempStep * *errorPos;
                *errorPos++ = 0.0f;
			}
		}
	}
    if (deltaWAccess)
		deltaWAccess->reportSpatialAccess();
}
void Momentum::updateDLL(Soma *aSoma)
{
	if (m_pCallingObject)
        (*(Momentum_MAP_CSW)dllStruct->function)(dllStruct->instance, &(aSoma->w(0)), aSoma->W(),&(errorSoma->w(0)),&step(0),individualSteps,m_nStepDivisor,m_bDecayWeights,m_fDecayRate,momentum(0),&dW(0),m_pCallingObject);
	else
        (*(Momentum_MAP)dllStruct->function)(dllStruct->instance, &(aSoma->w(0)), aSoma->W(),&(errorSoma->w(0)),&step(0),individualSteps,m_nStepDivisor,m_bDecayWeights,m_fDecayRate,momentum(0),&dW(0));
    register NSFloat *pos=&(errorSoma->w(0)), *endPos=pos+aSoma->W();
    while (pos<endPos)
		*pos++ = 0.0f;
    if (deltaWAccess)
		deltaWAccess->reportSpatialAccess();
}
void Momentum::networkReset(void) {
    register NSFloat *pos=&dW(0), *endPos=pos+linksAllocated;
    while (pos<endPos)
		*pos++ = 0.0f;
	SpatialAccess::m_bRefreshAccess = TRUE;
    if (deltaWAccess)
		deltaWAccess->reportSpatialAccess();
	SpatialAccess::m_bRefreshAccess = FALSE;
    setDefaultMomentum(defaultMomentum);
    Step::networkReset();
}
Momentum::~Momentum(void) {
	if (delta)
		free((char*)delta);
	if (momentumSize)
		free((char*)momentumSize);
}

// Author: Wesley Curtis Lefebvre

Quickprop::Quickprop() {
	deltaEAccess = nil;
	deltaE = NULL;
	individualMomentums = YES;
}
void Quickprop::allocateLinks(int anInt) {
	Momentum::allocateLinks(anInt);
	if (deltaE)
		free((char*)deltaE);
	deltaE = (NSFloat*)calloc(linksAllocated, sizeof(NSFloat));
	setDeltaEAccess(deltaEAccess);
}
const char *Quickprop::dllFunctionName()
{
	return "performQuickprop";
}
void Quickprop::setDeltaEAccess(SpatialAccess* anNSAccess) {
	if (deltaEAccess = anNSAccess)
		deltaEAccess->setData(deltaE, linksAllocated);
}
void Quickprop::setDefaultMomentum(NSFloat aFloat) {
	defaultMomentum = aFloat;
}
void Quickprop::update(Soma *aSoma) {
	if (m_bDecayWeights) {
		for (register int i=0; i<errorSoma->W(); i++) {
			momentum(i) = errorSoma->w(i)/(dE(i) - errorSoma->w(i));
			if (momentum(i) > defaultMomentum)
				momentum(i) = defaultMomentum;
			if (momentum(i) < -defaultMomentum)
				momentum(i) = -defaultMomentum;
			dW(i) = momentum(i)*dW(i) + dE(i) * errorSoma->w(i) < 0? 0 :(step(i)/(NSFloat)m_nStepDivisor)*errorSoma->w(i);
			aSoma->w(i) += dW(i) - m_fDecayRate * aSoma->w(i);
			dE(i) = errorSoma->w(i);
		}
	}
	else {
		for (register int i=0; i<errorSoma->W(); i++) {
			momentum(i) = errorSoma->w(i)/(dE(i) - errorSoma->w(i));
			if (momentum(i) > defaultMomentum)
				momentum(i) = defaultMomentum;
			if (momentum(i) < -defaultMomentum)
				momentum(i) = -defaultMomentum;
			dW(i) = momentum(i)*dW(i) + dE(i) * errorSoma->w(i) < 0? 0 :(step(i)/(NSFloat)m_nStepDivisor)*errorSoma->w(i);
			aSoma->w(i) += dW(i);
			dE(i) = errorSoma->w(i);
		}
	}
	SpatialAccess::m_bRefreshAccess = TRUE;
    if (deltaWAccess)
        deltaWAccess->reportSpatialAccess();
    if (deltaEAccess)
        deltaEAccess->reportSpatialAccess();
	SpatialAccess::m_bRefreshAccess = FALSE;
    Gradient::networkReset();
}
void Quickprop::updateDLL(Soma *aSoma)
{
	if (m_pCallingObject)
        (*(Quickprop_MAP_CSW)dllStruct->function)(dllStruct->instance, &(aSoma->w(0)), aSoma->W(),&(errorSoma->w(0)),&step(0),individualSteps,m_nStepDivisor,m_bDecayWeights,m_fDecayRate,&dW(0),&momentum(0),defaultMomentum,&dE(0),m_pCallingObject);
	else
        (*(Quickprop_MAP)dllStruct->function)(dllStruct->instance, &(aSoma->w(0)), aSoma->W(),&(errorSoma->w(0)),&step(0),individualSteps,m_nStepDivisor,m_bDecayWeights,m_fDecayRate,&dW(0),&momentum(0),defaultMomentum,&dE(0));
    register NSFloat *pos=&(errorSoma->w(0)), *endPos=pos+aSoma->W();
    while (pos<endPos)
		*pos++ = 0.0f;
	SpatialAccess::m_bRefreshAccess = TRUE;
    if (deltaWAccess)
		deltaWAccess->reportSpatialAccess();
    if (deltaEAccess)
		deltaEAccess->reportSpatialAccess();
	SpatialAccess::m_bRefreshAccess = FALSE;
    Gradient::networkReset();
}
void Quickprop::loadStateVariables(FILE *file, unsigned short int version)
{
	if (version > 123) {
		int elementCount;
		NSFloat dummy;
		fscanf(file,"%d", &elementCount);
		for (int i=0; i<elementCount; i++)
			fscanf(file,"%f", i<errorSoma->W() ? &dE(i) : &dummy);
	}
	Momentum::loadStateVariables(file, version);
}
void Quickprop::saveStateVariables(FILE *file)
{
	fprintf(file,"%d ",errorSoma->W());
	for (int i=0; i<errorSoma->W(); i++)
		fprintf(file,"%21.15e ", dE(i));
	fprintf(file,"\n");
	Momentum::saveStateVariables(file);
}
void Quickprop::networkReset(void) {
    for (register int i=0; i<linksAllocated; i++)
		dE(i) = (NSFloat)0;
	SpatialAccess::m_bRefreshAccess = TRUE;
    if (deltaEAccess)
		deltaEAccess->reportSpatialAccess();
	SpatialAccess::m_bRefreshAccess = FALSE;
    Momentum::networkReset();
}
Quickprop::~Quickprop(void) {
	if (deltaE)
		free((char*)deltaE);
}

// Author: Wesley Curtis Lefebvre

DeltaBarDelta::DeltaBarDelta() {
	kappaValue = (NSFloat)0.01;
	betaValue = (NSFloat)0.8;
	zetaValue = (NSFloat)0.5;
	individualSteps = YES;
	individualMomentums = NO;
}
const char *DeltaBarDelta::dllFunctionName()
{
	return "performDeltaBarDelta";
}
void DeltaBarDelta::networkReset(void) {
	setDefaultStepSize(m_fDefaultStepSize);
	Quickprop::networkReset();
}
void DeltaBarDelta::setDefaultMomentum(NSFloat aFloat) {
	Momentum::setDefaultMomentum(aFloat);
}
void DeltaBarDelta::setBeta(NSFloat aFloat) {
	betaValue = aFloat;
}
void DeltaBarDelta::setIndividualSteps(BOOL aFlag) {
}
void DeltaBarDelta::setKappa(NSFloat aFloat) {
	kappaValue = aFloat;
}
void DeltaBarDelta::setZeta(NSFloat aFloat) {
	zetaValue = aFloat;
}
void DeltaBarDelta::update(Soma *aSoma) {
    for (register int i=0; i<aSoma->W(); i++) {
        if (S(i)*D(i) > 0)
            step(i) += kappa();
        else
            if (S(i)*D(i) < 0)
                step(i) -= beta()*step(i);
			S(i) = (1-zeta())*D(i) + zeta()*S(i);
    }
    if (deltaEAccess)
        deltaEAccess->reportSpatialAccess();
    if (stepAccess)
        stepAccess->reportSpatialAccess();
    if (momentumAccess)
        momentumAccess->reportSpatialAccess();
    Momentum::update(aSoma);
}
void DeltaBarDelta::updateDLL(Soma *aSoma)
{
	if (m_pCallingObject)
        (*(DeltaBarDelta_MAP_CSW)dllStruct->function)(dllStruct->instance, &step(0),aSoma->W(),momentum(0),&dW(0),&D(0),&S(0),beta(),kappa(),zeta(),m_pCallingObject);
	else
        (*(DeltaBarDelta_MAP)dllStruct->function)(dllStruct->instance, &step(0),aSoma->W(),momentum(0),&dW(0),&D(0),&S(0),beta(),kappa(),zeta());
    if (deltaEAccess)
		deltaEAccess->reportSpatialAccess();
    if (stepAccess)
		stepAccess->reportSpatialAccess();
    if (momentumAccess)
		momentumAccess->reportSpatialAccess();
    Momentum::update(aSoma);
}

// Author: Wesley Curtis Lefebvre
UnsupervisedFull::UnsupervisedFull() {
	m_bLearning = m_bTempLearning = YES;
	step = m_fStep = 0.01f;
	stepAccess = NULL;
	feed = forcingFeed = NULL;
	forcingAccess = NULL;
	m_bForcing = NO;
	m_bWeightNormalize = NO;
	m_bLearningOnReset = TRUE;
}
const char *UnsupervisedFull::dllFunctionName()
{
	return "performUnsupervised";
}
void UnsupervisedFull::fireFeed(void) {
	FullSynapse::fireFeed();
	if (m_bLearning) {
		fireRule();
		if (linkAccess)
			linkAccess->reportSpatialAccess();
	}
}
void UnsupervisedFull::fireDLL()
{
	FullSynapse::fireFeed();
	if (m_bLearning) {
		fireRuleDLL();
		if (linkAccess)
			linkAccess->reportSpatialAccess();
	}
}
void UnsupervisedFull::fireRule()
{
	if (m_bWeightNormalize) {
		register int i,j;
		NSFloat Wnorm;
		for (i=0; i<U(); i++) {
			Wnorm = 0.0f;
			for (j=0; j<V(); j++)
				Wnorm += W(i,j)*W(i,j);
			Wnorm = (NSFloat) sqrt(double(Wnorm));
			for (j=0; j<V(); j++)
				W(i,j) /= Wnorm;
		}
	}
}
void UnsupervisedFull::fireRuleDLL()
{
	if (m_pCallingObject)
        (*(Unsupervised_MAP_CSW)dllStruct->function)(dllStruct->instance, &V(0), Last->rows(), Last->cols(), &(Feed->V(0)), Feed->rows(), Feed->cols(), &w(0), step, m_pCallingObject);
	else
        (*(Unsupervised_MAP)dllStruct->function)(dllStruct->instance, &V(0), Last->rows(), Last->cols(), &(Feed->V(0)), Feed->rows(), Feed->cols(), &w(0), step);
}
void UnsupervisedFull::networkReset()
{
	FullSynapse::networkReset();
	setLearning(m_bLearningOnReset);
	setStepSize(m_fStep);
}
void UnsupervisedFull::setStepAccess(SpatialAccess *anId)
{
	if (stepAccess = anId)
		stepAccess->setData(&step, 1);
}
void UnsupervisedFull::setStepSize(NSFloat aFloat) {
	step = m_fStep = aFloat;
	if (stepAccess)
		stepAccess->reportSpatialAccess();
}
void UnsupervisedFull::allocateForce(void) {
	if (forcingFeed) {
		free ((char*)forcingFeed);
		forcingFeed = NULL;
	}
	if (Feed)
		if (m_bForcing) {
			feed = forcingFeed = (NSFloat*)calloc(U(), sizeof(NSFloat));
			forcingAccess->setData(forcingFeed, U());
		} else
			feed = &(Feed->V(0));
}
void UnsupervisedFull::allocateSynapticLinks(void) {
	FullSynapse::allocateSynapticLinks();
	allocateForce();
}
void UnsupervisedFull::fire(void) {
	if (m_bForcing) {
		register NSFloat *pos=forcingFeed, *endPos=pos+U();
		while (pos<endPos)
			*pos++ = 0.0f;
		forcingAccess->reportSpatialAccess();
	}
	FullSynapse::fire();
}
void UnsupervisedFull::setForcingAccess(SpatialAccess *aNSAccess) {
	if (forcingAccess = aNSAccess)
		forcingAccess->setData(forcingFeed, U());
}
void UnsupervisedFull::setForcing(BOOL aBOOL) {
	m_bForcing = aBOOL && forcingAccess;
	allocateForce();
}
// SVMInputSynapse
SVMInputSynapse::SVMInputSynapse()
{
	m_bRunningFirstEpoch = TRUE;
	m_bDotProductMetric = FALSE;
	m_nExemplarCount = 0;
}
void SVMInputSynapse::epochEnded()
{
	m_bRunningFirstEpoch = FALSE;
	m_nExemplarCount = 0;
	UnsupervisedFull::epochEnded();
}
void SVMInputSynapse::fireFeed(void)
{
    register NSFloat *pos=&V(0), *feedPos=&(Feed->V(0)), *endPos=pos+V(), *endFeedPos=feedPos+U(), *weightPos=&w(0);
    if (m_bDotProductMetric) {
        while (feedPos<endFeedPos) {
            pos=&V(0);
            while (pos<endPos)
				*feedPos += *weightPos++ * *pos++;
            feedPos++;
        }
	}
    else {
        while (feedPos<endFeedPos) {
            pos=&V(0);
            while (pos<endPos) {
				register NSFloat x = *weightPos++ - *pos++;
				*feedPos += x * x;
            }
			*(feedPos++) = (NSFloat) sqrt(*feedPos);
        }
	}
    if (m_bLearning) {
        fireRule();
        if (linkAccess)
			linkAccess->reportSpatialAccess();
    }
}
void SVMInputSynapse::fireRule(void) {
	if (m_bRunningFirstEpoch && (m_nExemplarCount < U())) {
		if (m_bDotProductMetric) {
			NSFloat sum = 0.0f;
            int i;
			for (i=0; i<V(); i++)
				sum += V(i)*V(i);
			sum = (NSFloat)sqrt(sum);
			for (i=0;i<V();i++)
				w(i+m_nExemplarCount*V()) = (sum > 0.0f) ? V(i)/sum : 0.0f;
		}
		else
			for (int i=0;i<V();i++)
				w(i+m_nExemplarCount*V()) = V(i);
			++m_nExemplarCount;
	}
	UnsupervisedFull::fireRule();
}
void SVMInputSynapse::loadStateVariables(FILE *file, unsigned short int version)
{
	if (version > 219)
		fscanf(file,"%d", &m_bRunningFirstEpoch);
	if (version > 221)
		fscanf(file,"%d", &m_nExemplarCount);
	UnsupervisedFull::loadStateVariables(file, version);
}
void SVMInputSynapse::networkReset()
{
	m_bRunningFirstEpoch = TRUE;
	m_nExemplarCount = 0;
	UnsupervisedFull::networkReset();
}
void SVMInputSynapse::saveStateVariables(FILE *file)
{
	fprintf(file,"%d\n",m_bRunningFirstEpoch);
	fprintf(file,"%d\n",m_nExemplarCount);
    UnsupervisedFull::saveStateVariables(file);
}
// Author: Wesley Curtis Lefebvre
//
HebbianFull::HebbianFull() {
	//        feed = forcingFeed = NULL;
	//        forcingAccess = NULL;
	//        m_bForcing = NO;
}
/*
void HebbianFull::allocateForce(void) {
if (forcingFeed) {
free ((char*)forcingFeed);
forcingFeed = NULL;
}
if (Feed)
if (m_bForcing) {
feed = forcingFeed = (NSFloat*)calloc(U(), sizeof(NSFloat));
forcingAccess->setData(forcingFeed, U());
} else
feed = &(Feed->V(0));
}
  void HebbianFull::allocateSynapticLinks(void) {
  UnsupervisedFull::allocateSynapticLinks();
  allocateForce();
  }
	void HebbianFull::fire(void) {
	if (m_bForcing) {
	register NSFloat *pos=forcingFeed, *endPos=pos+U();
	while (pos<endPos)
	*pos++ = 0.0f;
	forcingAccess->reportSpatialAccess();
	}
	UnsupervisedFull::fire();
	}
*/
void HebbianFull::fireRule(void) {
	register NSFloat
		*pos=feed,
		*endPos=pos+U(),
		*weightPos=&w(0),
		*myPos=&V(0),
		*myEndPos=myPos+V();
	while (pos<endPos) {
		myPos=&V(0);
		while (myPos<myEndPos)
			*weightPos++ += step * *myPos++ * *pos;
		pos++;
	}
	UnsupervisedFull::fireRule();
}
void HebbianFull::fireRuleDLL()
{
	if (m_pCallingObject)
        (*(Unsupervised_MAP_CSW)dllStruct->function)(dllStruct->instance, &V(0), Last->rows(), Last->cols(), &(Feed->V(0)), Feed->rows(), Feed->cols(), &w(0), step, m_pCallingObject);
	else
        (*(Unsupervised_MAP)dllStruct->function)(dllStruct->instance, &V(0), Last->rows(), Last->cols(), &(Feed->V(0)), Feed->rows(), Feed->cols(), &w(0), step);
}
/*
void HebbianFull::setForcingAccess(SpatialAccess *aNSAccess) {
if (forcingAccess = aNSAccess)
forcingAccess->setData(forcingFeed, U());
}
  void HebbianFull::setForcing(BOOL aBOOL) {
  m_bForcing = aBOOL;
  allocateForce();
  }
*/
void HebbianFull::storeHopfield(int samples) {
	int i,j,s;
	for (i=0; i<w(); i++)
		w(i) = 0.0f;
	for (s=0; s<samples; s++) {
		for (i=0; i<U(); i++)
			*(forcingFeed+i) = 0.0f;
		forcingAccess->reportSpatialAccess();
		for (i=0; i<U(); i++)
			for (j=0; j<V(); j++)
				if (i != j)
					W(i,j) += *(forcingFeed+i) * *(forcingFeed+j);
	}
	for (i=0; i<w(); i++)
		w(i) /= V();
	linkAccess->reportSpatialAccess();
}

// Author: Wesley Curtis Lefebvre

void OjasFull::fireRule(void) {
	for (register int i,j=0,k; j<V(); j++) {
        NSFloat partialSum=(NSFloat)0.0;
		for (k=0; k<U(); k++)
			partialSum += Feed->V(k) * W(k,j);
		for (i=0; i<U(); i++)
			W(i,j) += step * (Feed->V(i) * (V(j) - partialSum));
	}
	UnsupervisedFull::fireRule();
}

// Author: Wesley Curtis Lefebvre

void SangersFull::fireRule(void) {
	for (register int i, j=0; j<V(); j++)
		for (NSFloat partialSum=(NSFloat)(i=0); i<U(); i++)
			W(i,j) += step * (Feed->V(i) * (V(j) - (partialSum += Feed->V(i) * W(i,j))));
		UnsupervisedFull::fireRule();
}

// Author: Wesley Curtis Lefebvre

CompetitiveFull::CompetitiveFull() {
    m_eMetricType = Euclidean;
	unifiedDistanceAccess = NULL;
	m_fpDistanceMatrix = NULL;
	m_fQuantizationMetric = 0.0f;
	quantizationMetricAccess = NULL;
	m_nNumSamples = 0;
	m_fSumError = 0.0f;
}
#define weightElement(i,j,k)	weights[k + (j)*orig_cols + (i)*orig_cols*koh_cols]
NSFloat CompetitiveFull::distance(int r1, int c1, int r2, int c2, NSFloat *weights, int koh_cols, int orig_cols) {
	int i;
	NSFloat sum=0.0f, temp;
	for (i=0;i<orig_cols;++i){
		temp=(weightElement(r1,c1,i)-weightElement(r2,c2,i));
		sum+=temp*temp;
	}
	return (NSFloat)sqrt(sum);
}
const char *CompetitiveFull::dllFunctionName()
{
	return "performCompetitive";
}
#define distanceElement(i,j)	m_fpDistanceMatrix[j + (i)*(koh_cols*2-1)]
void CompetitiveFull::epochEnded()
{
	UnsupervisedFull::epochEnded();
	if (unifiedDistanceAccess && Feed && links) {
		int orig_cols = V();
		int koh_cols = Feed->cols();
		int koh_rows = Feed->rows();
        int r;
		for (r=0; r<koh_rows; r++) {
			for (int c=0;c<koh_cols;c++) {
				if (c<koh_cols-1)
					distanceElement(r*2,c*2+1)=distance(r,c,r,c+1,&w(0), koh_cols, orig_cols);
				if (r<koh_rows-1)
					distanceElement(r*2+1,c*2)=distance(r,c,r+1,c, &w(0), koh_cols, orig_cols);
				if ((r<koh_rows-1) && (c<koh_cols-1)) {
					NSFloat dz1 = distance(r,c,r+1,c+1, &w(0), koh_cols, orig_cols);
					NSFloat dz2 = distance(r+1,c,r,c+1, &w(0), koh_cols, orig_cols);
					distanceElement(r*2+1,c*2+1) = (dz1+dz2)/2.828f;
				}
			}
		}
		for (r=0; r<koh_rows; r++) {
			for (int c=0;c<koh_cols;c++) {
				if ((r>0) && (c>0) && (r<koh_rows-1) && (c<koh_cols-1))
					distanceElement(r*2,c*2) = (distanceElement(r*2,c*2+1) + distanceElement(r*2+1,c*2) + distanceElement(r*2,c*2-1) + distanceElement(r*2-1,c*2)) / 4.0f;
				else
					if ((r==0) && (c>0) && (c<koh_cols-1))
						distanceElement(r*2,c*2) = (distanceElement(r*2,c*2+1) + distanceElement(r*2+1,c*2) + distanceElement(r*2,c*2-1)) / 3.0f;
					else
						if ((r==koh_rows-1) && (c>0) && (c<koh_cols-1))
							distanceElement(r*2,c*2) = (distanceElement(r*2,c*2+1) + distanceElement(r*2,c*2-1) + distanceElement(r*2-1,c*2)) / 3.0f;
						else
							if ((r>0) && (c==0) && (r<koh_rows-1))
								distanceElement(r*2,c*2) = (distanceElement(r*2,c*2+1) + distanceElement(r*2+1,c*2) + distanceElement(r*2-1,c*2)) / 3.0f;
							else
								if ((r>0) && (c==koh_cols-1) && (r<koh_rows-1))
									distanceElement(r*2,c*2) = (distanceElement(r*2+1,c*2) + distanceElement(r*2,c*2-1) + distanceElement(r*2-1,c*2)) / 3.0f;
								else
									if ((r==0) && (c==0))
										distanceElement(r*2,c*2) = (distanceElement(r*2,c*2+1) + distanceElement(r*2+1,c*2)) / 2.0f;
									else
										if ((r==0) && (c==koh_cols-1))
											distanceElement(r*2,c*2) = (distanceElement(r*2+1,c*2) + distanceElement(r*2,c*2-1)) / 2.0f;
										else
											if ((r==koh_rows-1) && (c==0))
												distanceElement(r*2,c*2) = (distanceElement(r*2,c*2+1) + distanceElement(r*2-1,c*2)) / 2.0f;
											else
												if ((r==koh_rows-1) && (c==koh_cols-1))
													distanceElement(r*2,c*2) = (distanceElement(r*2,c*2-1) + distanceElement(r*2-1,c*2)) / 2.0f;
			}
		}
        unifiedDistanceAccess->reportSpatialAccess();
	}
	if (quantizationMetricAccess) {
		if (m_nNumSamples>0) {
			m_fQuantizationMetric=m_fSumError/m_nNumSamples;
			m_nNumSamples=0;
			m_fSumError=0.0f;
		}
		else
			m_fQuantizationMetric=0.0f;
        quantizationMetricAccess->reportSpatialAccess();
	}
}
void CompetitiveFull::fireDLL()
{
	fireFeed();
}
void CompetitiveFull::fireFeed(void) {
	register NSFloat *pos=&V(0), *feedPos=&(Feed->V(0)), *endPos=pos+V(), *endFeedPos=feedPos+U(), *weightPos=&w(0);
	switch (m_eMetricType) {
	case DotProduct:
		while (feedPos<endFeedPos) {
			pos=&V(0);
			while (pos<endPos)
				*feedPos += *weightPos++ * *pos++;
			feedPos++;
		}
		break;
	case Euclidean:
		while (feedPos<endFeedPos) {
			pos=&V(0);
			while (pos<endPos) {
				register NSFloat x = *weightPos++ - *pos++;
				*feedPos += x * x;
			}
			*(feedPos++) = (NSFloat) sqrt(*feedPos);
		}
		break;
	case BoxCar:
		while (feedPos<endFeedPos) {
			pos=&V(0);
			while (pos<endPos)
				*feedPos += (NSFloat)fabs(*weightPos++ - *pos++);
			feedPos++;
		}
		break;
		//      default:
		//              ASSERT(NO);
	}
	if (m_bLearning) {
		fireRule();
		if (linkAccess)
			linkAccess->reportSpatialAccess();
	}
	if (quantizationMetricAccess && Feed) {
		int koh_cols = Feed->cols();
		int koh_rows = Feed->rows();
		NSFloat min=99999.0f;
		for (int i=0; i<koh_rows; i++)
			for (int j=0; i<koh_cols; i++)
				if (Feed->V(i+j*koh_rows)<min)
					min=Feed->V(i+j*koh_rows);
				m_fSumError+=min;
				++m_nNumSamples;
	}
}
void CompetitiveFull::fireRule(void) {
	int i = 1;
	winningIndex = 0;
	register NSFloat
		*pos=&(Feed->V(1)),
		*endPos=pos+U()-1,
		winningFloat=Feed->V(0);
	if (m_eMetricType == DotProduct)
		while (pos<endPos) {
			if (*pos++ > winningFloat) {
				winningIndex = i;
				winningFloat = Feed->V(i);
			}
			i++;
		}
        else
			while (pos<endPos) {
				if (*pos++ < winningFloat) {
					winningIndex = i;
					winningFloat = Feed->V(i);
				}
				i++;
			}
			if (dllStruct)
                fireWinnerDLL();
			else
                fireWinner();
}
void CompetitiveFull::fireWinner()
{
}
void CompetitiveFull::fireWinnerDLL()
{
	if (m_pCallingObject)
        (*(Competitive_MAP_CSW)dllStruct->function)(dllStruct->instance, &V(0), Last->rows(), Last->cols(), &(Feed->V(0)), Feed->rows(), Feed->cols(), &w(0), step, winningIndex, m_pCallingObject);
	else
        (*(Competitive_MAP)dllStruct->function)(dllStruct->instance, &V(0), Last->rows(), Last->cols(), &(Feed->V(0)), Feed->rows(), Feed->cols(), &w(0), step, winningIndex);
}
void CompetitiveFull::linksAltered(void) {
    if (unifiedDistanceAccess && Feed && (Feed->rows()*2-1)*(Feed->cols()*2-1)) {
		if (m_fpDistanceMatrix)
			free(m_fpDistanceMatrix);
		m_fpDistanceMatrix = (NSFloat *)calloc((Feed->rows()*2-1)*(Feed->cols()*2-1),sizeof(NSFloat));
        unifiedDistanceAccess->setData(m_fpDistanceMatrix, (Feed->rows()*2-1), (Feed->cols()*2-1));
	}
	UnsupervisedFull::linksAltered();
}
void CompetitiveFull::networkReset()
{
	if (quantizationMetricAccess) {
		m_nNumSamples=0;
		m_fSumError=0.0f;
		quantizationMetricAccess->reportSpatialAccess();
	}
	UnsupervisedFull::networkReset();
}
void CompetitiveFull::setQuantizationMetricAccess(SpatialAccess *anId)
{
    quantizationMetricAccess = anId;
    if (quantizationMetricAccess)
        quantizationMetricAccess->setData(&m_fQuantizationMetric, 1);
}
void CompetitiveFull::setUnifiedDistanceAccess(SpatialAccess *anId)
{
    unifiedDistanceAccess = anId;
    if (unifiedDistanceAccess && Feed && (Feed->rows()*2-1)*(Feed->cols()*2-1)) {
		if (m_fpDistanceMatrix)
			free(m_fpDistanceMatrix);
		m_fpDistanceMatrix = (NSFloat *)calloc((Feed->rows()*2-1)*(Feed->cols()*2-1),sizeof(NSFloat));
        unifiedDistanceAccess->setData(m_fpDistanceMatrix, (Feed->rows()*2-1), (Feed->cols()*2-1));
	}
}
// Author: Wesley Curtis Lefebvre
/*
void StandardFull::fireWinner(void) {
register NSFloat *pos=&V(0), *endPos=pos+V(), *weightPos=&W(winningIndex,0);
while (pos<endPos)
*weightPos++ += step * (*pos++ - *weightPos);
}
*/
void StandardFull::fireWinner(void) {
	register NSFloat *pos=&V(0), *endPos=pos+V(), *weightPos=&W(winningIndex,0);
	if (m_bForcing && *(forcingFeed+winningIndex)<0.5)
		while (pos<endPos)
			*weightPos++ -= step * (*pos++ - *weightPos);
		else
			while (pos<endPos)
				*weightPos++ += step * (*pos++ - *weightPos);
}
// Author: Wesley Curtis Lefebvre

ConscienceFull::ConscienceFull()
{
	m_fBeta = 0.01f;
	m_fGamma = 0.3f;
	frequency = nil;
	bias = nil;
	m_bFrequencyDataLocal = FALSE;
	m_bBiasDataLocal = FALSE;
}
ConscienceFull::~ConscienceFull()
{
	if (frequency && m_bFrequencyDataLocal)
		delete frequency;
	if (bias && m_bBiasDataLocal)
		delete bias;
}
void ConscienceFull::allocateSynapticLinks()
{
	//      ASSERT(U());
	UnsupervisedFull::allocateSynapticLinks();
	if (!frequency)
		setFrequencyData(NULL);
	if (!bias)
		setBiasData(NULL);
	if (Feed) {
		int koh_cols = Feed->cols();
		int koh_rows = Feed->rows();
		frequency->setSpatialDimension(koh_rows, koh_cols);
		bias->setSpatialDimension(koh_rows, koh_cols);
	}
	else {
		frequency->setSpatialDimension(U());
		bias->setSpatialDimension(U());
	}
	bias->zero();
	frequency->assignScalar(1.0f/U());
}
void ConscienceFull::fireRule(void) {
	winningIndex = 0;
	register int i=0;
	register NSFloat
		*pos=&(Feed->V(0)),
		*endPos=pos+U(),
		*biasPos=&(bias->S(0)),
		*freqPos=&(frequency->S(0));
	if (m_eMetricType == DotProduct)
		while (pos<endPos) {
			*pos -= *biasPos;
			if (*pos > Feed->V(winningIndex))
				winningIndex = i;
			*pos++, i++;
			*biasPos++ = m_fGamma * (U() * *freqPos - 1.0f);
			*freqPos++ -= m_fBeta * *freqPos;
		}
        else
			while (pos<endPos) {
				*pos += *biasPos;
				if (*pos < Feed->V(winningIndex))
					winningIndex = i;
				*pos++, i++;
				*biasPos++ = m_fGamma * (U() * *freqPos - 1.0f);
				*freqPos++ -= m_fBeta * *freqPos;
			}
			frequency->S(winningIndex) += m_fBeta;
			frequency->reportSpatialAccess();
			bias->reportSpatialAccess();
			if (dllStruct)
                fireWinnerDLL();
			else
                fireWinner();
}
void ConscienceFull::setBiasData(SpatialData *anId)
{
	if (bias && m_bBiasDataLocal)
		delete bias;
	if (anId) {
        bias = anId;
		m_bBiasDataLocal = FALSE;
	}
	else {
		bias = new SpatialData;
		m_bBiasDataLocal = TRUE;
	}
}
void ConscienceFull::setFrequencyData(SpatialData *anId)
{
	if (frequency && m_bFrequencyDataLocal)
		delete frequency;
	if (anId) {
        frequency = anId;
		m_bFrequencyDataLocal = FALSE;
	}
	else {
		frequency = new SpatialData;
		m_bFrequencyDataLocal = TRUE;
	}
}
void ConscienceFull::networkReset()
{
	UnsupervisedFull::networkReset();
	if (U()) {
		if (!bias)
			setBiasData(NULL);
		bias->zero();
		if (!frequency)
			setFrequencyData(NULL);
		frequency->assignScalar(1.0f/U());
	}
}

// Author: Wesley Curtis Lefebvre

KohonenFull::KohonenFull()
{
    m_fNeighborhood = 1.0f;
	m_iNeighborhood = 0;
    startRow = stopRow = startCol = stopCol = 0;
    neighborhoodAccess = NULL;
	componentPlaneAccess = NULL;
	m_fpComponentPlaneMatrix = NULL;
	m_nComponentPlane = 0;
}
const char *KohonenFull::dllFunctionName()
{
    return "performKohonen";
}
#define componentPlaneElement(i,j)	m_fpComponentPlaneMatrix[j + (i)*koh_cols]
void KohonenFull::epochEnded()
{
	ConscienceFull::epochEnded();
	if (componentPlaneAccess && Feed && links) {
		int orig_cols = V();
		int koh_cols = Feed->cols();
		int koh_rows = Feed->rows();
		NSFloat *weights = &w(0);
		for (int r=0; r<koh_rows; r++)
			for (int c=0;c<koh_cols;c++)
				componentPlaneElement(r,c) = weightElement(r,c,m_nComponentPlane);
			componentPlaneAccess->reportSpatialAccess();
	}
}
void KohonenFull::fireWinner() {
	winningRow = (int)(winningIndex/Feed->cols());
	winningCol = winningIndex - winningRow*Feed->cols();
	startRow = winningRow - neighborhood();
	stopRow = winningRow + neighborhood();
	if (startRow < 0)
		startRow = 0;
	if (stopRow >= Feed->rows())
		stopRow = Feed->rows()-1;
	startCol = winningCol - neighborhood();
	stopCol = winningCol + neighborhood();
	if (startCol < 0)
		startCol = 0;
	if (stopCol >= Feed->cols())
		stopCol = Feed->cols()-1;
	fireWinningShape();
}
void KohonenFull::fireWinnerDLL()
{
    winningRow = (int)(winningIndex/Feed->cols());
    winningCol = winningIndex - winningRow*Feed->cols();
	if (m_pCallingObject)
        (*(Kohonen_MAP_CSW)dllStruct->function)(dllStruct->instance, &V(0), Last->rows(), Last->cols(), &(Feed->V(0)), Feed->rows(), Feed->cols(), &w(0), step, winningRow, winningCol, neighborhood(), m_pCallingObject);
	else
        (*(Kohonen_MAP)dllStruct->function)(dllStruct->instance, &V(0), Last->rows(), Last->cols(), &(Feed->V(0)), Feed->rows(), Feed->cols(), &w(0), step, winningRow, winningCol, neighborhood());
}
void KohonenFull::fireWinningShape()
{
}
void KohonenFull::linksAltered(void)
{
    if (componentPlaneAccess && Feed && Feed->rows()*Feed->cols()) {
		if (m_fpComponentPlaneMatrix)
			free(m_fpComponentPlaneMatrix);
		m_fpComponentPlaneMatrix = (NSFloat *)calloc(Feed->rows()*Feed->cols(),sizeof(NSFloat));
        componentPlaneAccess->setData(m_fpComponentPlaneMatrix, Feed->rows(), Feed->cols());
	}
	ConscienceFull::linksAltered();
}
void KohonenFull::networkReset()
{
	ConscienceFull::networkReset();
	setNeighborhood(m_iNeighborhood);
}
void KohonenFull::setNeighborhoodAccess(SpatialAccess *anId)
{
	if (neighborhoodAccess = anId)
		neighborhoodAccess->setData(&m_fNeighborhood, 1);
}
void KohonenFull::setNeighborhood(int anInt) {
	m_fNeighborhood = (NSFloat)anInt;
	m_iNeighborhood = anInt;
	if (anInt<0)
		m_fNeighborhood = 0.0f;
	if (neighborhoodAccess)
		neighborhoodAccess->reportSpatialAccess();
}
void KohonenFull::setComponentPlane(int anInt) {
	m_nComponentPlane = anInt;
    if (anInt < 0)
		m_nComponentPlane = 0;
    if (anInt >= V())
		m_nComponentPlane = V()-1;
    if (componentPlaneAccess)
        epochEnded();
}
void KohonenFull::setComponentPlaneAccess(SpatialAccess *anId)
{
    componentPlaneAccess = anId;
    if (componentPlaneAccess && Feed && Feed->rows()*Feed->cols()) {
		if (m_fpComponentPlaneMatrix)
			free(m_fpComponentPlaneMatrix);
		m_fpComponentPlaneMatrix = (NSFloat *)calloc(Feed->rows()*Feed->cols(),sizeof(NSFloat));
        componentPlaneAccess->setData(m_fpComponentPlaneMatrix, Feed->rows(), Feed->cols());
	}
}
// Author: Wesley Curtis Lefebvre

void LineKohonenFull::fireWinner(void) {
	int start = winningIndex - neighborhood();
	int stop = winningIndex + neighborhood();
	if (start < 0)
		start = 0;
	if (stop >= U())
		stop = U()-1;
	for (register int i=start,j; i<=stop; i++)
		for (j=0; j<V(); j++)
			standardUpdate(i, j);
}

// Author: Wesley Curtis Lefebvre

void SquareKohonenFull::fireWinningShape(void) {
	register NSFloat *endPos=&V(V());
	for (register int i=startRow,j; i<=stopRow; i++)
		for (j=startCol; j<=stopCol; j++) {
			register NSFloat *pos=&V(0), *weightPos=&W(j+i*Feed->cols(),0);
			while (pos<endPos)
				*weightPos++ += step * (*pos++ - *weightPos);
		}
}
// Author: Wesley Curtis Lefebvre

void DiamondKohonenFull::fireWinningShape(void) {
	for (register int i=startRow,j,k; i<=stopRow; i++)
		for (j=startCol; j<=stopCol; j++)
			if (abs(i-winningRow) + abs(j-winningCol) <= neighborhood())
				for (k=0; k<V(); k++)
					standardUpdate(j+i*Feed->cols(), k);
}

// Author: Wesley Curtis Lefebvre

Transmitter::Transmitter()
{
	dllStruct = NULL;
	m_pSendFunction = NULL;
	receiverCount = 0;
	m_pCallingObject = NULL;
}
Transmitter::~Transmitter()
{
	for (unsigned int i=0; i<receiverCount; i++)
		free(receiverName[i]);
}
void Transmitter::addReceiver(void *anId, const char *aString)
{
	receiverID[receiverCount] = anId;
	receiverName[receiverCount] = (char *)malloc(strlen(aString)*sizeof(char)+1);
	strcpy(receiverName[receiverCount++],aString);
	if (receiverCount >= 128) {
		fprintf(stderr, "Too many receivers");
		exit(1);
	}
}
void Transmitter::setDLLFunction(DLLStruct *aDLL)
{
	dllStruct = aDLL;
	initializeDLL();
}
void Transmitter::setDLL(void *function, void *constructor, void *destructor, void *instance, void *callingObject)
{
	dllStruct = new DLLStruct;
	dllStruct->function = function;
	dllStruct->constructor = constructor;
	dllStruct->destructor = destructor;
	dllStruct->instance = (DLLData*)instance;
	m_pCallingObject = callingObject;
	initializeDLL();
}
void Transmitter::setSendFunction(void *aFunction)
{
	m_pSendFunction = aFunction;
}
void Transmitter::constructDLL(void) {
	if (dllStruct->constructor) {
		if (m_pCallingObject)
			dllStruct->instance = (*(DLL_Constructor_CSW)dllStruct->constructor)(dllStruct->instance, m_pCallingObject);
		else
			dllStruct->instance = (*(DLL_Constructor)dllStruct->constructor)(dllStruct->instance);
	}
}
void Transmitter::destroyDLL(DLLData *instance) {
	if (dllStruct->destructor) {
		if (m_pCallingObject)
			(*(DLL_Destructor_CSW)dllStruct->destructor)(instance, m_pCallingObject);
		else
			(*(DLL_Destructor)dllStruct->destructor)(instance);
	}
}
void Transmitter::initializeDLL(void) {
	if (dllStruct) {
		DLLData *oldInstance = dllStruct->instance;
		constructDLL();
		destroyDLL(oldInstance);
	}
}
void Transmitter::reportSpatialData()
{
	if (m_bAccessEnabled && !m_bRefreshAccess && ((dllStruct && dllStruct->function) ? shouldTransmitDLL(data): shouldTransmit(data)))
		sendTransmitterActions();
	SpatialData::reportSpatialData();
}
void Transmitter::sendTransmitterActions()
{
	for (unsigned int i=0; i<receiverCount; i++)
		(*(TransmitterSend_FUNCTION)m_pSendFunction)(receiverID[i], receiverName[i]);
}
BOOL Transmitter::shouldTransmit(NSFloat *someData)
{
	return NO;
}
BOOL Transmitter::shouldTransmitDLL(NSFloat *someData)
{
	if (m_pCallingObject)
        return (*(Transmitter_MAP_CSW)dllStruct->function)(dllStruct->instance, someData, spatialCount, m_pCallingObject);
	else
        return (*(Transmitter_MAP)dllStruct->function)(dllStruct->instance, someData, spatialCount);
}
// Author: Wesley Curtis Lefebvre
// Name: ThresholdTransmitter.m

ThresholdTransmitter::ThresholdTransmitter()
{
	m_threshold = 1.0f;
	m_thresholdDecay = 1.0f;
	m_startingThreshold = 0.0f;
	m_fBeta = 0.7f;
	m_fInitialValue = 1.0f;
	m_sendAfter = 1;
	m_sendAfterCounter = 0;
	m_thresholdType = 0;
	m_lessThan = FALSE;
	m_bMultBy = TRUE;
	m_bAbs = TRUE;
	m_thresholdAccess = nil;
	m_pSmooth = nil;
	m_bSmoothingDataLocal = FALSE;
	m_nChannelNumber = 0;
}
ThresholdTransmitter::~ThresholdTransmitter()
{
	if (m_pSmooth && m_bSmoothingDataLocal)
		delete m_pSmooth;
}
BOOL ThresholdTransmitter::allLessThan(NSFloat *data, int length)
{
	register NSFloat *pos=data, *endPos=pos+length;
	while (pos<endPos)
		if (*pos++ > m_threshold)
			return NO;
        return YES;
}
BOOL ThresholdTransmitter::averageLessThan(NSFloat *data, int length)
{
	//      ASSERT(length);
	register NSFloat *pos=data, *endPos=pos+length, average=0.0f;
	while (pos<endPos)
		average += *pos++;
	return (average /= length) < m_threshold;
}
void ThresholdTransmitter::constructDLL(void) {
	if (dllStruct->constructor) {
		if (m_pCallingObject)
			dllStruct->instance = (*(ThresholdTransmitter_Constructor_CSW)dllStruct->constructor)(dllStruct->instance, rowCount, colCount, m_pCallingObject);
		else
			dllStruct->instance = (*(ThresholdTransmitter_Constructor)dllStruct->constructor)(dllStruct->instance, rowCount, colCount);
	}
}
void ThresholdTransmitter::networkReset()
{
	m_threshold = m_startingThreshold;
	SpatialAccess::m_bRefreshAccess = TRUE;
    if (m_thresholdAccess)
		m_thresholdAccess->reportSpatialAccess();
	SpatialAccess::m_bRefreshAccess = FALSE;
	setInitialValue(m_fInitialValue);
}
BOOL ThresholdTransmitter::oneLessThan(NSFloat *data, int length)
{
	register NSFloat *pos=data, *endPos=pos+length;
	while (pos<endPos)
		if (*pos++ < m_threshold)
			return YES;
        return NO;
}
BOOL ThresholdTransmitter::overThreshold(NSFloat *someData)
{
	switch (m_thresholdType) {
	case 0:
		if (m_lessThan)
			return allLessThan(someData,spatialCount);
		return !oneLessThan(someData,spatialCount);
		break;
	case 1:
		if (m_lessThan)
			return oneLessThan(someData,spatialCount);
		return !allLessThan(someData,spatialCount);
		break;
	case 2:
		if (m_lessThan)
			return averageLessThan(someData,spatialCount);
		return !averageLessThan(someData,spatialCount);
		break;
	case 3:
		if (m_lessThan)
			return oneLessThan(someData+m_nChannelNumber,1);
		return !allLessThan(someData+m_nChannelNumber,1);
		break;
	}
	return NO;
}
void ThresholdTransmitter::sendTransmitterActions()
{
	Transmitter::sendTransmitterActions();
	if (m_bMultBy)
		m_threshold *= m_thresholdDecay;
	else
		m_threshold += m_thresholdDecay;
	if (m_thresholdAccess)
		m_thresholdAccess->reportSpatialAccess();
}
void ThresholdTransmitter::setChannelNumber(int anInt)
{
	m_nChannelNumber = anInt;
}
void ThresholdTransmitter::setData(NSFloat *aFloat, int rowInt, int colInt)
{
	SpatialData::setData(aFloat, rowInt, colInt);
	if (m_pSmooth)
		setSmoothingData(m_pSmooth);
}
void ThresholdTransmitter::setSpatialDimension(int rowInt, int colInt) {
	SpatialData::setSpatialDimension(rowInt, colInt);
	if (m_pSmooth)
		setSmoothingData(m_pSmooth);
}
void ThresholdTransmitter::setSendAfter(int anInt)
{
	m_sendAfter = anInt;
	if (m_sendAfter < 0)
		m_sendAfter = 0;
}
void ThresholdTransmitter::setLessThan(BOOL aFlag)
{
	m_lessThan = aFlag;
}
void ThresholdTransmitter::setMultBy(BOOL aFlag)
{
	m_bMultBy = aFlag;
}
void ThresholdTransmitter::setThreshold(NSFloat aFloat)
{
	m_startingThreshold = m_threshold = aFloat;
	if (m_thresholdAccess)
		m_thresholdAccess->reportSpatialAccess();
}

void ThresholdTransmitter::setThresholdDecay(NSFloat aFloat)
{
	m_thresholdDecay = aFloat;
}
void ThresholdTransmitter::setThresholdType(int anInt)
{
	m_thresholdType = anInt;
}
void ThresholdTransmitter::setSmoothingData(SpatialData *anId)
{
	if (!anId || (anId != m_pSmooth)) {
		if (m_pSmooth && m_bSmoothingDataLocal)
			delete m_pSmooth;
		if (anId) {
			m_pSmooth = anId;
			m_bSmoothingDataLocal = FALSE;
		}
		else {
			m_pSmooth = new SpatialData;
			m_bSmoothingDataLocal = TRUE;
		}
	}
    m_pSmooth->setSpatialDimension(rowCount,colCount);
    m_pSmooth->assignScalar(m_fInitialValue);
}
void ThresholdTransmitter::setThresholdAccess(SpatialAccess *anId)
{
	if (m_thresholdAccess = anId)
		m_thresholdAccess->setData(&m_threshold,1);
}
void ThresholdTransmitter::setInitialValue(NSFloat aFloat)
{
	m_fInitialValue = aFloat;
	if (m_pSmooth)
		m_pSmooth->assignScalar(m_fInitialValue);
}
BOOL ThresholdTransmitter::shouldTransmit(NSFloat *someData)
{
	return overThreshold(weightedAverage(someData));
}
BOOL ThresholdTransmitter::shouldTransmitDLL(NSFloat *someData)
{
	if (m_pCallingObject)
        return (*(ThresholdTransmitter_MAP_CSW)dllStruct->function)(dllStruct->instance, weightedAverage(someData), rowCount, colCount, m_threshold, m_lessThan, m_thresholdType, m_nChannelNumber, m_pCallingObject);
	else
        return (*(ThresholdTransmitter_MAP)dllStruct->function)(dllStruct->instance, weightedAverage(someData), rowCount, colCount, m_threshold, m_lessThan, m_thresholdType, m_nChannelNumber);
}
NSFloat *ThresholdTransmitter::weightedAverage(NSFloat *aFloat)
{
	register NSFloat *pos=&(m_pSmooth->S(0)), *endPos=pos+m_pSmooth->S(), *dataPos=aFloat;
	while (pos<endPos)
		*pos++ = (1.0f-m_fBeta) * absolute(*dataPos++) + m_fBeta * *pos;
	m_pSmooth->reportSpatialAccess();
	return &(m_pSmooth->S(0));
}
// Author: Wesley Curtis Lefebvre
// Name: DeltaTransmitter.m

DeltaTransmitter::DeltaTransmitter()
{
	m_lastData = nil;
	m_deltaData = nil;
}
DeltaTransmitter::~DeltaTransmitter()
{
	if (m_lastData)
		free((char*)m_lastData);
	if (m_deltaData)
		free((char*)m_deltaData);
}

DeltaTransmitter *DeltaTransmitter::allocateLastData()
{
	if (m_lastData)
		free((char*)m_lastData);
	m_lastData = (NSFloat *)calloc(spatialCount, sizeof(NSFloat));
	if (m_deltaData)
		free((char*)m_deltaData);
	m_deltaData = (NSFloat *)calloc(spatialCount, sizeof(NSFloat));
	return this;
}
NSFloat *DeltaTransmitter::weightedAverage(NSFloat *someData)
{
	register NSFloat *pos=someData, *endPos=pos+spatialCount, *deltaPos=m_deltaData, *lastPos=m_lastData;
	while (pos<endPos) {
		*deltaPos++ = *pos - *lastPos;
		*lastPos++ = *pos++;
	}
	return ThresholdTransmitter::weightedAverage(m_deltaData);
}
void DeltaTransmitter::setData(NSFloat *aFloat, int rowInt, int colInt)
{
	ThresholdTransmitter::setData(aFloat, rowInt, colInt);
	allocateLastData();
}
void DeltaTransmitter::setSpatialDimension(int rowInt, int colInt) {
	ThresholdTransmitter::setSpatialDimension(rowInt, colInt);
	allocateLastData();
}

//      Auther: Wesley Curtis Lefebvre

Scheduler::Scheduler()
{
	m_iStart = 1;
	m_iStop = 1000;
	m_fBeta = 0.9f;
	m_fMin = 0.0f;
	m_fMax = 1.0f;
	dllStruct = NULL;
	m_pCallingObject = NULL;
}
void Scheduler::applyDLL()
{
	if (m_pCallingObject)
        (*(Scheduler_MAP_CSW)dllStruct->function)(dllStruct->instance, (NSFloat *)data, S(), (NSFloat)m_fBeta, m_pCallingObject);
	else
        (*(Scheduler_MAP)dllStruct->function)(dllStruct->instance, (NSFloat *)data, S(), (NSFloat)m_fBeta);
    register NSFloat *pos=data, *endPos=pos+S();
    while (pos<endPos)
		constrain(*pos++);
}
void Scheduler::attachTo(SpatialAccess *otherData) {
	SpatialData::attachTo(otherData);
	initializeDLL();
}
const char *Scheduler::dllFunctionName()
{
	return "performScheduler";
}
void Scheduler::epochEnded(int epoch)
{
	if (m_bAccessEnabled && (epoch+1 >= m_iStart) && (epoch+1 <= m_iStop)) {
		if (dllStruct)
			applyDLL();
		else
			applySchedule();
		if (stackedAccess)
			stackedAccess->reportSpatialAccess();
	}
}
void Scheduler::constructDLL(void) {
	if (dllStruct->constructor) {
		if (m_pCallingObject)
			dllStruct->instance = (*(Scheduler_Constructor_CSW)dllStruct->constructor)(dllStruct->instance, S(), m_pCallingObject);
		else
			dllStruct->instance = (*(Scheduler_Constructor)dllStruct->constructor)(dllStruct->instance, S());
	}
}
void Scheduler::destroyDLL(DLLData *instance) {
	if (dllStruct->destructor) {
		if (m_pCallingObject)
			(*(DLL_Destructor_CSW)dllStruct->destructor)(instance, m_pCallingObject);
		else
			(*(DLL_Destructor)dllStruct->destructor)(instance);
	}
}
void Scheduler::initializeDLL(void) {
	if (dllStruct) {
		DLLData *oldInstance = dllStruct->instance;
		constructDLL();
		destroyDLL(oldInstance);
	}
}
void Scheduler::setData(NSFloat *aFloat, int rowInt, int colInt) {
	SpatialData::setData(aFloat, rowInt, colInt);
	initializeDLL();
}
void Scheduler::setDLL(void *function, void *constructor, void *destructor, void *instance, void *callingObject)
{
	dllStruct = new DLLStruct;
	dllStruct->function = function;
	dllStruct->constructor = constructor;
	dllStruct->destructor = destructor;
	dllStruct->instance = (DLLData*)instance;
	m_pCallingObject = callingObject;
	initializeDLL();
}
void Scheduler::setSpatialDimension(int rowInt, int colInt) {
	SpatialData::setSpatialDimension(rowInt, colInt);
	initializeDLL();
}

//      Auther: Wesley Curtis Lefebvre

ExpScheduler::ExpScheduler()
{
}
void ExpScheduler::applySchedule()
{
	register NSFloat *pos=data, *endPos=pos+S();
	while (pos<endPos) {
		*pos *= m_fBeta;
		constrain(*pos);
		pos++;
	}
}
//      Auther: Wesley Curtis Lefebvre


LogScheduler::LogScheduler()
{
}
void LogScheduler::applySchedule()
{
	register NSFloat *pos=data, *endPos=pos+S();
	while (pos<endPos) {
		if (*pos > 0.0f) {
			*pos -= m_fBeta / *pos;
			constrain(*pos);
		}
		pos++;
	}
}

//      Auther: Wesley Curtis Lefebvre

LinearScheduler::LinearScheduler()
{
}
void LinearScheduler::applySchedule()
{
	register NSFloat *pos=data, *endPos=pos+S();
	while (pos<endPos) {
		*pos += m_fBeta;
		constrain(*pos);
		pos++;
	}
}
